;************************************************************************
; Routine : TDC.EXE 	V1.00 Copyright 2001 GK
; Type of file: 32 bit Windows EXE - Windows 95 or later
; Purpose: "Time Delay Caller" Counts down for a specified time, then
;    dials a specified number and plays a specified (voice) message.
;    Requires a voice-mode modem.
; Usage: Run from Windows
; Revisions:
;	V1.00 11/11/2001  - First working version
; Notes:
; - Source files required (to produce TDC.EXE)
;   TDC.ASM - This file
;   TDC.RC - Contains resources (menus and dialog boxes)
;   PHONE.ICO - The bitmap of the icon used by this program
; - Files required for help (to produce TDC.HLP)
;   TDC.RTF - The main help file
;   ASH.BMP - A bitmap used by the help system
;   TDC.HPJ - Defines the help system (files, exports, etc.)
; - This is intended to be assembled using MASM 6.11
;   The following is an example of a batch file that will compile and
;   link all required files, including the help file:
; ---------------------
; rem ** Compile **
; ml /c /coff /nologo /Fotdc.obj tdc.asm
;
; rem ** Create Resource File [must use 32 bit resource compiler] **
; rc tdc.rc
;
; rem ** Link [must use 32 bit linker] **
; set lib=...
; link tdc tdc.res kernel32.lib user32.lib gdi32.lib comdlg32.lib 
;                  tapi32.lib winmm.lib ole32.lib /entry:start 
;                  /subsystem:Windows /nologo
;
; rem ** Compile Help File **
; hc31 tdc
; ---------------------
;
; Possible Future:
; - Install/Uninstall
; - Need to handle program exit while delivering message.
; - Volume of message may need to be adjustable (waveOutSetVolume ?)
; - Playback function needs "via modem speaker" option.
; - Possible option to convert from different WAV formats
; - Multiple numbers, messages
; - some data should be in local variables
; - Startup check to prevent multiple instances doesn't work if TAPI
; initialization locks up.  Would be good to have a smarter startup 
; check although in this case the bigger problem is TAPI locking up.  
; Could possibly put TAPI init in a different thread and kill it if 
; it locks up.
;
;************************************************************************
.386					;
.model flat				;

	; ***** Constants *****
window_width 		= 400 		;
window_height 		= 105		; Height of main window
;;debug_height 		= 256		; Height of debug window
button_x 		= 0		; Top left corner of first (start) button
button_y 		= 0		;
button_height 		= 25		; Size of buttons
button_width 		= 60		;
MaxAlloc		= 65536		; Maximum memory to allocate for variable-sized items

	; ***** Windows Constants *****

ANSI_CHARSET 		= 0
BM_SETCHECK 		= 1
BN_CLICKED 		= 0
BS_AUTOCHECKBOX		= 3
BS_AUTORADIOBUTTON	= 9
BS_3STATE		= 5
BS_CHECKBOX		= 2
BS_DEFPUSHBUTTON	= 1
BS_PUSHLIKE		= 1000h
BS_RADIOBUTTON		= 4
BST_CHECKED 		= 1
CALLBACK_NULL		= 1
CALLBACK_FUNCTION	= 030000h
CALLBACK_EVENT		= 050000h
CALLBACK_WINDOW		= 010000h
CB_ADDSTRING		= 0143h
CB_GETCURSEL		= 0147h
CB_GETITEMDATA		= 0150h
CB_SELECTSTRING         = 014dh
CB_SETITEMDATA		= 0151h
CB_SETCURSEL		= 014eh
CLIP_DEFAULT_PRECIS 	= 0
CS_VREDRAW		= 0001h
CS_HREDRAW		= 0002h
CS_DBLCLKS		= 0008h
CS_OWNDC		= 0020h
CW_USEDEFAULT	        = 80000000h
COINIT_APARTMENTTHREADED = 2
CLSCTX_LOCAL_SERVER    	= 4
CLSCTX_ALL 		= 7
COLOR_WINDOW		= 5
COLOR_WINDOWTEXT	= 8
DEFAULT_QUALITY 	= 0
DEFAULT_PITCH 		= 0
DT_CENTER		= 1
DT_NOCLIP		= 0100h
DT_SINGLELINE		= 020h
DT_VCENTER		= 4
DT_WORDBREAK		= 010h
DTR_CONTROL_ENABLE 	= 1
ERROR_IO_PENDING	= 997
FILE_FLAG_OVERLAPPED	= 040000000h
FILE_ATTRIBUTE_NORMAL	= 080h
FILE_END		= 2
GENERIC_READ		= 080000000h
GENERIC_WRITE		= 040000000h
HELP_CONTEXT		= 1
HELP_QUIT		= 2
HELP_INDEX    		= 3
HKEY_CURRENT_USER 	= 080000001h
HKEY_LOCAL_MACHINE 	= 080000002h
IDC_ARROW		= 32512
IDC_WAIT 		= 32514
IDOK                	= 1
IDCANCEL            	= 2
IDABORT             	= 3
IDRETRY             	= 4
IDIGNORE            	= 5
IDYES               	= 6
IDNO                	= 7
INVALID_HANDLE_VALUE	= -1
KEY_QUERY_VALUE 	= 1
KEY_SET_VALUE		= 2
KEY_CREATE_SUB_KEY 	= 4
KEY_ENUMERATE_SUB_KEYS  = 8
KEY_NOTIFY 		= 10h
KEY_CREATE_LINK 	= 20h
KEY_ALL_ACCESS 		= ((STANDARD_RIGHTS_ALL Or KEY_QUERY_VALUE Or KEY_SET_VALUE Or KEY_CREATE_SUB_KEY Or KEY_ENUMERATE_SUB_KEYS Or KEY_NOTIFY Or KEY_CREATE_LINK) And (Not SYNCHRONIZE))
LB_ADDSTRING		= 0180h
LBS_MULTICOLUMN 	= 0200h
LBS_NOINTEGRALHEIGHT    = 0100h
LINEADDRESSMODE_ADDRESSID = 1
LINEBEARERMODE_VOICE	= 1
LINECALLPARAMFLAGS_IDLE = 2
LINECALLSTATE_BUSY	= 040h
LINECALLSTATE_CONNECTED	= 0100h
LINECALLSTATE_RINGBACK	= 020h
LINECALLSTATE_DISCONNECTED	= 04000h
LINECALLSTATE_DIALTONE	= 08h
LINECALLSTATE_DIALLING	= 010h
LINECALLSTATE_IDLE	= 1
LINECALLSTATE_PROCEEDING	= 0200h
LINEERR_REINIT 		= 080000052h
LINEINITIALIZEEXOPTION_USECOMPLETIONPORT = 3
LINEINITIALIZEEXOPTION_USEEVENT = 2
LINEINITIALIZEEXOPTION_USEHIDDENWINDOW = 1
LINEMEDIAMODE_AUTOMATEDVOICE 	= 8
LINEMEDIAMODE_DATAMODEM = 010h
LINECALLPRIVILEGE_NONE	= 1
LINECALLSELECT_CALL	= 4
LINECALLSELECT_LINE 	= 1
LINE_CALLSTATE		= 2
LINE_REPLY		= 12
MAX_PATH   		= 260		; Maximum length of path string
MB_OK                   = 00h
MB_OKCANCEL             = 01h
MB_ABORTRETRYIGNORE     = 02h
MB_YESNOCANCEL          = 03h
MB_YESNO                = 04h
MB_RETRYCANCEL          = 05h
MB_ICONHAND             = 0010h
MB_ICONQUESTION         = 0020h
MB_ICONEXCLAMATION      = 0030h
MB_ICONASTERISK         = 0040h
MB_HELP                 = 04000h
MB_SETFOREGROUND 	= 010000h
MF_BYCOMMAND		= 0
MF_CHECKED		= 8
MF_ENABLED		= 0000h
MF_GRAYED		= 0001h
MF_DISABLED		= 0002h
MMIO_ALLOCBUF		= 010000h
MMIO_CREATE		= 01000h
MMIO_CREATERIFF 	= 020h
MMIO_DELETE		= 0200h
MMIO_EXIST      	= 04000h
MMIO_FINDCHUNK          = 010h
MMIO_FINDRIFF           = 020h
MMIO_READ		= 0
MMIO_WRITE		= 1
MMIO_READWRITE		= 2
MM_WIM_OPEN		= 03BEh
MM_WIM_CLOSE		= 03BFh
MM_WIM_DATA		= 03C0h
OFN_READONLY            = 00000001h
OFN_OVERWRITEPROMPT     = 00000002h
OFN_HIDEREADONLY        = 00000004h
OFN_NOCHANGEDIR         = 00000008h
OFN_SHOWHELP            = 00000010h
OFN_ENABLEHOOK          = 00000020h
OFN_ENABLETEMPLATE      = 00000040h
OFN_ENABLETEMPLATEHANDLE= 00000080h
OFN_NOVALIDATE          = 00000100h
OFN_ALLOWMULTISELECT    = 00000200h
OFN_EXTENSIONDIFFERENT  = 00000400h
OFN_PATHMUSTEXIST       = 00000800h
OFN_FILEMUSTEXIST       = 00001000h
OFN_CREATEPROMPT        = 00002000h
OFN_SHAREAWARE          = 00004000h
OFN_NOREADONLYRETURN    = 00008000h
OFN_NOTESTFILECREATE    = 00010000h
OPEN_ALWAYS 		= 4	; Actually means: Open if file exists, Create if it doesn't
OPEN_EXISTING		= 3
OUT_DEFAULT_PRECIS 	= 0
;;PHONEPRIVILEGE_OWNER	= 2
PBM_SETPOS		= 0402h
REG_OPTION_NON_VOLATILE = 0
REG_SZ 			= 1
SM_CYCAPTION            = 4
SM_CYMENU               = 15
SND_FILENAME		= 020000h
SPI_GETWORKAREA		= 48
STANDARD_RIGHTS_ALL 	= 1F0000h
STATUS_PENDING		= 0103h
SWP_NOMOVE          	= 2
SWP_NOZORDER        	= 4
SYNCHRONIZE		= 100000h
VER_PLATFORM_WIN32s     = 0
VER_PLATFORM_WIN32_WINDOWS = 1
VER_PLATFORM_WIN32_NT   = 2
WAVE_FORMAT_QUERY	= 1
WHDR_BEGINLOOP		= 4
WHDR_ENDLOOP		= 8
WAVE_MAPPER 		= -1
WM_INITDIALOG		= 0110h
WM_CREATE		= 0001h
WM_DESTROY		= 0002h
WM_PAINT		= 000Fh
WM_TIMECHANGE		= 001Eh
WM_CLOSE		= 0010h
WM_COMMAND		= 0111h
WM_TIMER		= 0113h
WOM_DONE 		= 03bdh
WS_OVERLAPPED       	= 0
WS_POPUP            	= 080000000h
WS_CHILD            	= 040000000h
WS_MINIMIZE         	= 020000000h
WS_VISIBLE          	= 010000000h
WS_DISABLED         	= 008000000h
WS_CLIPSIBLINGS     	= 004000000h
WS_CLIPCHILDREN     	= 002000000h
WS_MAXIMIZE         	= 001000000h
WS_CAPTION          	= 000C00000h
WS_BORDER           	= 000800000h
WS_DLGFRAME         	= 000400000h
WS_HSCROLL          	= 000100000h
WS_SYSMENU          	= 000080000h
WS_THICKFRAME       	= 000040000h
WS_GROUP            	= 000020000h
WS_TABSTOP          	= 000010000h
WS_MINIMIZEBOX      	= 000020000h
WS_MAXIMIZEBOX      	= 000010000h
WS_VSCROLL          	= 000200000h

	; ***** Include Files *****

; none

	; ***** External Functions *****

	; ADVAPI32
extrn _RegCloseKey@4:near
extrn _RegCreateKeyExA@36:near
extrn _RegDeleteValueA@8:near
extrn _RegSetValueExA@24:near

	; COMDLG32
extrn _GetOpenFileNameA@4:near
extrn _GetSaveFileNameA@4:near

	; GDI32
extrn _CreateFontIndirectA@4:near
extrn _DeleteObject@4:near
extrn _SelectObject@8:near
extrn _SetBkColor@8:near
;; extrn _SetMapMode@8:near
extrn _SetTextColor@8:near

	; KERNEL32
;; extrn _CancelIo@4:near
;; extrn _ClearCommError@12:near
extrn _CloseHandle@4:near
;; extrn _CreateEventA@16:near
extrn _CreateFileA@28:near
extrn _CreateProcessA@40:near
extrn _CreateThread@24:near
;; extrn _DeleteFileA@4:near
;; extrn _EscapeCommFunction@8:near
extrn _ExitProcess@4:near
;; extrn _FileTimeToSystemTime@8:near
;; extrn _GetCommState@8:near
;; extrn _GetCommTimeouts@8:near
extrn _GetCommandLineA@0:near
extrn _GetCurrentDirectoryA@8:near
extrn _GetFileAttributesA@4:near
;; extrn _GetLastError@0:near
extrn _GetLocalTime@4:near
extrn _GetModuleFileNameA@12:near
extrn _GetModuleHandleA@4:near
;; extrn _GetOverlappedResult@16:near
extrn _GetPrivateProfileIntA@16:near
extrn _GetPrivateProfileStringA@24:near
extrn _GetProcessHeap@0:near
extrn _GetSystemTime@4:near
extrn _GetVersionExA@4:near
extrn _HeapAlloc@12:near
extrn _HeapFree@12:near
extrn _MultiByteToWideChar@24:near
;; extrn _ReadFile@20:near
;; extrn _ResetEvent@4:near
extrn _SetFileAttributesA@8:near
extrn _SetFilePointer@16:near
extrn _Sleep@4:near
extrn _SystemTimeToFileTime@8:near
;; extrn _WaitForSingleObject@8:near
extrn _WriteFile@20:near
extrn _WritePrivateProfileStringA@16:near

	; OLE32
extrn _CoCreateInstance@20:near
extrn _CoInitializeEx@8:near
extrn _CoUninitialize@0:near

	; TAPI32 (all TAPI 1.4+ except where stated)
extrn _lineClose@4:near			;
extrn _lineConfigDialog@12:near		;
;; extrn _lineDial@12:near		;
extrn _lineDrop@12:near			;
;; extrn _lineGetCallStatus@8:near	;
extrn _lineGetDevCapsA@20:near		;
extrn _lineGetID@24:near		;
;; extrn _lineGetMessage@12:near	; TAPI 2.0+
extrn _lineInitialize@20:near		; TAPI 1.4 only
;; extrn _lineInitializeExA@28:near	; TAPI 2.0+
extrn _lineMakeCall@20:near		;
;; extrn _lineMonitorMedia@8:near	;
extrn _lineNegotiateAPIVersion@24:near	;
extrn _lineOpen@36:near			;
;; extrn _lineSetStatusMessages@12:near	;
extrn _lineShutdown@4:near		;
;;extrn _phoneClose@4:near		;
;;extrn _phoneGetID@12:near		;
;;extrn _phoneInitialize@20:near	; TAPI 1.4 only
;;extrn _phoneOpen@28:near		;
;;extrn _phoneShutdown@4:near		;

	; USER32
extrn _BeginPaint@8:near
extrn _CheckDlgButton@12:near
extrn _CheckMenuItem@12:near
extrn _CheckRadioButton@16:near
extrn _CreateWindowExA@48:near
extrn _DialogBoxParamA@20:near
extrn _DispatchMessageA@4:near
extrn _DefWindowProcA@16:near
extrn _DestroyIcon@4:near
extrn _DestroyWindow@4:near
extrn _DrawTextA@20:near
extrn _EnableMenuItem@12:near
extrn _EnableWindow@8:near
extrn _EndDialog@8:near
extrn _EndPaint@8:near
extrn _FindWindowA@8:near
;;extrn _GetClientRect@8:near
;;extrn _GetDesktopWindow@0:near
extrn _GetDlgItem@8:near
extrn _GetDlgItemInt@16:near
extrn _GetDlgItemTextA@16:near
;;extrn _GetDlgItemTextW@16:near
extrn _GetMenu@4:near
;;extrn _GetMenuState@12:near
extrn _GetMessageA@16:near
;;extrn _GetParent@4:near
extrn _GetSysColor@4:near
extrn _GetSystemMetrics@4:near
extrn _GetWindowRect@8:near
extrn _InvalidateRect@12:near
extrn _IsDlgButtonChecked@8:near
extrn _KillTimer@8:near
extrn _LoadCursorA@8:near
extrn _LoadIconA@8:near
extrn _MapDialogRect@8:near
extrn _MessageBoxA@16:near
extrn _MoveWindow@24:near
extrn _PostQuitMessage@4:near
extrn _RegisterClassExA@4:near
extrn _RegisterWindowMessageA@4:near
extrn _SendDlgItemMessageA@20:near
extrn _SendDlgItemMessageW@20:near
extrn _SendMessageA@16:near
extrn _SetCursor@4:near
extrn _SetDlgItemTextA@12:near
extrn _SetDlgItemInt@16:near
extrn _SetForegroundWindow@4:near
extrn _SetTimer@16:near
extrn _SetWindowPos@28:near
extrn _SetWindowTextA@8:near
extrn _SystemParametersInfoA@16:near
extrn _WinHelpA@16:near

	; WINMM
extrn _mmioAscend@12:near		;
extrn _mmioClose@8:near			;
extrn _mmioCreateChunk@12:near		;
extrn _mmioDescend@16:near		;
extrn _mmioOpenA@12:near		;
extrn _mmioRead@12:near			;
extrn _mmioWrite@12:near		;
extrn _waveInAddBuffer@12:near		;
extrn _waveInClose@4:near		;
extrn _waveInGetPosition@12:near	;
extrn _waveInOpen@24:near		;
extrn _waveInPrepareHeader@12:near	;
extrn _waveInReset@4:near		;
extrn _waveInStart@4:near		;
extrn _waveInStop@4:near		;
extrn _waveInUnprepareHeader@12:near	;
extrn _waveOutClose@4:near		;
;;extrn _waveOutGetDevCapsA@12:near	;
extrn _waveOutGetPosition@12:near	;
;;extrn _waveOutGetVolume@8:near		;
extrn _waveOutOpen@24:near		;
extrn _waveOutPrepareHeader@12:near	;
extrn _waveOutReset@4:near		;
;;extrn _waveOutSetVolume@8:near		;
extrn _waveOutUnprepareHeader@12:near	;
extrn _waveOutWrite@12:near		;


PUBLIC		_start			; application startup address

.data		; ***** Data *****

		; ***** Constant Data *****
		; Note: Could use .const, but this creates an extra segment
szAppName	db "TDC",0
szAppTitle	db "Time Delay Caller",0

		; Standard Window Types
szListBox	db "LISTBOX",0
szButton	db "BUTTON",0
szProgressClassA db "msctls_progress32",0	; Progress Bar
;;szTrackbarClassA db "msctls_trackbar32",0	; Slider (Track Bar)

		; Pushbuttons on screen
szStartButton	db "Start",0
szStopButton	db "Stop",0
szResetButton	db "Reset",0

	;  Windows message name for common dialogue help:
szHelpMsg	db "commdlg_help",0	; HELPMSGSTR

szWaveOut	db "wave/out",0

		; Error messages
szNoTimers	db "Too many timers in use",0
szNoMemory	db "Failed to allocate memory",0
szNoCOM		db "Failed to initialise COM Library Interface",0
szNoTTS		db "Can't open text-to-speech engine",0
szNoWaveIn	db "Can't open wave input device",0
szNoWaveOut	db "Can't open wave output device",0
szNoVoices	db "Text-to-speech engine has no voices available",0
szNoText	db "Please enter some text to speak",0
;;szNoFileName	db "Please enter a file name to write to",0
szNothingRecorded db "Unable to write to file: Nothing was recorded",0
szNoIniFile	db "Unable to write to .INI file",0
szCantStart	db "Cannot start a second copy",0
szAttBad	db "Number of Times to Dial must be in the range 1 to 100",0
szRepBad	db "Number of times to Repeat Message must be in the range 1 to 100",0
szITimeBad	db "Countdown Delay Time is bad",0
szRTimeBad	db "Redial Delay Time is bad",0
szDaysBad	db "Days must be 0 to 100",0
szHoursBad	db "Hours must be 0 to 23",0
szMinutesBad	db "Minutes must be 0 to 59",0
szSecondsBad	db "Seconds must be 0 to 59",0
szFormatBad	db "Must be in the format DDD HH:MM:SS",0
szModemBad	db "A valid modem must be selected",0
szNoModem	db "No Modem Selected",0
szNoModems	db "There are no usable modems installed on this computer",13
		db "You will not be able to use this program until one is installed",0
szBadWAVFile	db "The format of the WAV file is not usable by this program",0
szCantCreate	db "Failed to create WAV File",0
szErrorCreating	db "Error creating WAV File",0

		; Queries
szCountInProg	db "Countdown in progress.  Exit Anyway ?",0
szTryConfig	db "Unable to start counting because this program is not configured.",13
		db "Do you want to configure this program ?",0

		; Info Messages
		; These messages appear on the "Synthesise" and "Record" dialogs
szDone		db "Done",0
szPlaying	db "Playing Message..",0
szWriting	db "Writing to file...",0
szRecording	db "Recording...",0

		; Debug messages
szProgStart	db "TDC Started",0
szStarted	db "Countdown Started",0
szStopped	db "Countdown Stopped",0
szReset		db "Countdown Reset",0
szInvalidTime	db "Invalid time in INI file",0
;;szCantOpen	db "Failed to open WAV File ",0
;;szBadFormat	db "Invalid WAV file format ",0
;;szCantAlloc	db "Failed to allocate memory for WAV File ",0
;;szCantRead	db "Failed to read WAV File ",0
;;szReadOK	db "WAV File read OK",0
;;szWrittenOK	db "WAV File written OK",0

szDialTone	db "Dial Tone",0
szDialling	db "Dialling",0
szBusy		db "Busy",0
szRing		db "Ringing",0
szConnected	db "Connected",0
szDisconnected	db "Disconnected",0

szFinishedSpeaking	db "Finished "
szSpeaking	db "Speaking Message",0


;;	; ## TEST
;;test1		db "*1*",0
;;test2		db "*2*",0
;;test3		db "*3*",0
;;test4		db "*4*",0
;;test5		db "*5*",0
;;test6		db "*6*",0
;;test7		db "*7*",0
;;test8		db "*8*",0
;;test9		db "*9*",0
;;test10		db "*10*",0
;;test11		db "*11*",0
;;test12		db "*12*",0
;;test13		db "*13*",0
;;test14		db "*14*",0
;;test15		db "*15*",0
;;test16		db "*16*",0
;;test17		db "*17*",0
;;test18		db "*18*",0
;;test19		db "*19*",0

	; INI file contents
szWindow	db "Window",0		; [Window]
szX		db "X",0		; X= <integer>
szY		db "Y",0		; Y= <integer>

szSettings	db "Settings",0		; [Settings]
szDebug		db "Debug",0		; Debug= <boolean>
szConfigured	db "Configured",0	; Configured= <boolean>
szAttempts	db "Attempts",0		; Attempts= <integer>
szAutoStart	db "AutoStart",0	; Autostart= <boolean>
szAutoStop	db "AutoStop",0		; Autostop= <boolean>
szModem		db "Modem",0		; Modem= <integer>
szInitialDelay	db "Initial Delay",0	; Initial Delay= <string D H:M:S>
szNumber	db "Number",0		; Number= <string>
szPlay		db "Play",0		; Play= <string>
szRepeats	db "Repeats",0		; Repeats= <integer>
szRetryDelay	db "Retry Delay",0	; Retry Delay= <string D H:M:S>
szVoice		db "Voice",0		; Voice= <integer>
		; 
szDir		db "Dir",0		; Dir= <string>

szRun		db "Run",0		; [Run] also used for [Settings] Run= <string>
szTimeHi	db "TimeHi",0		; TimeHo=
szTimeLo	db "TimeLo",0		; TimeLo=

szBufferLen	db "Buffer Length",0	; [Record] Buffer Length = <integer>

	; Dialog names
szAboutBox	db "AboutBox",0		; Also used for [AboutBox]
szConfigure	db "Configure",0	; Also used for [Configure]
szRecord	db "Record",0		; Also used for [Record]
szSynthesise	db "Synthesise",0	; Also used for [Synthesise]
szPlayBack	db "PlayBack",0		; Also used for [Playback]


szMSDOSSYS	db "C:\MSDOS.SYS",0	;
szZero		db "0",0		;
szOne		db "1",0
szOptions	db "Options",0		;  [Options] Autoscan
szAutoScan	db "Autoscan",0		;
szRunKey	db "Software\Microsoft\Windows\CurrentVersion\Run",0
szKeyClass 	db "1",0
szTimeDelayCaller	db "Time Delay Caller",0


	; Choices for file open
szBrowseVfiles	db "Select Voice File",0
vfile_filter	db "Wave Files (*.WAV)",0,"*."
szWAV		db "wav",0		; Note: Overlap here
		db "All Files (*.*)",0,"*.*",0,0
		db 26			; EOF symbol

szBrowseRfiles	db "Select Executable File",0
rfile_filter	db "Executable Files (*.EXE)",0,"*."
szExe		db "exe",0		; Note: Overlap here
		db "All Files (*.*)",0,"*.*",0,0
		db 26			; EOF symbol


	; WAVEFORMATEX Structure (for recording)
cSizeOfWaveFormatEx = 18		;
WaveFormat	dw 1			; wFormatTag (=WAVE_FORMAT_PCM)
		dw 1			; nChannels
		dd 8000			; nSamplesPerSec
		dd 16000		; nAvgBytesPerSec
		dw 2			; nBlockAlign
		dw 16			; wBitsPerSample
		dw 0			; cbSize


		; COM IDs for accessing SAPI (Speech API)

CLSID_TTSEnumerator	dd 0d67c0280h, 011cdc743h, 0aa00e580h, 0504b3e00h
CLSID_MMAudioDest	dd 0cb96b400h, 011cdc743h, 0aa00e580h, 0504b3e00h
IID_ITTSBufNotifySink	dd 0e4963d40h, 011cdc743h, 0aa00e580h, 0504b3e00h
IID_IAudioMMDevice 	dd 0b68ad320h, 011cdc743h, 0aa00e580h, 0504b3e00h
IID_ITTSEnumA		dd 005eb6c6dh, 011cddbabh, 0aa00cab3h, 04fba4700h
IID_ITTSFindA		dd 005eb6c6eh, 011cddbabh, 0aa00cab3h, 04fba4700h
CLSID_AudioDestFile	dd 0d4623720h, 011cfe4b9h, 0a000568dh, 07e4a03c9h
IID_IAudioFile		dd 0fd7c2320h, 011b93d6dh, 0d6fe00c0h, 0a9b1a3cbh
CLSID_AudioDestTel	dd 02ec5a8a6h, 011d0e65bh, 00008ac8fh, 02ae6e42bh
IID_IAudioTel		dd 02ec5a8a7h, 011d0e65bh, 00008ac8fh, 02ae6e42bh

IID_IUnknown		dd 0,          0,          0c0h,       046000000h

		; COM vtable for our notify interface for the TTS engine
pITTSBufNotifySink	dd offset ITTSBufNotifySink
ITTSBufNotifySink 	dd offset ITTSBufNotifySink_QueryInterface	; QueryInterface (REFIID, LPVOID FAR *)
			dd offset ITTSBufNotifySink_AddRef		; AddRef(void)
			dd offset ITTSBufNotifySink_AddRef		; Release(void)
			dd offset ITTSBufNotifySink_TextDataDone	; (TextDataDone)    (QWORD, DWORD)
			dd offset ITTSBufNotifySink_TextDataStarted	; (TextDataStarted) (QWORD)
			dd offset ITTSBufNotifySink_BookMark		; (BookMark)        (QWORD, DWORD)
			dd offset ITTSBufNotifySink_BookMark		; (WordPosition)   (THIS_ QWORD, DWORD) PURE

		; ***** Initialised Data *****

ALIGN 4		; DWORD align this data

		; ** WNDCLASSEX structure **
wcxSize         dd 48    		; cbSize, size of WNDCLASSEX
wcxStyle        dd CS_VREDRAW + CS_HREDRAW + CS_DBLCLKS + CS_OWNDC 	; style
wcxWndProc      dd WndProc	    	; lpfnWndProc
wcxClsExtra     dd 0    		; cbClsExtra
wcxWndExtra     dd 0    		; cbWndExtra
hInstance     	dd ?    		; hInstance
wcxIcon         dd ?    		; hIcon
wcxCursor       dd ?    		; hCursor
wcxBkgndBrush   dd COLOR_WINDOW+1      	; hbrBackground
wcxMenuName     dd szAppName    	; lpszMenuName
wcxClassName    dd szAppName    	; lpszClassName
wcxSmallIcon    dd ?    		; hIconSm

		; RECT structure of main window
main_rect	dd 0			; Left
		dd button_height	; Top
		dd window_width		; Right
		dd window_height	; Bottom
;;debug_top	dd window_height	; Bottom = Top of debug window

		; OPENFILENAME structure
ofn_struct	dd 76 			;DWORD  lStructSize;
hwndOwner	dd ?			;HWND   hwndOwner;
		dd 0			;HINSTANCE     hInstance;
lpstrFilter	dd ?			;LPCTSTR       lpstrFilter;
		dd 0			;LPTSTR        lpstrCustomFilter;
		dd 0			;DWORD         nMaxCustFilter;
		dd 0			;DWORD         nFilterIndex;
lpstrFile	dd ?			;LPTSTR        lpstrFile;
		dd MAX_PATH		;DWORD         nMaxFile;
		dd offset szFileTitle	;LPTSTR        lpstrFileTitle;
		dd 32			;DWORD         nMaxFileTitle;
lpstrInitialDir	dd 0			;LPCTSTR       lpstrInitialDir;
lpstrTitle	dd 0			;LPCTSTR       lpstrTitle;
OFNFlags	dd OFN_PATHMUSTEXIST+OFN_FILEMUSTEXIST+OFN_SHOWHELP
					;DWORD         Flags;
nFileOffset	dw 0			;WORD          nFileOffset;
nFileExtension	dw 0			;WORD          nFileExtension;
lpstrDefExt	dd ?			;LPCTSTR       lpstrDefExt;
		dd 0			;DWORD         lCustData;
		dd 0			;LPOFNHOOKPROC lpfnHook;
		dd 0			;LPCTSTR       lpTemplateName;

	; WAVEHDR Structure (for wave output)
WaveOutHdr    	dd ?			; lpData
WaveHdr_dwBufferLength	dd ?		; dwBufferLength
   	 	dd 0			; dwBytesRecorded
    		dd 0			; dwUser
WaveHdr_dwFlags dd 0			; dwFlags
WaveHdr_dwLoops	dd 0			; dwLoops
    		dd ?			; struct wavehdr_tag * lpNext
    		dd ?			; reserved

	; WAVEHDR Structure (for wave preview output)
WavePreviewHdr   dd ?			; lpData
WavePreviewHdr_dwBufferLength	dd ?	; dwBufferLength
   	 	dd 0			; dwBytesRecorded
    		dd 0			; dwUser
 		dd WHDR_BEGINLOOP + WHDR_ENDLOOP	; dwFlags
		dd 1			; dwLoops
    		dd 0			; struct wavehdr_tag * lpNext
    		dd 0			; reserved

	; WAVEHDR Structure (for recording)

WaveInHeaderRec dd ?			; lpData
SizeOfWaveInBufferRec dd 16384*60	; dwBufferLength ( ~60 second buffer)
dwBytesRecorded	dd 0			; dwBytesRecorded
		dd 0			; dwUser
		dd 0			; dwFlags
		dd 0			; dwLoops (not used)
		dd 0			; struct wavehdr_tag * lpNext
		dd 0			; reserved

	; OVERLAPPED structure (for writing to debug file)
Overlapped	dd 0			; Internal
		dd 0			; InternalHigh
		dd 0			; Offset
		dd 0			; OffsetHigh
		dd 0			; hEvent

TAPI_Retries	dd 3			; Number of times to retry TAPI Init
;;Timer_Retries	dd 3			; Number of times to retry Timer Allocation

dwVoiceSelected	dd 0			; Which voice was selected on the Synthesise Menu

	; Items on Heap (note: not a complete list)
;;lpFmtChunk	dd 0			; Pointer for Format data chunk
;;dwFmtSize	dd 0			; Size of Format data chunk
lpDataChunk	dd 0			; Pointer to Data chunk
dwDataSize	dd 0			; Size of Format data chunk
lpVoiceGUIDs	dd 0			; List of GUIDs for TTS Voices

; Font definition for big numbers that we display
					; LOGFONT structure
FontDef		dd 50			; lfHeight [Modified]
		dd 0			; lfWidth
		dd 0			; lfEscapement
		dd 0			; lfOrientation
		dd 900			; lfWeight (900=very heavy)
		db 0			; lfItalic
		db 0			; lfUnderline
		db 0			; lfStrikeOut
		db ANSI_CHARSET		; lfCharSet
		db OUT_DEFAULT_PRECIS	; lfOutPrecision
		db CLIP_DEFAULT_PRECIS	; lfClipPrecision
		db DEFAULT_QUALITY	; lfQuality
		db DEFAULT_PITCH	; lfPitchAndFamily
		db "arial"		; lfFaceName[LF_FACESIZE]
		db 27 dup (0)		;

szFileTitle	db 33 dup (0)		;


.data?		; ***** Uninitialised Data *****

main_height	dd ?			; Height of main window incl title bar
xcoord		dd ?			; Position of main window
ycoord		dd ?			;
xcoord_about	dd ?			; Position of About dialogue
ycoord_about	dd ?			;
xcoord_config	dd ?			; Position of Config dialogue
ycoord_config	dd ?			;
xcoord_record	dd ?			; Position of Record dialogue
ycoord_record	dd ?			;
xcoord_synth	dd ?			; Position of Synthesise dialogue
ycoord_synth	dd ?			;
xcoord_play	dd ?			; Position of Play Back dialogue
ycoord_play	dd ?			;


		; SYSTIME structure
wYear		dw ?			;
wMonth		dw ?			;
wDayOfWeek	dw ?			;
wDay		dw ?			;
wHour		dw ?			;
wMinute		dw ?			;
wSecond		dw ?			;
wMilliseconds	dw ?			;

		; FILETIME structure
dwLowDateTime	dd ?			;
dwHighDateTime 	dd ?			;
					; ** Message Structure **
msgHwnd    	dd ?    		; hwnd
msgMessage 	dd ?    		; message
msgWparam  	dd ?    		; wParam
msgLparam  	dd ?    		; lParam
msgTime    	dd ?    		; time, message posting time
msgPtX     	dd ?    		; pt.x, cursor position
msgPtY     	dd ?    		; pt.y, cursor position

		; Handles
hProcessHeap	dd ?			; Process Heap
hWnd		dd ?			; Main window
hWndProgressBar	dd ?			; Progress bar on Playback & Record dialogues
hDlg		dd ?			; Dialog boxes
hDlgPlay	dd ?			; "Playback" dialogue box (called from within Config dialogue)
hFile		dd ?			; File Handle
hMenu		dd ?			; Menu Handle
hBigFont	dd ?			; Big Font
;;hMedFont	dd ?			; Medium Font
;;hHourglass	dd ?			; Hourglass cursor
hLineApp	dd ?			; TAPI Handle for Lines
;;hPhoneApp	dd ?			; TAPI Handle for Phones
hLine		dd ?			; Line (modem)
;;hPhone		dd ?			; Phone (modem speaker)
hCall		dd ?			; Call in progress
hWavOutDev	dd ?			; WAV output device associated with call
hWavOutDevPlay	dd ?			; WAV output device associated with previewing
hWavInDevRec	dd ?			; WAV input device associated with recording
hButtonFinished	dd ?			; Window handles to dialogue buttons
hButtonRecord	dd ?			;
hButtonPreview	dd ?			;
hButtonStop	dd ?			;
hButtonWrite	dd ?			;
hButtonPlay	dd ?			; Button on Playback dialogue
hButtonStopPlaying	dd ?		; Button on Playback dialogue
hButtonFinishedPlaying	dd ?		; Button on Playback dialogue
hRegKey		dd ?			; Handle to Registery key for auto start

	; Parameters - these are the user-entered program parameters
	; which must be correct for the program to run
szVoiceFileName		db MAX_PATH dup (?)	; Name of voice file
szRunFileName		db MAX_PATH dup (?)	; Name of run file
szNumber2Dial		db 32 dup (?)	; Number to dial
ICountdownTime		dd ?		; Initial Time to count down 0-8,726,399
RCountdownTime		dd ?		; Retry Time to count down 0-8,726,399
Max_attempts		dd ?		; Maximum number of attempts allowed 1-100
dwRepeats		dd ?		; Number of times to repeat message 1-100
dwPermanentLineID 	dd ?		; (dwPermanentLineID) Device for modem
dwDeviceID		dd ?		; Device ID of modem

	; Temporary copies of parameters above for checking
	; PLEASE NOTE: This data may be block-copied over the parameters
	; above therefore do not change the order of anything.
szNewVoiceFileName	db MAX_PATH dup (?)	;
szNewRunFileName	db MAX_PATH dup (?)	;
szNewNumber2Dial	db 32 dup (?)	;
NewICountdownTime	dd ?		;
NewRCountdownTime	dd ?		;
NewMax_attempts		dd ?		;
NewRepeats		dd ?		;
dwNewPermanentLineID	dd ?		;
dwNewDeviceID		dd ?		;

dwBytesWritten		dd ?		; Bytes written to debug file

	; Program status control (i.e. what are we doing now ?) (0=No, 1=Yes)
bCountingDown	db ?			; Flag: Are we currently counting down ?
bAutoStart	db ?			; Flag: Is AutoStart option selected ?
bAutoStop	db ?			; Flag: Is AutoStop option selected ?
bInitialConfig	db ?			; Flag: Has program been configured at least once ?

bConfigValid	db ?			; Flag: Is there a valid configuration  ?
bWriteToFile	db ?			; Flag: Does TTS write to file (1) or audio device (0)
bPlaying	db ?			; Flag: Are we playing a message (to file or speaker)
;;bModemSpeaker	db ?			; Flag: Playing back via Modem Speaker (1) or Multimedia device (0)

bDebugToFile	db ?			; Flag: Are we writing debug information to file ?
tRecording	db ?			; Tristate Flag: 0=Idle, -1=Previewing, 1=Recording
bAbortDial	db ?			; Flag: Abort of dial required.
bDialStatus	db ?			; Byte to indicate status of call
	; 0   = not dialiing
	; 10* = Preparing to Dial
	; 20  = Waiting for LINE_REPLY (to dial request)
	; 30  = Waiting for LINECALLSTATE_CONNECTED
	; 40* = Preparing to speak Message
	; 70  = Waiting for WOM_DONE
	; 80* = Preparing to Hang Up
	; 90  = Waiting for second LINE_REPLY (to hangup request)
	;100* = Shutting down

		db 1 dup(?)		; Dummies to preserve alignment

Sec2Go		dd ?			; Seconds to go 0-8,726,399
Attempts2Go	dd ?			; Number of attempts remaining 0-100
dwRepeatsRemaining dd ?			; Number of message repeats remaining 1-100

help_message	dd ?			; Message number of Windows message for Help buttons
dwNumLineDevs	dd ?			; Number of TAPI Line Devices
;;dwNumPhoneDevs	dd ?			; Number of TAPI Phone Devices
nDevs		dd ?			; Number of modem devices found
DevIndex	dd ?			; Index into listbox of default modem device
;;dwInDevID	dd ?			; ID of WAV IN Device accociated with call
;;dwOutDevID	dd ?			; ID of WAV OUT Device accociated with call
idTAPIAssynch	dd ?			; Request identifier for current assynch TAPI command
idDialThread	dd ?			; Thread ID for Dial procedure
dwnVoices	dd ?			; Number of voices available with TTS engine

dwDisposition	dd ?			; Used for opening registry key

pITTSFind	dd ?			; Pointers to COM vtables for TTS functions
pITTSCentral	dd ?			;
gpIAudio	dd ?			;
gpIAudioTel	dd ?			;

szWaveDirName	db MAX_PATH dup (?)		; Directory for WAV files

szIniFileName	db MAX_PATH dup (?)		; Name of .INI file
szHelpFileName	db MAX_PATH dup (?)		; Name of .HLP file
szLogFileName	db MAX_PATH dup (?)		; Name of .LOG file

wszVoiceFile	dw MAX_PATH dup(?)		; Unicode ## local memory ##
SizeOfTextToSpeak	= 2048
sTextToSpeak	db SizeOfTextToSpeak dup (?)	; ## LOCAL MEMORY PLEASE ##
dwlTextToSpeak	dd ?			; Length of above

szMainText 	db 64 dup (?)		; Text to write on screen
szWindowCaption db 64 dup(?)		; Window caption
debug_buffer	db 256 dup (?)		; Debug messages
lTempBuffer = 256			; Length of temporary buffer
temp_buffer	db lTempBuffer+4 dup (?); Temporary use


.code			; ***** Code *****

;****************************************************************************
; Name: _start
; Purpose : Main routine called by Windows on startup.  Basically,
; this initialises various things, then runs the main message loop.
;****************************************************************************
_start PROC


	; ** Get hInstance **
	xor eax,eax			;
	pushd eax			;
	call _GetModuleHandleA@4	; Get hInstance of EXE file
	mov hInstance,eax		;

	; ** Get Icon **
	pushd offset szAppName		; Icon name = program name
	pushd eax			;
	call _LoadIconA@8		;
	mov wcxIcon,eax			;
	mov wcxSmallIcon,eax		;

	; ** Get Cursor **
	xor eax,eax			;
	pushd IDC_ARROW			;
	pushd eax			;
	call _LoadCursorA@8		;
	mov wcxCursor,eax		;

	; *** Get INI file settings ***
	; Note: The .INI file sits in the same directory as the .EXE file.
	; We don't want it cluttering up the \WINDOWS directory.
	;
	pushd 255			;
	pushd offset szIniFileName	;
	pushd hInstance			;
	call _GetModuleFileNameA@12	;
	; eax contains length of string (no null terminator)
	sub eax,3			; Discard EXE
	push eax			; Save length (minus terminator)

	;** Convert EXE file name to INI, HLP and LOG file names **

	mov esi,offset szIniFileName	;
	mov edi,offset szHelpFileName	; Also construct help file name
	mov ecx,eax			;
	rep movsb			; Copy character sto HLP file name
	mov eax,"INI"			;
	mov dword ptr [esi],eax		; Overwrite "EXE" with "INI",0
	mov eax,"PLH"			; Overwrite "EXE" with "HLP",0
	stosd				;

	pop ecx				;
	mov esi,offset szIniFileName	;
	mov edi,offset szLogFileName	; Also construct log file name
	rep movsb			; Copy character to LOG file name
	mov eax,"GOL"			; Overwrite "INI" with "LOG",0
	stosd				;

	; Check to see if already running
	; See Q124134 - This previous instance detection code has a potential
	; race condition.
	xor eax,eax			;
	pushd eax			; lpWindowName
	pushd offset szAppName		; lpClassName
	call _FindWindowA@8		; Is the program already running ?
	or eax,eax			;
	jz no_prev_instance		; If not, jump

	pushd eax			; Save handle of previous instance

	; We don't want to allow a second copy to start
	xor eax,eax
	pushd MB_ICONEXCLAMATION	;
	pushd offset szAppTitle		;
	pushd offset szCantStart	;
	pushd eax			; hWnd =0
	call _MessageBoxA@16		;

	; Note: Handle of previous instance is on stack
	call _SetForegroundWindow@4	; Force previous window into foreground

	jmp already_running		;
no_prev_instance:			;

	; ** Register Window Class **
	pushd offset wcxSize		; WNDCLASSEX structure
	call _RegisterClassExA@4	;
	or eax,eax			; Was an error returned ?
	jz failed2register		; If so, jump

;;   	; ** Get handle of Hourglass Cursor **
;;	pushd IDC_WAIT			; The hourglass
;;	pushd  0			;
;;	call _LoadCursorA@8		;
;;	mov hHourglass,eax		;

	; ** Create Fonts **
	pushd offset FontDef		;
	call _CreateFontIndirectA@4	;
	mov hBigFont,eax		;

;;	mov eax,25			; Size of medium font
;;	mov [FontDef],eax		;
;;	pushd offset FontDef		;
;;	call _CreateFontIndirectA@4	;
;;	mov hMedFont,eax		;

	; *** What message number will common dialogue help buttons use ? ***
	pushd offset szHelpMsg
	call _RegisterWindowMessageA@4
	mov help_message,eax		;

	; ** Get handle to heap **
	call _GetProcessHeap@0		; Get handle of main heap
	mov hProcessHeap,eax		;

	; ** Initalize TAPI **
Retry_TAPI_Init:
	pushd offset dwNumLineDevs	; lpdwNumLineDevs
	pushd offset szAppTitle		; lpszAppName
	pushd offset lineCallbackFunc	; lpfnCallback
	pushd hInstance			; hInstance
	pushd offset hLineApp		; lphLineApp
	call _lineInitialize@20

	cmp eax,LINEERR_REINIT		; Should we retry ?
	jne Dont_Retry_TAPI		; Jump if not

	;* Wait Between Retries *
	pushd 500			; 500 mSec delay
	call _Sleep@4			;
	mov eax,TAPI_Retries		;
	dec eax				; Decrement retries
	cmp eax,0			;
	jbe TAPI_failed			; Give up if we run out of retries
	mov TAPI_Retries,eax		;
	jmp Retry_TAPI_Init		; Retry
Dont_Retry_TAPI:			;
	or eax,eax			; Check return code
	jnz TAPI_failed			; Jump if failed

	; Note: We loop using dwNumLineDevs and so it is probably a good idea
	; that this doesn't contain a gigantic number.
	cmp dwNumLineDevs, 1000		;
	jbe dwNumLineDevs_OK		;
	mov dwNumLineDevs, 1000		; Limit it to 1000
dwNumLineDevs_OK:				;

	; *** Read window X & Y coords from .INI file ***
	mov esi,offset szX		; First read X coord
get_y_coord:				;
        pushd offset szIniFileName    	;
	pushd 200			; could have CW_USEDEFAULT, but upsets clipping logic
	pushd esi			;
	pushd offset szWindow		;
	call _GetPrivateProfileIntA@16  ;
	cmp esi,offset szX		;
	jne got_both_coords		;
	mov xcoord,eax			; Save X coord
	mov esi,offset szY		; Then read Y coord
	jmp get_y_coord			;
got_both_coords:			;
	mov ycoord,eax			; Save Y coord

	; ** Calculate required window height **
	mov edi,window_height		;
	pushd SM_CYCAPTION		;
	call _GetSystemMetrics@4	; Title (" TDC [x]")
	add edi,eax			;
	pushd SM_CYMENU			;
	call _GetSystemMetrics@4	; Menu bar ("File Help")
	add edi,eax			; EDI = Height
	mov main_height,edi		; Save height

	; ** Ensure window will fit on screen **
	mov esi, offset temp_buffer	;
	xor eax,eax			;
	pushd eax			; fWinIni = 0
	pushd esi			; pvParam
	inc eax				;
	pushd eax			; uiParam = 1
	pushd SPI_GETWORKAREA		; uiAction
	call _SystemParametersInfoA@16	;

	; * Overlap on right ? *
	mov eax,xcoord			;
	add eax,window_width		; eax = window right
	cmp eax,[esi+8]			; Does it overlap on right ?
	jle no_r_olap			; If not, jump
	mov eax,[esi+8]			; If so, fix it
no_r_olap:				;
	sub eax,window_width		; eax = new "X"
	; ** Overlap on left **
	cmp eax,[esi]			; Does it overlap at left
	jge no_l_olap			; if not, jump
	mov eax,[esi]			; If so, fix it
no_l_olap:
	mov xcoord,eax			;

	; * Overlap at bottom *
	mov eax,ycoord			;
	add eax,edi			; eax = dialog box bottom
	cmp eax,[esi+12]		; Does it overlap at bottom ?
	jle no_b_olap			; If not, jump
	mov eax,[esi+12]		; If so, fix it
no_b_olap:				;
	sub eax,edi			; eax = new "Y"
	; ** Overlap at top **
	cmp eax,[esi+4]			; Does it overlap at top
	jge no_t_olap			; if not, jump
	mov eax,[esi+4]			; If so, fix it
no_t_olap:
	mov ycoord,eax			;

	; ** Create Window **
	xor eax,eax			;
    	pushd eax            		; lpParam
   	pushd hInstance   		; hInstance
   	pushd eax            		; menu hmenu
    	pushd eax            		; parent hwnd
    	pushd edi          		; height
    	pushd window_width          	; width
    	pushd ycoord          		; y
    	pushd xcoord         		; x
    	pushd WS_VISIBLE+WS_CAPTION+WS_SYSMENU+WS_MINIMIZEBOX	; style
    	pushd offset szAppTitle        	; Window text (caption)
    	pushd offset szAppName  		; Class name
    	pushd eax            		; extended style
	call _CreateWindowExA@48	;
	or eax,eax			; Problems ?
	jz window_createfailed		; If so, exit
	; Note: eax contains hWnd, which we could save if we needed to

	; ***** Message Loop *****
msg_loop:
	xor eax,eax			;
	pushd eax			; uMsgFilterMax
	pushd eax			; uMsgFilterMin
	pushd eax			; hWnd (filter), 0 = all windows
	pushd offset msgHwnd		; lpMsg
	call _GetMessageA@16		; Returns FALSE if WM_QUIT
	or eax,eax			;
	jz end_msg_loop			;
	pushd offset msgHwnd		;
	call _DispatchMessageA@4	;
	jmp msg_loop			;
end_msg_loop:				;
	; ***** End of Message Loop *****

	call FreeWAVMemory		; Free memory, if any, associated with WAV files

window_createfailed:			;
	; *** Shut down TAPI ***
	pushd hLineApp			; hLineApp
	call _lineShutdown@4		;
TAPI_failed:				;

	; **** Finished with fonts ****
;;	pushd hMedFont			;
;;	call _DeleteObject@4		;
	pushd hBigFont			;
	call _DeleteObject@4		;

failed2register:			;
already_running:			;
	; Remove custom icon
	pushd wcxIcon			;
	call _DestroyIcon@4		;

	; ***** Exit Program *****
	xor eax,eax			;
	pushd eax			; Error code 0, no error
	call _ExitProcess@4		;

	ret				; Not needed
_start ENDP


;****************************************************************************
; Name: WndProc
; Purpose : Standard message processing routine called by Windows.
; Parameters:
;	[esp+4] = hWnd
;	[esp+8] = iMessage
;	[esp+12] = wParam
;	[esp+16] = lParam
;****************************************************************************

WndProc PROC
	push ebx			;
	push esi			;
	push edi			;

	; ** Jump to appropriate location **
	mov eax,dword ptr [esp+8+12]	; iMessage
	cmp eax,WM_PAINT		;
	je wndproc_wm_paint		;
	cmp eax,WM_TIMER		;
	je wndproc_wm_timer		;
	cmp eax,WM_COMMAND		;
	je wndproc_wm_command		;
	cmp eax,WM_CLOSE		;
	je wndproc_wm_close		;
	cmp eax,WM_DESTROY		;
	je wndproc_wm_destroy		;
	cmp eax,WM_CREATE		;
	je wndproc_wm_create		;
	cmp eax,WM_TIMECHANGE		;
	je wndproc_wm_timechange	;
	cmp eax,WOM_DONE		;
	je wndproc_wm_done		;

wndproc_dont_process:
	; Don't process message		;
	pop edi				;
	pop esi				;
	pop ebx				;
	jmp _DefWindowProcA@16		;
end_wndproc:				;
	pop edi				;
	pop esi				;
	pop ebx				;
	xor eax,eax			; Normal exit
	ret 16				;

wndproc_wm_create:	;**** WM_CREATE ****
	; Note: This is called within _CreateWindowExA@48.  It occurs
	; after the window is created but before it becomes visible.

	mov ebx,dword ptr [esp+4+12]	; hWnd
	mov hWnd,ebx			; save it for later

	pushd ebx			; hWnd
	call _GetMenu@4			; Get handle of menu
	mov hMenu,eax			;

	; ** Create Buttons **
	; ** Start Button **
	xor eax,eax			;
	pushd eax			; lpParam
	pushd hInstance			; hInstance
   	pushd 12            		; hmenu/child window identifier
    	pushd ebx            		; parent hwnd
	pushd button_height		; hHeight
	pushd button_width		; hWidth
	pushd eax ;= button_y		; y
	pushd eax ;= button_x		; x
	pushd WS_VISIBLE + WS_CHILD + BS_AUTORADIOBUTTON + BS_PUSHLIKE+ WS_TABSTOP
	pushd offset szStartButton	;
	pushd offset szButton		;
	pushd eax			; Extended Style
	call _CreateWindowExA@48	;

	; ** Stop Button **
	xor eax,eax			;
	pushd eax			; lpParam
	pushd hInstance			; hInstance
   	pushd 13            		; hmenu/child window identifier
    	pushd ebx            		; parent hwnd
	pushd button_height		; hHeight
	pushd button_width		; hWidth
	pushd eax ;= button_y		; y
	pushd button_x +button_width	; x
	pushd WS_VISIBLE + WS_CHILD + BS_AUTORADIOBUTTON + BS_PUSHLIKE + WS_TABSTOP
	pushd offset szStopButton	;
	pushd offset szButton		;
	pushd eax			; Extended Style
	call _CreateWindowExA@48	;

	; ** Reset Button **
	xor eax,eax			;
	pushd eax			; lpParam
	pushd hInstance			; hInstance
   	pushd 14            		; hmenu/child window identifier
    	pushd ebx            		; parent hwnd
	pushd button_height		; hHeight
	pushd button_width		; hWidth
	pushd eax ;= button_y		; y
	pushd button_x +2*button_width	; x
	pushd WS_VISIBLE + WS_CHILD + WS_TABSTOP
	pushd offset szResetButton	;
	pushd offset szButton		;
	pushd eax			; Extended Style
	call _CreateWindowExA@48	;

	; *** Read window X & Y coords of About dialog from .INI file ***
	mov esi, offset szAboutBox	; ** About **
	call ReadDlgXY			;
	mov xcoord_about,eax		; Save X coord
	mov ycoord_about,ebx		; Save Y coord

	mov esi, offset szConfigure	; ** config **
	call ReadDlgXY			;
	mov xcoord_config,eax		; Save X coord
	mov ycoord_config,ebx		; Save Y coord

	mov esi, offset szRecord	; ** Record **
	call ReadDlgXY			;
	mov xcoord_record,eax		; Save X coord
	mov ycoord_record,ebx		; Save Y coord

	mov esi, offset szSynthesise	; ** Synthesise **
	call ReadDlgXY			;
	mov xcoord_synth,eax		; Save X coord
	mov ycoord_synth,ebx		; Save Y coord

	mov esi, offset szPlayBack	; ** Play Back **
	call ReadDlgXY			;
	mov xcoord_play,eax		; Save X coord
	mov ycoord_play,ebx		; Save Y coord

	; *** Was Autostart selected ? ***
	; (Default = No)
	xor eax,eax			;
        pushd offset szIniFileName    	;
	pushd eax			;
	pushd offset szAutoStart	;
	pushd offset szSettings	;
	call _GetPrivateProfileIntA@16  ; [Settings] Autostart
	or eax,eax			; Did user say "No" (=0) ?
	jz no_astart			; If so, jump
	mov al,1			; Yes=1
no_astart:				;
	mov bAutoStart,al		; Set a flag for later

	; *** Was Autostop selected ? ***
	; (Default = No)
	xor eax,eax			;
        pushd offset szIniFileName    	;
	pushd eax			;
	pushd offset szAutoStop		;
	pushd offset szSettings		;
	call _GetPrivateProfileIntA@16  ; [Settings] AutoStop
	or eax,eax			; Did user say "No" (=0) ?
	jz no_astop			; If so, jump
	mov al,1			; Yes=1
no_astop:				;
	mov bAutoStop,al		; Set a flag for later

	; *** Was debug selected ? ***
        pushd offset szIniFileName    	;
	pushd 1				; Default = Yes
	pushd offset szDebug		;
	pushd offset szSettings		;
	call _GetPrivateProfileIntA@16  ; [Settings] Debug
	or eax,eax			; Did user say "No" (=0) ?
	jz no_debug			; If so, jump
	mov al,1			;
no_debug:				;
	mov bDebugToFile,al		;

	; ** Debug Log that the program has started **
	push offset szProgStart		;
	call DebugMsg			;

	; *** Space to allocate for record buffer  ***
        pushd offset szIniFileName    	;
	pushd 60			;
	pushd offset szBufferLen	;
	pushd offset szRecord		;
	call _GetPrivateProfileIntA@16  ; [Record] Buffer Length (default=60 seconds)
	cmp eax,8			; Limit to a minimum of 8 seconds
	jg bufferlen_not_low		;
	mov eax,8			;
bufferlen_not_low:			;
	cmp eax,1800			; And a maximum to 1/2 hour
	jl bufferlen_not_high		;
	mov eax,1800			;
bufferlen_not_high:			;
	mov ecx,16000			; 16000 bytes/sec
	mul ecx				; Buffer length in bytes
	mov SizeOfWaveInBufferRec,eax	;

	; *** Voice selected for Synth  ***
	xor eax,eax			;
        pushd offset szIniFileName    	;
	pushd eax			;
	pushd offset szVoice	;
	pushd offset szSettings		;
	call _GetPrivateProfileIntA@16  ; [Settings] Voice (default = 0)
	mov dwVoiceSelected,eax		;

	; *** Default Directory to use  ***
	mov esi,offset szWaveDirName	;
	mov byte ptr [esi],0		; Initially null string
	pushd offset szIniFileName	;
	pushd MAX_PATH			;
	pushd esi			;
	pushd esi			;
	pushd offset szDir		;
	pushd offset szSettings		;
	call _GetPrivateProfileStringA@24; [Settings] Dir (default "")

	; *** Voice file to play ? ***
	mov esi,offset szVoiceFileName	;
	mov byte ptr [esi],0		; Initially null string
	pushd offset szIniFileName	;
	pushd MAX_PATH			;
	pushd esi			;
	pushd esi			;
	pushd offset szPlay		;
	pushd offset szSettings	;
	call _GetPrivateProfileStringA@24; [Settings] Play (default "")

	; *** File to run (if any) ? ***
	mov esi,offset szRunFileName	;
	mov byte ptr [esi],0		; Initially null string
	pushd offset szIniFileName	;
	pushd MAX_PATH			;
	pushd esi			;
	pushd esi			;
	pushd offset szRun		;
	pushd offset szSettings	;
	call _GetPrivateProfileStringA@24; [Settings] Run (default "")

	; *** Number to Dial (if any) ? ***
	mov esi,offset szNumber2Dial	;
	mov byte ptr [esi],0		; Initially null string
	pushd offset szIniFileName	;
	pushd 32			;
	pushd esi			;
	pushd esi			;
	pushd offset szNumber		;
	pushd offset szSettings	;
	call _GetPrivateProfileStringA@24; [Settings] Number (default "")

	; *** Number of attempts ? ***
        pushd offset szIniFileName    	;
	pushd 3				;
	pushd offset szAttempts	;
	pushd offset szSettings	;
	call _GetPrivateProfileIntA@16  ; [Settings] Attempts (default=3)
	mov NewMax_attempts,eax		;
	cmp eax,100			; Check within range
	jle attempts_unhi		; Jump if not too high
	mov eax,100			; Too high, so clip
attempts_unhi:				;
	cmp eax,0			;
	jg attempts_unlow		; Jump if not too low
	mov eax,1			; Too low, so clip
attempts_unlow:				;
	mov Max_attempts,eax		;

	; *** Number of repeats ? ***
        pushd offset szIniFileName    	;
	pushd 5				;
	pushd offset szRepeats		;
	pushd offset szSettings		;
	call _GetPrivateProfileIntA@16  ; [Settings] Repeats (default=5)
	mov NewRepeats,eax			;
	cmp eax,100			; Check within range
	jle repeats_unhi		; Jump if not too high
	mov eax,100			; Too high, so clip
repeats_unhi:				;
	cmp eax,0			;
	jg repeats_unlow		; Jump if not too low
	mov eax,1			; Too low, so clip
repeats_unlow:				;
	mov dwRepeats,eax		;

	; *** Line Device (Modem) ***
	xor eax,eax			;
        pushd offset szIniFileName    	;
	pushd eax			;
	pushd offset szModem		;
	pushd offset szSettings	;
	call _GetPrivateProfileIntA@16  ; [Settings] Modem (default=0)
	mov dwNewPermanentLineID,eax	;
	mov dwPermanentLineID,eax	;

	; *** Initial time delay ***
	mov esi,offset temp_buffer	;
	mov word ptr [esi],0		; Initially null string
	pushd offset szIniFileName	;
	pushd 32			;
	pushd esi			;
	pushd esi			;
	pushd offset szInitialDelay	;
	pushd offset szSettings		;
	call _GetPrivateProfileStringA@24

	call StoTime			; Convert d h:m:s to seconds in EAX
	mov NewICountdownTime, eax	;
	; If the value in the INI file was invalid, substitute the default
	jnc init_time_ok		; Jump if OK
	mov eax,7200			; Error, so default to 2 hrs
init_time_ok:				;
	mov ICountdownTime, eax		;

	; *** Retry Time Delay ***
	mov esi,offset temp_buffer	;
	mov word ptr [esi],0		; Initially null string
	pushd offset szIniFileName	;
	pushd 32			;
	pushd esi			;
	pushd esi			;
	pushd offset szRetryDelay	;
	pushd offset szSettings	;
	call _GetPrivateProfileStringA@24

	call StoTime			; Convert d h:m:s to seconds in EAX
	mov NewRCountdownTime, eax	;
	; If the value in the INI file was invalid, substitute the default
	jnc retry_time_ok		; Jump if OK
	mov eax,7200			; Error, so default to 2 hrs
retry_time_ok:				;
	mov RCountdownTime, eax		;

	; *** Check to see if the config data we loaded was OK ***
	call check_config		; Check config
	jc ini_config_bad		; Jump if bad
	mov bConfigValid,1		; Config is OK
	jmp ini_config_ok		;
ini_config_bad:				;
	mov bConfigValid,0		; Config is bad
ini_config_ok:

	; *** Was Program Configured ? ***
	; (Default = No)
	xor eax,eax			;
        pushd offset szIniFileName    	;
	pushd eax			; Default="No"
	pushd offset szConfigured	;
	pushd offset szSettings		;
	call _GetPrivateProfileIntA@16  ; [Settings] Configured
	or eax,eax			; Was this program unconfigured ?
	jz not_configed			; If so, jump
	mov al,1			; Yes=1
not_configed:				;
	mov bInitialConfig,al		; Set a flag for later
	; The configuration can only be valid if both the configuration
	; checked out OK (tested for earlier) and an initial configuration
	; was performed
	and bConfigValid,al		;

	; *** AutoStart of countdown ***
	; If we auto-restarted while counting, continue counting
	; otherwise, not.  

	; ** Command Line **
	call _GetCommandLineA@0		;
	; A command line of " -r" implies that the program was started 
	; automatically on restart of Windows
	mov edi,eax			;
	mov ecx,1024			; limit number of iterations
	xor al,al			;
	repne scasb			; Find null terminator
	mov eax,[edi-4]			;
	cmp eax,"r- "			; Was this an Auto Start ?
	je auto_restart			; Jump if so

	; *** Not counting down ****
	mov eax,ICountdownTime		; Reset countdown time
	mov Sec2Go,eax			;
	call StopCounting		; Not Counting
	jmp not_auto_restart		;

auto_restart:
	; **** Continue Counting ****
	call StartCounting		; Counting

	; ** Set number of attempts to go **
        pushd offset szIniFileName    	;
	pushd Max_attempts		; Default is full number of attempts
	pushd offset szAttempts	;
	pushd offset szRun	;
	call _GetPrivateProfileIntA@16  ; [Run] Attempts
	mov Attempts2Go,eax		;

	; ** Set countdown time **
	; Read target time from file

	xor eax,eax			;
	pushd offset szIniFileName	;
	pushd eax			;
	pushd offset szTimeLo		;
	pushd offset szRun		;
	call _GetPrivateProfileIntA@16  ; Read [Run] TimeLo
	mov esi,eax			;

	xor eax,eax			;
	pushd offset szIniFileName	;
	pushd eax			;
	pushd offset szTimeHi		;
	pushd offset szRun		;
	call _GetPrivateProfileIntA@16  ; Read [Run] TimeHi
	mov edi,eax			;

	; Get Current Time
	pushd offset wYear		; SYSTEMTIME structure
	call _GetSystemTime@4		; Get UTC

	; Convert time to FILETIME (64 bit time & date in units of 100nSec) format
	pushd offset dwLowDateTime	; FILETIME structure
	pushd offset wYear		; SYSTEMTIME structure
	call _SystemTimeToFileTime@8	;

	; Note:
	; Target time is in edi::esi
	; Current time is in dwHighDateTime::dwLowDateTime

	sub esi,dwLowDateTime
	sbb edi,dwHighDateTime

	; edi::esi now contains our new countdown (in units of 100nSec)
	; There are now 3 possibilities:
	; 1) We are past the countdown already,
	; 2) Something is badly wrong and our count is invalid (too high)
	; 3) The count is OK
	; So we check for these...

	jc countdown_finished		; Result is negative - we have passed time
	cmp edi,20318			; Is time delay too long (>8,726,399 seconds)
	ja countdown_invalid		; If so, jump

	; Get countdown in seconds
	mov ebx,10000000		;
	mov eax,esi			;
	mov edx,edi			;
	div ebx				; Note: ebx < 10000000 so won't overflow
	; eax contains a valid coundown in seconds
	jmp countdown_valid		;
countdown_finished:			;
	; We are past the countdown and should transmit immediately,
	; however, a few extra seconds won't matter much, so we count
	; down from 10 to give the rest of the system time a bit of time
	; to start up.
	mov eax,10			;
	jmp countdown_valid		;
countdown_invalid:			;
	; The time is bad *but* the INI file says we were counting down, so
	; we had better do *something*.  Probably safest to restart from the
	; default countdown.

	pushd offset szInvalidTime	; Debug that we have a problem
	call DebugMsg			;

	mov eax,ICountdownTime		;
					;
countdown_valid:			;
	mov Sec2Go,eax			;

not_auto_restart:	


	; **** Set up timer ****
retry_timer:				;
	xor eax,eax			;
	pushd eax			; lpTimerFunc
	pushd 1000			; uElapse = 1 Sec
	inc eax				;
	pushd eax			; nIDEvent = 1
	pushd hWnd			; hWnd
	call _SetTimer@16		;
	or eax,eax			;
	jnz timer_ok			;
	pushd MB_ICONEXCLAMATION + MB_RETRYCANCEL
	pushd offset szAppTitle		;
	pushd offset szNoTimers	;
	pushd hWnd			; hWnd
	call _MessageBoxA@16		;
	cmp eax,IDRETRY			; Does user want to retry ?
	je retry_timer			; If so, jump back
	xor eax,eax			;
	pushd eax			; If not, exit program
	call _PostQuitMessage@4		;
	jmp end_wndproc		;
timer_ok:				;
;;	mov nTimer,eax			;

	; Update time string for initial WM_PAINT
	jmp write_new_count		; WARNING, jumps into WM_TIMER routine

	;**** End of WM_CREATE ****

wndproc_wm_command:	;**** WM_COMMAND ****
	mov eax,dword ptr [esp+12+12]	; wParam in EAX
	cmp eax,12			; "Start Countdown" ?
	je wndproc_cmd_start		;
	cmp eax,13			; "Stop Countdown" ?
	je wndproc_cmd_stop		;
	cmp eax,14			; "Reset Countdown" ?
	je wndproc_cmd_reset		;
	cmp eax,20			; "Exit" ?
	je wndproc_cmd_exit		;
	cmp eax,100			; "Record Message..." ?
	je wndproc_cmd_rec		;
	cmp eax,101			; "Synthesise Message..." ?
	je wndproc_cmd_synth		;
	cmp eax,202			; "Configure..." ?
	je wndproc_cmd_configure	;
	cmp eax,300			; "Contents" ?
	je wndproc_cmd_contents		;
	cmp eax,301			; "About" ?
	je wndproc_cmd_about		;
	; Don't process message		;

	jmp wndproc_dont_process	;

wndproc_cmd_configure:	;**** Command: Tools | "Configure..." ****

	; Display "Configure" dialogue
	xor eax,eax			;
	pushd eax			;
	pushd offset ConfigProc		;
	pushd hWnd			;
	pushd offset szConfigure	;
	pushd hInstance			;
	call _DialogBoxParamA@20	;
	jmp end_wndproc			;

wndproc_cmd_start:	;**** Command: CountDown | Start ****
	; Resume counting - called by both menu item and pushbutton
	cmp bConfigValid,0		; Is our config OK ?
	je cant_start			; If not, jump (can't start)
can_now_start:
	
	call StartCounting

	mov eax,Max_attempts		;
	mov Attempts2Go,eax		; Reset attempts

	cmp bAutoStart,0		; Was Autostart specified ?
	jz no_autostart			; If not, jump
	; **** Autostart specified ****

	call AddAutoRun			; Add Auto Run key to registry
no_autostart:				;


	jmp write_new_count		; WARNING, jumps into WM_TIMER routine

cant_start:	; We can't start because the config is bad

	; Ask if user wants to configure
	pushd MB_ICONQUESTION + MB_YESNO
	pushd offset szAppTitle		;
	pushd offset szTryConfig	;
	pushd hWnd			; hWnd
	call _MessageBoxA@16		;

	cmp eax,IDYES			; Did user want to configure ?
	jne dont_start			; If not, exit
					; If so, display "Configure" dialogue
	xor eax,eax			;
	pushd eax			;
	pushd offset ConfigProc		;
	pushd hWnd			;
	pushd offset szConfigure	;
	pushd hInstance			;
	call _DialogBoxParamA@20	;

	cmp bConfigValid,0		; Is our config Now OK ?
	jne can_now_start		; If so, start counting
					; If not, give up
dont_start:
	; Fall through to Stop routine if we fail to start, as we
	; need to de-select the "Start" radio button.

wndproc_cmd_stop:	;**** Command: CountDown | "Stop" ****
	; Called by both menu item and pushbutton
	; Pause counting, but do not reset anything

	; Debug what we are doing
	pushd offset szStopped		;
	call DebugMsg			;


	call StopCounting

	;**** Were We Dialling ? ****
	mov al,bDialStatus		;
	or al,al			;
	jz stop_not_dialling		; Jump if not
	; **** We were dialling and we need to abort this. ****

	; Were we in the process of hanging up ?
	cmp al,90			;
	je stop_hanging_up		;

	; Were we in the process of calling the number ?
	cmp al,20			;
	je stop_calling			;
	cmp al,30			;
	jne stop_not_calling		;

stop_calling:				;
stop_hanging_up:			;
	; ** Close Line **
	pushd hLine			;
	call _lineClose@4		;
	mov bDialStatus,0		; Stopped dialling
	jmp stop_dialling		;

	; Were we in the process of speaking the message ?
stop_not_calling:			;
	cmp al,70			;
	jne stop_not_speaking		;
	call Dial_ManualStop_during_speak
	jmp stop_dialling		;
stop_not_speaking:			;
	; If we get here, we were in the middle of doing something
	; (e.g. preparing a message to send, etc.) when the user 
	; clicked the button.  So, we set a flag which gets acted on
	; when ready.

	mov bAbortDial,1

stop_dialling:				;

	; OK, the user has clicked "stop" while we were dialling and 
	; we have aborted to dialling process.  
	; Now, what do we do with the time ?
	; Probably it is best to reset it...
	mov eax,ICountdownTime		;
	mov Sec2Go,eax			; Reset time
	mov eax,Max_attempts		;
	mov Attempts2Go,eax		; Reset  attempts

stop_not_dialling:



write_new_count:			;
	mov eax,Sec2Go			;
	jmp write_new_count2		; WARNING, jumps into WM_TIMER routine


wndproc_cmd_reset:	;**** Command: CountDown | "Reset" ****
	; Called by both menu item and pushbutton
	; Reset the count, but do not change whether we are counting or
	; not

	; Debug what we are doing
	pushd offset szReset		;
	call DebugMsg			;

	mov eax,Max_attempts		;
	mov Attempts2Go,eax		; Reset  attempt

	mov eax,ICountdownTime		;
	mov Sec2Go,eax			; Reset time
	push eax			;
	call record_new_starttime	; Record time and attempts in INI file
	pop eax				;
	jmp write_new_count2		;

wndproc_cmd_exit:	;**** Command: "Exit" ****
automatic_exit:
	xor eax,eax			;
	pushd eax			;
	pushd eax			;
	pushd WM_CLOSE			;
	pushd hWnd			;
	call _SendMessageA@16		;
	jmp end_wndproc			;

wndproc_cmd_rec:	;**** Command: Message | Record... ****
	; Display "Record" dialogue
	xor eax,eax			;
	pushd eax			;
	pushd offset RecordProc		;
	pushd hWnd			;
	pushd offset szRecord		;
	pushd hInstance			;
	call _DialogBoxParamA@20	;
	jmp end_wndproc			;

wndproc_cmd_synth:	;**** Command: Message | Synthesise... ****
	; Display "Record" dialogue
	xor eax,eax			;
	pushd eax			;
	pushd offset SynthProc		;
	pushd hWnd			;
	pushd offset szSynthesise	;
	pushd hInstance			;
	call _DialogBoxParamA@20	;
	jmp end_wndproc			;

wndproc_cmd_contents:	;**** Command: Help | Contents ****
	; Display help contents
	xor eax,eax			;
	pushd eax			;
	pushd HELP_INDEX		;
	pushd offset szHelpFileName	;
	pushd hWnd			;
	call _WinHelpA@16		;
	jmp end_wndproc			;

wndproc_cmd_about:	;**** Command: Help | About ****
	; Display "About" dialogue
	xor eax,eax			;
	pushd eax			;
	pushd offset AboutProc		;
	pushd hWnd			;
	pushd offset szAboutBox		;
	pushd hInstance			;
	call _DialogBoxParamA@20	;
	jmp end_wndproc			;

	;**** End of WM_COMMAND ****

wndproc_wm_paint:	;**** WM_PAINT ****

	sub esp,52			; Allocate PAINTSTRUCT on stack
		; PAINTSTRUCT structure
;	dd ?				; hdc
;	dd ?				; fErase
;	dd ?				; rcPaint
;	dd ?				; fRestore
;	dd ?				; fIncUpdate
;	db 32 dup(?)			; rgbReserved

	pushd esp			;
	pushd hWnd			; hWnd
	call _BeginPaint@8		;
	mov edi,eax			; Save hDc

	pushd COLOR_WINDOW		; Use default Windows colour
	call _GetSysColor@4		;
	pushd eax			;
	pushd edi			; hDc
	call _SetBkColor@8		; for background

	pushd hBigFont			;
	pushd edi			; hDc
	call _SelectObject@8		; Select the BIG font
	push eax			; Save handle of prev font

	pushd COLOR_WINDOWTEXT		; Use default Windows text colour
	call _GetSysColor@4		;

	pushd eax			;
	pushd edi			; hDc
	call _SetTextColor@8		; for text

	mov eax,DT_CENTER + DT_NOCLIP + DT_VCENTER + DT_SINGLELINE ; Assume single line only 

	pushd eax			; uFormat
	pushd offset main_rect		; lpRect
	pushd -1			; nCount (-1 = null terminated)
	pushd offset szMainText		; lpString - text to print
	pushd edi			; hDc
	call _DrawTextA@20		;

	; handle of previous font is already on stack...
	pushd edi				; hDc
	call _SelectObject@8		;

	pushd esp			;
	pushd hWnd			; hWnd
	call _EndPaint@8		;  Finished Painting

	add esp,52			; Deallocate PAINTSTRUCT

	jmp end_wndproc	;**** End of WM_PAINT ****

wndproc_wm_timer:	;**** WM_TIMER ****

	cmp bCountingDown,0		; Are we counting down ?
	je end_wndproc			; If not, jump

	cmp bDialStatus,0		; Are we currently dialling ?
	jne end_wndproc			; Jump if so

	mov eax,Sec2Go			; Seconds to go
	dec eax				; One less second, now
	or eax,eax			; Have we reached zero ?
	jg countdown_not_over		; Jump if not

	; **** **** Initiate Dial **** *****
	call dial			;

	jmp end_wndproc			;

countdown_not_over:			;

	; **** Still Counting Down ****
	mov Sec2Go, eax			; Save new seconds

write_new_count2:			; WARNING: Jumps from various commands
	mov edi, offset szMainText ; Where to write it

	; *** Convert time in seconds to text HH:MM:SS ***
	xor edx,edx			;
	pushd edx			; Null terminator
	mov ebx,10			; Units Seconds
	div ebx				;
	add dl,"0"			;
	pushd edx			;
	or eax,eax			;
	jz end_time			; Jump if <10 Sec
	mov ebx,6			; Tens Seconds
	xor edx,edx			;
	div ebx				;
	add dl,"0"			;
	pushd edx			;
	or eax,eax			;
	jz end_time			; Jump if <1 Min
	pushd ":"			; Units Minutes
	mov ebx,10			;
	xor edx,edx			;
	div ebx				;
	add dl,"0"			;
	pushd edx			;
	or eax,eax			;
	jz end_time			; Jump if <10 Min
	mov ebx,6			; Tens Minutes
	xor edx,edx			;
	div ebx				;
	add dl,"0"			;
	pushd edx			;
	or eax,eax			;
	jz end_time			; Jump if <1 Hr
	pushd ":"			; Hours
	mov ebx,24			;
	xor edx,edx			;
	div ebx				;
	mov ecx,eax			; Save days in ECX
	mov eax,edx			; Total hours
	xor edx,edx			;
	mov ebx,10			; Units Hours
	div ebx				;
	add dl,"0"			;
	push edx			;
	or eax,eax			;
	jz days_to_write		; Jump if <10 Hrs
	add al,"0"			;
	push eax			;
days_to_write:				;
	mov eax,ecx			; Get Days
	or eax,eax			;
	jz end_time			; Jump if <1 Day
	push eax			; Save days
	call itoa			; Convert days
	; Append " Day(s) "		;
	mov eax,"yaD "			;
	stosd				;
	pop eax				; Restore days
	cmp al,1			;
	je one_day			; Jump if only one day
	mov al,"s"			;
	stosb				; " Days "
one_day:				;
	mov al," "			;
	stosb				;
end_time:				;
next_char:				;
	pop eax				;
	stosb				;
	or al,al			;
	jnz next_char			;
dialling_now:				;


	call update_window		; Update title & force redraw

	jmp end_wndproc	;**** End of WM_TIMER ****

wndproc_wm_timechange:	;**** WM_TIMECHANGE ****
	; Someone has changed the time, so if we are running, we should now
	; record a new restart time.

	call record_new_starttime	; So do it

	jmp end_wndproc	;**** End of WM_TIMECHANGE ****

wndproc_wm_done:	; **** WOM_DONE ****
	cmp bDialStatus,70		;
	jne end_wndproc			;
	call dial_wom_done	;
	jmp end_wndproc	;**** End of WOM_DONE ****

wndproc_wm_close:	;**** WM_CLOSE ****
	; Note: This is called BEFORE the window is destroyed.
	; Here we can prevent the user from exiting if required.
	; In order to destroy the window we need to call
	; _DestroyWindow@4 explicitly

	mov al,bCountingDown		;
	cmp al,1			; Are we counting down ?
	jne onexit_not_counting 	; If not, jump

	; Warn user a countdown is in progress
	pushd MB_ICONQUESTION + MB_YESNO
	pushd offset szAppTitle		;
	pushd offset szCountInProg	;
	pushd hWnd			; hWnd
	call _MessageBoxA@16		;

	cmp eax,IDYES			; Did user really want to exit ?
	jne dont_exit			; If not, jump

onexit_not_counting:			;
					;

	call RemoveAutoRun		;  Remove any auto-run option in registry

	pushd hWnd			; hWnd
	call _DestroyWindow@4		;

dont_exit:				;

	jmp end_wndproc	;**** End of WM_CLOSE ****

wndproc_wm_destroy:	;**** WM_DESTROY ****
	; Note: This is called AFTER the window is destroyed.
	; Here we clean up everything that was created by WM_CREATE

	; **** Shut down any help windows ****
	xor eax,eax			;
	pushd eax			;
	pushd HELP_QUIT			;
	pushd eax			;
	pushd hWnd			;
	call _WinHelpA@16		;

	; **** Delete all objects ****
	pushd 1				;
	pushd hWnd			; hWnd
	call _KillTimer@8		; Finished with timer

	; **** Save parameters in INI file ****
retry_ini_write1:			;
	; ** Window (x,y) coords of top left corner **
	; Get coords of window (in screen coords)
	pushd offset temp_buffer	; temp_buffer has a RECT structure written to it
	pushd hWnd			; hWnd
	call _GetWindowRect@8		;

	mov eax,dword ptr temp_buffer	; Get x coord
	pushd dword ptr temp_buffer+4	; Save y coord
	mov ebx,offset szX		; Start with X coord
do_y_coord:				;
	or eax,eax			; Is the coordinate negative ?
	jg coord_nonzero		; Jump if not
	xor eax,eax			; If negative, zero it
coord_nonzero:				;
	mov edi,offset temp_buffer	; Where to write coord as ASCIIZ
	call itoa			; Convert coord to string
	mov ecx,ebx			;
	mov eax,offset szWindow		;
	call Write_INI			; Write coord to INI file
	or eax,eax			; Did we write OK ?
	jz ini_write_failed		; If not, jump (skip all futher writes)
	cmp ebx,offset szX		; Have we done X and Y
	jne end_coords			; If so, jump
	pop eax				; Get y coord
	mov ebx,offset szY		;
	jmp do_y_coord			;
end_coords:				;

	; **** Save (X,Y) of all dialogues  ****

	mov ebx,offset szAboutBox	; ** About **
	mov esi,offset xcoord_about	;
	call SaveDlgXY			;
	jz ini_write_failed		;

	mov ebx,offset szConfigure	; ** config **
	mov esi,offset xcoord_config	;
	call SaveDlgXY			;
	jz ini_write_failed		;

	mov ebx,offset szRecord		; ** record **
	mov esi,offset xcoord_record	;
	call SaveDlgXY			;
	jz ini_write_failed		;

	mov ebx,offset szSynthesise	; ** synth **
	mov esi,offset xcoord_synth	;
	call SaveDlgXY			;
	jz ini_write_failed		;

	mov ebx,offset szPlayBack	; ** play **
	mov esi,offset xcoord_play	;
	call SaveDlgXY			;
	jz ini_write_failed		;

	; ** Default Directory **
	pushd offset szIniFileName	;
	pushd offset szWaveDirName	;
	pushd offset szDir		;
	pushd offset szSettings		;
	call _WritePrivateProfileStringA@16; [Settings] Dir
	or eax,eax			;
	jz ini_write_failed		;

	; *** Space to allocate for record buffer  ***
	mov eax,SizeOfWaveInBufferRec	;
	xor edx,edx			; EDX:EAX = buffer size in bytes
	mov ecx,16000			; 16000 bytes/sec
	div ecx				; EAX = buffer size in seconds
	mov edi,offset temp_buffer	; Where to write size as ASCIIZ
	call itoa			; Convert size to string
	mov ecx, offset szBufferLen	;
	mov eax, offset szRecord	;
	call Write_INI			; Write to INI file: [Record] Buffer Length
	or eax,eax			;
	jz ini_write_failed		;

	; *** Synthesised Voice Selected  ***
	mov eax,dwVoiceSelected		;
	mov edi,offset temp_buffer	; Where to write as ASCIIZ
	call itoa			; Convert to string
	mov ecx, offset szVoice		;
	mov eax, offset szSettings	;
	call Write_INI			; Write to INI file: [Settings] Voice
	or eax,eax			;
;;	jz ini_write_failed		;

	jnz ini_write_ok		;

ini_write_failed:			;
	; **** Failed to write INI file - retry ? ****
	pushd MB_ICONEXCLAMATION + MB_RETRYCANCEL
	pushd offset szAppTitle		;
	pushd offset szNoIniFile	;
	pushd hWnd			; hWnd
	call _MessageBoxA@16		;
	cmp eax,IDRETRY			; Does user want to retry ?
	je retry_ini_write1		; If so, jump back
ini_write_ok:				;

	xor eax,eax			;
	pushd eax			;
	call _PostQuitMessage@4		; Tell processing loop to exit
	jmp end_wndproc	;**** End of WM_DESTROY ****

WndProc ENDP

;****************************************************************************
; Name: ReadDlgXY
; Purpose: Reads X and Y coordinates for a dialogue box from the INI file
; Parameters: ESI = offset of dialogue box name
; Returns: EAX=X coord, EBX=Y coord
;****************************************************************************
ReadDlgXY PROC
	pushd offset szIniFileName    	;
	mov eax,ycoord			;
	add eax,200			; Default = parent + 200
	pushd eax			;
	pushd offset szY		;
	pushd esi		;
	call _GetPrivateProfileIntA@16  ;
	mov ebx,eax			; Save Y coord
	pushd offset szIniFileName    	;
	mov eax,xcoord			;
	add eax,200			;
	pushd eax			; Default = parent + 200
	pushd offset szX		;
	pushd esi			;
	call _GetPrivateProfileIntA@16  ;
	ret
ReadDlgXY ENDP


;****************************************************************************
; Name: SaveDlgXY
; Purpose: Saves X and Y coordinates for a dialogue box to the INI file
; Parameters: EBX = offset of dialogue box name ESI= offset of X coordinate
; Returns: NZ=OK, Z=Failed
;****************************************************************************
SaveDlgXY PROC
	lodsd				; Get X coord
	or eax,eax			; Is the c oordinate negative ?
	jg x_coord_nonzero		; Jump if not
	xor eax,eax			; If negative, zero it
x_coord_nonzero:			;
	mov edi,offset temp_buffer	; Where to write Y coord as ASCIIZ
	call itoa			; Convert coord to string
	mov ecx,offset szX		;
	mov eax,ebx			;
	call Write_INI			; Write X coord to INI file
	or eax,eax			;
	jz end_save_coords		; Jump if failed

	lodsd				; Get Y coord
	or eax,eax			; Is the Y oordinate negative ?
	jg y_coord_nonzero		; Jump if not
	xor eax,eax			; If negative, zero it
y_coord_nonzero:			;
	mov edi,offset temp_buffer	; Where to write X coord as ASCIIZ
	call itoa			; Convert coord to string
	mov ecx,offset szY		;
	mov eax,ebx			;
	call Write_INI			; Write Y coord to INI file
	or eax,eax			;
end_save_coords:
	ret
SaveDlgXY ENDP

;****************************************************************************
; Name: check_config
; Purpose : Checks the configuration data.  This is used to determine if we
; have a valid configuration.
;****************************************************************************

check_config PROC

	; ** Check Maximum number of attempts **
	mov eax,NewMax_attempts		;
	cmp eax,100			; Is it in the range 1-100 ?
	ja check_config_bad		; Jump if not
	cmp eax,1			; Is it below 1 ?
	jl check_config_bad		; Jump if not

	; ** Check number of Repeats **
	mov eax,NewRepeats		;
	cmp eax,100			; Is it in the range 1-100 ?
	ja check_config_bad		; Jump if not
	cmp eax,1			; Is it below 1 ?
	jl check_config_bad		; Jump if not

	; **** Check Inital time Delay ****
	cmp NewICountdownTime,0		;
	jl check_config_bad		;

	; **** Check Retry time Delay ****
	cmp NewRCountdownTime,0		;
	jl check_config_bad		;

	; **** Check Modem Selection ****
	cmp dwNumLineDevs,0			;
	jle check_config_bad		; Jump if no modems exist

	; **** See what modems exist ****
	; Unfortunately, we can't look the modem up by dwPermanentLineID
	; So we have to search all modems and check that this ID exists
	;
	; ESI - Device Index
	; EDI - SizeOf(LineDevCaps)
	xor esi,esi			; Start device count at zero
check_next_modem:			;

	call GetPermanentLineID		;
	jc check_bad_modem		; Jump if error

	cmp eax,dwNewPermanentLineID	; Was this modem the selected one ?
	je check_modem_found		; Jump if found

check_bad_modem:			;
	inc esi				; Next device next time
	cmp esi, dwNumLineDevs		; Have we tried all devices ?
	jl check_next_modem		; Try next modem next time

	jmp check_config_bad		; Checked all without luck - Error

check_modem_found:
	mov dwNewDeviceID,esi		; Record device number of modem
	mov dwDeviceID,esi		;

	; **** Check telephone number ****
	; We don't want to exclude numbers containing weird stuff, so
	; don't do much checking.  
	cmp szNumber2Dial,0		; Is string empty ?
	jz check_config_bad		; Jump if so

	; **** Check WAV file exists and read it ****
	push offset szVoiceFileName
	call ReadWAVFile		;
	jc check_config_bad		;


	clc				; No error
	ret				;
check_config_bad:
	stc				; Error
	ret				;
check_config ENDP


;****************************************************************************
; Name: GetPermanentLineID
; Purpose: Given a Device ID in ESI, returns a Permanent Line ID in EAX
; Returns: NC=Device ID found,  Permanent Line ID in EAX, CY=not found
;****************************************************************************
GetPermanentLineID PROC
	mov eax,256			; Memory to allocate initially
GetPermanentLineID_resize:		;
	sub esp,eax			; Allocate stack space for lineDevCaps
	mov edi,eax			; Save it
	mov dword ptr [esp],eax		; Set dwTotalSize

	; ** Get API Version supported by device **
	sub esp,20			; Make space for dwAPIVersion and ExtensionID on stack
	lea eax,dword ptr [esp+16]	; lpdwAPIVersion
	pushd esp			; lpExtensionID
	pushd eax			; lpdwAPIVersion
	pushd 020001h			; dwAPIHighVersion
	pushd 010004h			; dwAPILowVersion
	pushd esi			; dwDeviceID
	pushd hLineApp			; hLineApp
	call _lineNegotiateAPIVersion@24
	add esp,16			; Dispose of ExtensionID
	pop ecx				; dwAPIVersion
	or eax,eax			; Look at return code
	jnz GetPermanentLineID_bad	; Jump if error

	xor eax,eax			;
	pushd esp			; lpLineDevCaps
	pushd eax			; dwExtVersion=0
	pushd ecx			; dwAPIVersion
	pushd esi			; dwDeviceID
	pushd hLineApp			; hLineApp
	call _lineGetDevCapsA@20	;
	or eax,eax			; Look at return code
	jnz GetPermanentLineID_bad	; Jump if error

	; lineGetDevCaps() succeeded, but was LineDevCaps big enough ?

	mov eax, dword ptr [esp+4]	; Get dwNeededSize
	cmp eax,MaxAlloc		; Check this is sensible
	ja GetPermanentLineID_bad	; Jump if not
	cmp eax,edi			; Did we allocate enough memory already ?
	jle GetPermanentLineID_memOK	; Jump if so
	; LineDevCaps was not big enough
	; As we loop back to lineGetDevCaps(), we could in theory get
	; stuck in an infinite loop if dwNeededSize changes.
	cmp edi,256			; Was this the first iteration ?
	jne GetPermanentLineID_bad	; Jump if not

	add esp,edi			; Deallocate existing space
	add eax,3			; Round up required size to nearest DWORD
	and eax,0fffffffch		; (this preserves stack alignment)
	jmp GetPermanentLineID_resize	; Try again

GetPermanentLineID_memOK:
	; We have successfully read the structure.
	mov eax, dword ptr [esp+28]	; Get dwPermanentLineID
	add esp,edi			; Deallocate memory
	clc				; No Error
	ret

GetPermanentLineID_bad:			;
	; We have failed to read the structure.
	add esp,edi			; Deallocate memory
	stc				; Error
	ret
GetPermanentLineID ENDP


;****************************************************************************
; Name: record_new_starttime
; Purpose: Writes a new start time to the INI file for auto-start purposes
;****************************************************************************

record_new_starttime PROC

	; ** Only actually do this is we are both counting down, and we are
	; required to auto-start **
	mov al,bCountingDown		; Were we counting down ?
	or al,bAutoStart		; Was Autostart specified ?
	jz record_new_starttime_end	; If not, exit (no need to record time)

	; Work out time to auto-start
	; Add CountdownTime to current time/date

	; Get Current Time
	pushd offset wYear		; SYSTEMTIME structure
	call _GetSystemTime@4		; Get UTC


	; Convert time to FILETIME (64 bit time & date in units of 100nSec) format
	pushd offset dwLowDateTime	; FILETIME structure
	pushd offset wYear		; SYSTEMTIME structure
	call _SystemTimeToFileTime@8	;

	; Get time to go
	mov eax,Sec2Go			; How long do we have to go ?
	mov ebx,10000000		;
	mul ebx				; Convert to units of 100nSec

	; Add to current time
	add eax,dwLowDateTime		;
	adc dwHighDateTime,edx		;

	mov edi,offset temp_buffer	;
	call itoa			;

	mov ecx,offset szTimeLo		;
	mov eax,offset szRun		;
	call Write_INI			; Write to [Run] TimeLo

	mov eax,dwHighDateTime		;
	mov edi,offset temp_buffer	;
	call itoa			;

	mov ecx,offset szTimeHi	;
	mov eax,offset szRun	;
	call Write_INI			; Write to [Run] TimeHi

	; ** Also set number of attempts to go **
	; Note: This will be redundant if we are acting on a time-change
	mov eax,Attempts2Go		;
	mov edi,offset temp_buffer	;
	call itoa			;
	mov ecx, offset szAttempts	;
	mov eax, offset szRun		;
	call Write_INI			; [Run] Attempts
record_new_starttime_end:
	ret
record_new_starttime ENDP


;****************************************************************************
; Name: AddAutoRun
; Purpose: Add Auto Run option from Registery
;****************************************************************************
AddAutoRun PROC

	; *** Open Registry Key HKLM\Software\Microsoft\Windows\CurrentVersion\Run ***
	xor eax,eax			;
	pushd offset dwDisposition	; LPDWORD lpdwDisposition
	pushd offset hRegKey		; PHKEY phkResult
	pushd eax			; LPSECURITY_ATTRIBUTES lpSecurityAttributes
	pushd KEY_ALL_ACCESS		; REGSAM samDesired
	pushd eax			; DWORD dwOptions
	pushd offset szKeyClass		; LPTSTR lpClass
	pushd eax			; DWORD Reserved
	pushd offset szRunKey		; LPCTSTR lpSubKey
	pushd HKEY_LOCAL_MACHINE	; HKEY hKey
	call _RegCreateKeyExA@36	;
	or eax,eax			;
	jnz RegCreateKeyError		;
	mov esi,hRegKey			;

	; *** Get full name of this program (eg: C:\tdc\tdc.exe)***
	pushd 255			;
	pushd offset temp_buffer	;
	pushd hInstance			;
	call _GetModuleFileNameA@12	;
	mov dword ptr [temp_buffer + eax], "r- "	; change to "C:\tdc\tdc.exe -r",0
	add eax,4			;

	; *** Set value for this program ***
	pushd eax			; DWORD cbData
	pushd offset temp_buffer	; CONST BYTE *lpData
	pushd REG_SZ			; DWORD dwType
	pushd 0				; DWORD Reserved
	pushd offset szTimeDelayCaller	; LPCTSTR lpValueName
	pushd esi			; HKEY hKey
	call _RegSetValueExA@24		;

	; *** Close the Key ***
	pushd esi			; HKEY hKey
	call _RegCloseKey@4		;

	call record_new_starttime	; Record time and attempts in INI file
RegCreateKeyError:

	ret
AddAutoRun ENDP

;****************************************************************************
; Name: RemoveAutoRun
; Purpose: Removed Auto Run option from Registery
;****************************************************************************
RemoveAutoRun PROC

	; *** Open Registry Key HKLM\Software\Microsoft\Windows\CurrentVersion\Run ***
	xor eax,eax			;
	pushd offset dwDisposition	; LPDWORD lpdwDisposition
	pushd offset hRegKey		; PHKEY phkResult
	pushd eax			; LPSECURITY_ATTRIBUTES lpSecurityAttributes
	pushd KEY_ALL_ACCESS		; REGSAM samDesired
	pushd eax			; DWORD dwOptions
	pushd offset szKeyClass		; LPTSTR lpClass
	pushd eax			; DWORD Reserved
	pushd offset szRunKey		; LPCTSTR lpSubKey
	pushd HKEY_LOCAL_MACHINE	; HKEY hKey
	call _RegCreateKeyExA@36	;
	or eax,eax			;
	jnz RegCreateKeyError2		;
	mov esi,hRegKey			;

	; *** Delete value for this program ***
	pushd offset szTimeDelayCaller	; LPCTSTR lpValueName
	pushd esi			; HKEY hKey
	call _RegDeleteValueA@8		;

	; *** Close the Key ***
	pushd esi			; HKEY hKey
	call _RegCloseKey@4		;

RegCreateKeyError2:

	ret
RemoveAutoRun ENDP				;

;****************************************************************************
; Name: Write_INI
; Purpose: Writes string at temp_buffer to INI file section [EAX] ECX
; Returns: eax = return code of Write_INI
;****************************************************************************
Write_INI PROC
	pushd offset szIniFileName	;
	pushd offset temp_buffer	;
	pushd ecx			;
	pushd eax			;
	call _WritePrivateProfileStringA@16	;
	ret				;
Write_INI ENDP

;****************************************************************************
; Name: ReadWAVFile
; Purpose: Reads the user-selected WAV file into memory for later playing
; Parameters: Pointer to ASCIIZ file name on stack
; Returns: CY=Fail, NC=OK
; Note: Need to deallocate heap memory for lpFmtChunk and lpDataChunk after
; a successful call.
;****************************************************************************
ReadWAVFile PROC

	call FreeWAVMemory		; Free any existing memory allocations

	;** Open File **
	xor eax,eax			;
	mov ebx,dword ptr [esp+4]	;
	pushd MMIO_READ + MMIO_ALLOCBUF	; dwOpenFlags
	pushd eax			; lpmmioinfo=0
	pushd ebx			; szFilename
	call _mmioOpenA@12		;
	or eax,eax			;
	jnz mmioOpenA_OK		;
;;	mov esi,offset szCantOpen	;
	jmp mmioOpenA_fail		;
mmioOpenA_OK:				;
	mov edi,eax			; hmmio

	; Allocate memory on stack for 2 x MMCKINFO Structures
	sub esp,40

	;** Find RIFF Chunk **
	mov dword ptr [esp+8], "EVAW"	; mmckinfo1.fccType="WAVE"
	mov esi,esp			;
	xor eax,eax			;
	mov dword ptr [esi],eax		; mmckinfo1.ckid=0 [May not be needed]
	pushd MMIO_FINDRIFF		; wFlags
	pushd eax			; lpckParent=0
	pushd esi			; lpck
	pushd edi			; hmmio
	call _mmioDescend@16		;
	or eax,eax			;
	jz mmioDescend_OK1		;
mmioDescend_fmt_fail2:			;
;;	mov esi, offset szBadFormat	;
	jmp mmioDescend_fmt_fail	;
mmioDescend_OK1:			;

	;** Find Format Chunk **
	lea eax,[esp+20]		;
	mov dword ptr [eax], " tmf" 	; mmckinfo2.ckid = "fmt "
	pushd MMIO_FINDCHUNK		; wFlags
	pushd esi			; lpckParent
	pushd eax			; lpck
	pushd edi			; hmmio
	call _mmioDescend@16		;
	or eax,eax			;
	jnz mmioDescend_fmt_fail2	;

	;** Is the length OK ? **
	mov ebx,cSizeOfWaveFormatEx	;
	cmp dword ptr [esp+24],ebx	;
	jne mmioRead_fmt_fail		;

    	;** Read Format Chunk **
	sub esp,ebx			;
	mov ecx,esp			;
	pushd ebx			; cch
	pushd ecx			; pch
	pushd edi			; hmmio
	call _mmioRead@12		;
	cmp eax,ebx			;
	je mmioRead_fmt_OK		;
mmioRead_fmt_fail2:			;
	add esp,ebx			; Deallocate memory
	jmp mmioRead_fmt_fail		;
mmioRead_fmt_OK:			;

	mov eax,edi			; Save hmmio

	; ** Compare format with what is acceptable **
	mov esi, offset WaveFormat	;
	mov edi,esp			;
	mov ecx,ebx			;
	repe cmpsb			;
	jne mmioRead_fmt_fail2		;
	add esp,ebx			; Deallocate memory

	mov edi,eax			; Restore hmmio

	;** Go back to RIFF chunk **
	lea ebx,[esp+20]		; mmckinfo2
	xor eax,eax			;
	pushd eax			; wFlags
	pushd ebx			; lpck
	pushd edi			; hmmio
	call _mmioAscend@12		;
	or eax,eax			;
	jz mmioAscend_OK		;
mmioDescend_data_fail2:			;
;;	mov esi, offset szBadFormat	;
	jmp mmioAscend_fail		;
mmioAscend_OK:				;

	;** Find Data Chunk **
	mov dword ptr [ebx], "atad" 	; mmckinfo2.ckid = "data"
	pushd MMIO_FINDCHUNK		; wFlags
	pushd esi			; lpckParent
	pushd ebx			; lpck
	pushd edi			; hmmio
	call _mmioDescend@16		;
	or eax,eax			;
	jnz mmioDescend_data_fail2	;

	;** Allocate Memory for Data Chunk **
	mov eax,dword ptr[esp+24]	; Get size for data chunk mmckinfo2.cksize
	mov dwDataSize,eax		;
	pushd eax			; dwBytes
	xor eax,eax			;
	pushd eax			; dwFlags=0
	pushd hProcessHeap		; hHeap
	call _HeapAlloc@12
	or eax,eax			;
	jnz HeapAlloc_OK2		;
;;	mov esi, offset szCantAlloc	;
	jmp HeapAlloc_data_fail		;
HeapAlloc_OK2:				;
	mov lpDataChunk,eax		; Pointer to allocated data chunk memory

    	;** Read Data Chunk **
	pushd dwDataSize		; cch
	pushd eax			; pch
	pushd edi			; hmmio
	call _mmioRead@12		;
	cmp eax,dwDataSize		;
	je mmioRead_data_OK		;
;;	mov esi, offset szCantRead	;
	jmp mmioRead_data_fail		;
mmioRead_data_OK:			;

	; Deallocate memory on stack for the 2 x MMCKINFO Structures
	add esp,40			;

	; ** Close File **
	xor eax,eax			;
	pushd eax			; wFlags
	pushd edi			; hmmio
	call _mmioClose@8		;

	; *** create WAVEHDR structure ***

	; ** Set size and address of data **
	mov eax,lpDataChunk		;
	mov WaveOutHdr,eax		;
	mov eax,dwDataSize		;
	mov WaveHdr_dwBufferLength,eax	;

	;** Success **
	; We have successfully read the WAV file into memory
	clc				; No Error
	ret 4				; Finished

mmioRead_data_fail:			;
	; ** Deallocate data chunk memory **
	xor eax,eax			;
	pushd lpDataChunk		; lpMem
	pushd eax			; dwFlags
	pushd hProcessHeap		; hHeap
	call _HeapFree@12		;
HeapAlloc_data_fail:			;
mmioDescend_data_fail:			;
mmioAscend_fail:			;
mmioRead_fmt_fail:			;
mmioDescend_fmt_fail:			;

	; Deallocate memory on stack for the 2 x MMCKINFO Structures
	add esp,40			;

	; ** Close File **
	xor eax,eax			;
	pushd eax			; wFlags
	pushd edi			; hmmio
	call _mmioClose@8		;

mmioOpenA_fail:

	stc				; Error
	ret 4				;

ReadWAVFile ENDP

;****************************************************************************
; Name: FreeWAVMemory
; Purpose: Frees up heap memory for WAV file, if any, allocated by
;  ReadWAVFile above.
;****************************************************************************
FreeWAVMemory PROC
	; ** Have we allocated a data chunk ? **
	mov eax,lpDataChunk		; Get size
	or eax,eax			; Is it zero
	jz no_datachunk_alloc		; Jump if so
	; ** Deallocate data chunk memory **
	pushd eax			; lpMem
	xor eax,eax			;
	pushd eax			; dwFlags
	pushd hProcessHeap		; hHeap
	call _HeapFree@12		;
	xor eax,eax			;
	mov lpDataChunk,eax		;
no_datachunk_alloc:


	ret
FreeWAVMemory ENDP

;****************************************************************************
; Name: update_window
; Purpose: Given that new text has been written to szMainText,
; updates the window title and forces a repaint
;****************************************************************************
update_window PROC

	; ** Create caption from text **
	mov esi,offset szMainText		;
	mov edi,offset szWindowCaption		;
	mov eax,":CDT"			;
	stosd				;
	mov al," "			;
	stosb				;
new_caption_next:
	lodsb				;
	cmp al,13			; Replace <cr>
	jne not_cr			;
	mov al," "			; With a space
not_cr:					;
	stosb				;
	or al,al			;
	jnz new_caption_next		;

	; ** Write new window caption **
	push offset szWindowCaption	;
	pushd hWnd			; hWnd
	call _SetWindowTextA@8		; New window title

	; ** Force Repaint **
	pushd 1				;
	xor eax,eax			;
	pushd eax			;
	pushd hWnd			; hWnd
	call _InvalidateRect@12		; Force repaint (including background)
	ret
update_window ENDP


;****************************************************************************
; Name: AboutProc
; Purpose: Handles commands for "About" pop-up.  Basically just exits.
;	[esp+4] = hDlg
;	[esp+8] = iMessage
;	[esp+12] = wParam
;	[esp+16] = lParam
;****************************************************************************
AboutProc PROC

	push ebx			;
	push esi			;
	push edi			;

	mov eax,[esp+8+12]		; Look at what we have received
	cmp eax,WM_INITDIALOG		; Have we just initialized ?
	jne no_init_about		; If not, jump

	; ** Initialized **

	mov ebx,dword ptr [esp+4+12]	; hDlg
	mov hDlg,ebx			;

	pushd ycoord_about		; Resposition
	pushd xcoord_about		;
	call PositionDialog		;

	mov eax,1			; Exit with "True"
	jmp end_aboutproc		;

no_init_about:				;
	cmp eax,WM_COMMAND		;
	jne no_about_cmd		;
	mov eax,[esp+12+12]		; wParam
	cmp eax,IDOK			;
	je end_about_dialog		;
	cmp eax,IDCANCEL		;
	jne no_about_cmd		;
end_about_dialog:			;

	mov esi,offset temp_buffer	;
	mov edi,offset xcoord_about	;
	pushd esi			;
	pushd hDlg			;
	call _GetWindowRect@8		; Get bounding rectangle
	movsd				; Transfer X coord
	movsd				; Transfer Y coord

	mov ebx,[esp+4+12]		; hDlg
	xor eax,eax			;
	pushd eax			;
	pushd ebx			;
	call _EndDialog@8		;
	jmp end_aboutproc		;
no_about_cmd:				;
	xor eax,eax			; Exit with "False"
end_aboutproc:				;
	pop edi				;
	pop esi				;
	pop ebx				;
	ret 16				;
AboutProc ENDP


;****************************************************************************
; Name: ConfigProc
; Purpose: Handles commands for "Config" pop-up.
;	[esp+4] = hDlg
;	[esp+8] = iMessage
;	[esp+12] = wParam
;	[esp+16] = lParam
; Note:
;****************************************************************************
ConfigProc PROC
	push ebx			;
	push esi			;
	push edi			;
	mov eax,[esp+8+12]		; iMessage
	cmp eax,WM_INITDIALOG		;
	je config_init			;
	cmp eax,help_message		;
	je config_dlg_help		; [Help] button on "open file" dialogues
	cmp eax,WM_COMMAND		;
	jne exit_config_false		;

	;  ***** Command *****
	mov eax,[esp+12+12]		; wParam
	cmp eax,201			; Browse Voice Files
	je config_browse_vfile		;
	cmp eax,211			; Browse Executable Files
	je config_browse_rfile		;
	cmp eax,IDCANCEL		; [Cancel] Button
	je config_cancel_button		;
	cmp eax,IDOK			; [OK] button
	je config_ok_button		;
	cmp eax,140			; [Help] button
	je config_help_button		;
	cmp eax,123			; [Configure Modem] button
	je config_config_modem		;
	cmp eax,202			; [Play] Voice File button
	je config_play_sound		;
	; Otherwise Unknown command

exit_config_false:			;
	xor eax,eax			;
	jmp end_configproc

config_init: ; ***** Initialise Dialog *****

	mov ebx,dword ptr [esp+4+12]	; hDlg
	mov hDlg,ebx			;
	mov hwndOwner,ebx		; Set owner of Open File standard dialogues

	pushd ycoord_config		; Resposition Dialogue Box
	pushd xcoord_config		;
	call PositionDialog		;

	xor eax,eax			;
	mov al,bAutoStart		;
	pushd eax			;
	mov al,130			;
	pushd eax			;
	pushd ebx			;
	call _CheckDlgButton@12		; Set up "AutoStart" box

	xor eax,eax			;
	mov al,bAutoStop		;
	pushd eax			;
	mov al,131			;
	pushd eax			;
	pushd ebx			;
	call _CheckDlgButton@12		; Set up "AutoStop" box

	xor eax,eax			;
	mov al,bDebugToFile		;
	pushd eax			;
	mov al,132			;
	pushd eax			;
	pushd ebx			;
	call _CheckDlgButton@12		; Set up "Debug To File" box

	pushd offset szNumber2Dial	;
	pushd 105			;
	pushd ebx			;
	call _SetDlgItemTextA@12	; Set number

	xor eax,eax			;
	pushd eax			;
	pushd Max_attempts		;
	mov al,106			;
	pushd eax			;
	pushd ebx			;
	call _SetDlgItemInt@16		; Set attempts

	xor eax,eax			;
	pushd eax			;
	pushd dwRepeats			;
	mov al,107			;
	pushd eax			;
	pushd ebx			;
	call _SetDlgItemInt@16		; Set repeats

	pushd offset szVoiceFileName	;
	pushd 200			;
	pushd ebx			;
	call _SetDlgItemTextA@12	; Set voice file name

	pushd offset szRunFileName	;
	pushd 210			;
	pushd ebx			;
	call _SetDlgItemTextA@12	; Set run file name


	; **** Convert time delays ****
	mov eax,ICountdownTime		; Get initial delay time in seconds
	call TimetoS			; Convert to "DDD HH:MM:SS"
	pushd offset temp_buffer	;
	pushd 110			;
	pushd ebx			;
	call _SetDlgItemTextA@12	; Set initial delay

	mov eax,RCountdownTime		; Get retry delay time in seconds
	call TimetoS			; Convert to "DDD HH:MM:SS"
	pushd offset temp_buffer	;
	pushd 115			;
	pushd ebx			;
	call _SetDlgItemTextA@12	; Set retry delay

	; **** See what modems exist ****
	; We need to call lineGetDevCaps() for each line device, however
	; this function call uses a pointer to the structure LINEDEVCAPS
	; which is of variable size.  So what we do is to allocate initial
	; space on the stack for this structure, and call the function to
	; find the actual space required.  We can then reallocate
	; the space on the stack and call the function a second time
	; with the correct size.
	; ESI - Device Index
	; EDI - SizeOf(LineDevCaps)
	xor esi,esi			; Start device count at zero
	mov DevIndex,esi		; Initially assume default selection is at listbox index 0
	mov nDevs,esi			; Reset count of available devices
try_next_modem:				;
	push esi			; Save Device Count
	mov eax,256			; Memory to allocate initially
resize_lineDevCaps:			;
	sub esp,eax			; Allocate stack space for lineDevCaps
	mov edi,eax			; Save it
	mov dword ptr [esp],eax		; Set dwTotalSize

	; ** Get API Version supported **
	sub esp,20			; Make space for dwAPIVersion and ExtensionID on stack
	lea eax,dword ptr [esp+16]	; lpdwAPIVersion
	pushd esp			; lpExtensionID
	pushd eax			; lpdwAPIVersion
	pushd 020001h			; dwAPIHighVersion
	pushd 010004h			; dwAPILowVersion
	pushd esi			; dwDeviceID
	pushd hLineApp			; hLineApp
	call _lineNegotiateAPIVersion@24;
	add esp,16			; Dispose of ExtensionID
	pop ecx				; dwAPIVersion
	or eax,eax			; Look at return code
	jnz bad_modem			; Jump if error

	; ** Get Device Capabilities **
	pushd esp			; lpLineDevCaps
	xor eax,eax			;
	pushd eax			; dwExtVersion
	pushd ecx			; dwAPIVersion
	pushd esi			; dwDeviceID
	pushd hLineApp			; hLineApp
	call _lineGetDevCapsA@20	;
	or eax,eax			; Look at return code
	jnz bad_modem			; Jump if error

	; lineGetDevCaps() succeeded, but was LineDevCaps big enough ?

	mov eax, dword ptr [esp+4]	; Get dwNeededSize
	cmp eax,MaxAlloc		; Check this is sensible
	ja bad_modem			; Jump if not
	cmp eax,edi			; Did we allocate enough memory already ?
	jle lineDevCaps_memOK		; Jump if so
	; LineDevCaps was not big enough
	; As we loop back to lineGetDevCaps(), we could in theory get
	; stuck in an infinite loop if dwNeededSize changes.
	cmp edi,256			; Was this the first iteration ?
	jne bad_modem			; Jump if not

	add esp,edi			; Deallocate existing space
	add eax,3			; Round up required size to nearest DWORD
	and eax,0fffffffch		; (this preserves stack alignment)
	jmp resize_lineDevCaps		; Try again

lineDevCaps_memOK:
	; Now we have successfully read the structure, but there may still be
	; problems:  The device name comes from the modem SPI and is at the
	; mercy of the modem manufacturer.  The name may be empty, not have a
	; valid null terminator, be of an unknown format, or suffer from other
	; problems...


	; ** Does modem support Voice Mode ? **
	mov eax,dword ptr [esp+60]	; get dwMediaModes
	and eax,LINEMEDIAMODE_AUTOMATEDVOICE
	jz bad_modem			; Jump

	; Notes:
	; 1) The above test claims my Banksia PC Card (which supports Rockwell voice)
	; and my Lucent Winmodem (which says it is a voice modem) are not voice
	; modems.
	; 2) Could also test for  dwBearerModes & LINEBEARERMODE_VOICE
	; and dwLineFeatures & LINEFEATURE_MAKECALL

	; ** String format ** is at dwStringFormat   (=[esp+40])
	mov ebx,dword ptr [esp+40]	; get string format
	cmp ebx,1			; Is it STRINGFORMAT_ASCII ?
	je dwStringFormat_ok		; Jump if so
	cmp ebx,3			; Is it STRINGFORMAT_UNICODE ?
	jne bad_modem			; If not, don't know what to with it
dwStringFormat_ok:

	; ** String length ** in bytes is at dwLineNameSize   (=[esp+32])
	mov ecx, dword ptr [esp+32]	; ECX = dwLineNameSize
	or ecx,ecx			; Is size zero ?
	jz bad_modem			; If so, jump
	cmp ecx,lTempBuffer		; Will it overflow the buffer ?
	jge bad_modem			; Jump if so
	; String offset is at dwLineNameOffset (=[esp+36])
	mov esi, dword ptr [esp+36]
	add esi,esp			; ESI = LineName
	cmp byte ptr [esi],0		; Does string start with a null ?
	je bad_modem			; If so, jump
	push edi			;
	mov edi, offset temp_buffer	;
	rep movsb			; Copy string to temp buffer
	xor eax,eax			;
	stosd				; Append null terminator
	pop edi				;
	; Add modem name to list box
	pushd offset temp_buffer	; lParam
	xor eax,eax			;
	pushd eax			; wParam
	pushd CB_ADDSTRING		; Msg
	mov al,120			;
	pushd eax			; nIDDlgItem
	pushd hDlg			; hDlg
	; Check string type and call appropriate funtion
	cmp ebx,1			; If string ASCIIZ ?
	je modem_str_ascii		; Jump if so
	call _SendDlgItemMessageW@20	; Call Unicode function

	; WARNING: Not tested with Unicode

	jmp modem_str_uni		;
modem_str_ascii:			;
	call _SendDlgItemMessageA@20	; Call ASCII function
modem_str_uni:				;
	; CB_ADDSTRING returns the listbox index in EAX

	; Was this modem the selected one ?
	; If so, record its listbox index number

	mov ebx, dword ptr [esp+28]	; dwPermanentLineID
	cmp ebx,dwPermanentLineID	;
	jne not_selected_modem		;
	mov DevIndex,eax		;
not_selected_modem:			;

	add esp,edi			; Deallocate memory
	pop esi				; Get device ID

	; Now put the device number in the "extra data" section.
	pushd esi			; lParam = Device Number
	pushd eax			; wParam = Listbox Index
	pushd CB_SETITEMDATA		; Msg
	pushd 120			; nIDDlgItem
	pushd hDlg			; hDlg
	call _SendDlgItemMessageA@20	;

	inc nDevs			; Increment count of devices found
	jmp good_modem			;

bad_modem:				;
	add esp,edi			; Deallocate memory
	pop esi				; Get device ID
good_modem:
	inc esi				; Next device next time			;
	cmp esi, dwNumLineDevs		; Have we tried all devices ?
	jl try_next_modem		; Try next modem next time

	;**** Select the specified modem from this list ****
	; Note that if none of the selections match, we default to
	; the first item in the list box
	xor eax,eax			;
	pushd eax			;lParam
	pushd DevIndex			;wParam
	pushd CB_SETCURSEL		;Msg
	mov al,120			;
	pushd eax			;nIDDlgItem
	pushd hDlg			;hDlg
	call _SendDlgItemMessageA@20	;

	jmp exit_config_true		;

			;
config_browse_vfile:				; ***** Browse Voice Files ******

	pushd MAX_PATH			; Get file name from edit dialog
	pushd offset szNewVoiceFileName	;
	pushd 200			;
	pushd hDlg			;
	call _GetDlgItemTextA@16	;

	; Display standard open file dialogue
	mov OFNFlags, OFN_PATHMUSTEXIST+OFN_FILEMUSTEXIST+OFN_SHOWHELP
	mov lpstrTitle,offset szBrowseVfiles
	mov lpstrFilter,offset vfile_filter
	mov lpstrDefExt,offset szWAV
	mov lpstrFile, offset szNewVoiceFileName
	mov lpstrInitialDir,0
	push offset ofn_struct		;
	call _GetOpenFileNameA@4	;

	or eax,eax			; Error ?
	jz dont_open_vfile		; If so, jump

	; Write new file name into edit box
	pushd offset szNewVoiceFileName	;
	pushd 200			;
	pushd hDlg			;
	call _SetDlgItemTextA@12	; Set file name

dont_open_vfile:			;
	jmp exit_config_true		;

config_browse_rfile:				; ***** Browse Executable Files ******

	pushd MAX_PATH			; Get file name from edit dialog
	pushd offset szRunFileName	;
	pushd 210			;
	pushd hDlg			;
	call _GetDlgItemTextA@16	;

	; Display standard open file dialogue
	mov OFNFlags, OFN_PATHMUSTEXIST+OFN_FILEMUSTEXIST
	mov lpstrTitle,offset szBrowseRfiles
	mov lpstrFilter,offset rfile_filter
	mov lpstrDefExt,offset szExe
	mov lpstrFile, offset szRunFileName
	mov lpstrInitialDir,0
	push offset ofn_struct		;
	call _GetOpenFileNameA@4	;

	or eax,eax			; Error ?
	jz dont_open_rfile		; If so, jump

	; Write new file name into edit box
	pushd offset szRunFileName	;
	pushd 210			;
	pushd hDlg			;
	call _SetDlgItemTextA@12	; Set file name

dont_open_rfile:			;
	jmp exit_config_true		;

config_dlg_help:			; **** [Help] button on open file dialogs ****
	pushd 4				; Help Topic
	pushd HELP_CONTEXT		;
	pushd offset szHelpFileName	;
	pushd hDlg			;
	call _WinHelpA@16		;
	jmp exit_config_true		;


config_play_sound:			; ***** Play Sound Button ****

	; ** Get WAV File Name **
	pushd MAX_PATH			;
	pushd offset szNewVoiceFileName	;
	pushd 200			;
	pushd hDlg			;
	call _GetDlgItemTextA@16	;

	; ** Open this WAV file and read data into memory **
	push offset szNewVoiceFileName	;
	call ReadWAVFile		;
	jc ReadWAVFile_Fail		;

	; ** Display "PlayBack" dialogue **
	xor eax,eax			;
	pushd eax			;
	pushd offset PlayBackProc	;
	pushd hDlg			;
	pushd offset szPlayBack		;
	pushd hInstance			;
	call _DialogBoxParamA@20	;

	jmp exit_config_true		;

ReadWAVFile_Fail:			;
	mov esi,offset szBadWAVFile	;
	call ErrorMessage		;

	jmp exit_config_true		;

config_config_modem:			; ***** Configure Modem Button ****

	; ** Pop up configuration dialog for modem **

	cmp nDevs,0			; Check if there was at least one modem
	jle no_modems_present2		; Jump if not

	; Get index of currently selected modem
	xor eax,eax			;
	pushd eax			; lParam
	pushd eax			; wParam
	pushd CB_GETCURSEL		; Msg
	mov al,120			;
	pushd eax			; nIDDlgItem
	pushd hDlg			; hDlg
	call _SendDlgItemMessageA@20	;
	cmp eax,0			;
	jl no_modems_present2		; Jump if nothing selected

	; Get Item Data (=dwDeviceID of Modem)
	xor ecx,ecx			;
	pushd ecx			; lParam
	pushd eax			; wParam
	pushd CB_GETITEMDATA		; Msg
	mov cl,120			;
	pushd ecx			; nIDDlgItem
	pushd hDlg			; hDlg
	call _SendDlgItemMessageA@20	;
	; dwDeviceID is in EAX

	xor ecx,ecx			;
	push ecx			; lpszDeviceClass=0
	push hDlg			; hwndOwner
	push eax			; dwDeviceID
	call _lineConfigDialog@12	;

no_modems_present2:

	jmp exit_config_true		;


config_ok_button:			; ***** OK Button ******

	; **** User said OK, so check all data, and exit ****

	; **** Check Attempts ****
	xor eax,eax			;
	pushd eax			;
	pushd offset temp_buffer	;
	mov al,106			;
	pushd eax			;
	pushd hDlg			;
	call _GetDlgItemInt@16		;
	cmp dword ptr temp_buffer,0	; Was there an error (non-numeric) ?
	jz number_ret_bad		; If bad, jump
	cmp eax,100			; Is it in the range 1-100 ?
	ja number_ret_bad		; Jump if not
	cmp eax,1			; Is it below 1 ?
	jl number_ret_bad		; Jump if not
	mov NewMax_attempts,eax		; OK, so save

	; **** Check Repeats ****
	xor eax,eax			;
	pushd eax			;
	pushd offset temp_buffer	;
	mov al,107			;
	pushd eax			;
	pushd hDlg			;
	call _GetDlgItemInt@16		;
	cmp dword ptr temp_buffer,0	; Was there an error (non-numeric) ?
	jz number_rep_bad		; If bad, jump
	cmp eax,100			; Is it in the range 1-100 ?
	ja number_rep_bad		; Jump if not
	cmp eax,1			; Is it below 1 ?
	jl number_rep_bad		; Jump if not
	mov NewRepeats,eax		; OK, so save

	; **** Check Inital time Delay ****
	mov esi,offset temp_buffer	;
	mov dword ptr [esi], 0		;
	pushd 30			;
	pushd esi			;
	pushd 110			;
	pushd hDlg			;
	call _GetDlgItemTextA@16	;

	call StoTime			; Convert to time in seconds
	jnc itime_is_ok			; Jump if OK
	mov edi,offset szITimeBad	; Error message
config_bad3:				;
	cmp al,-2			; Days wrong ?
	jne days_not_bad1		;
	mov esi,offset szDaysBad	; Days wrong
	jmp config_bad2			;
days_not_bad1:				;
	cmp al,-3			; Hours wrong ?
	jne hours_not_bad1		;
	mov esi,offset szHoursBad	; Hours wrong
	jmp config_bad2			;
hours_not_bad1:				;
	cmp al,-4			; Minutes wrong ?
	jne minutes_not_bad1		;
	mov esi,offset szMinutesBad	; Minutes wrong
	jmp config_bad2			;
minutes_not_bad1:			;
	cmp al,-5			; Seconds wrong ?
	jne seconds_not_bad1		;
	mov esi,offset szSecondsBad	; Seconds wrong
	jmp config_bad2			;
seconds_not_bad1:			;

	mov esi, offset szFormatBad	; Else format is bad

	jmp config_bad2			;
itime_is_ok:				;

	mov NewICountdownTime,eax		;

	; **** Check Retry time Delay ****
	mov esi,offset temp_buffer	;
	mov byte ptr [esi], 0		;
	pushd 30			;
	pushd esi			;
	pushd 115			;
	pushd hDlg			;
	call _GetDlgItemTextA@16	;
	call StoTime			; Convert to time in seconds
	jnc rtime_is_ok			; Jump if OK
	mov edi,offset szRTimeBad	; Error message
	jmp config_bad3			; (same as for Initial Time Delay)
rtime_is_ok:				;
	mov NewRCountdownTime,eax	;

	; **** Check if WAV file is valid, and read it into memory ****
	; Get name entered
	pushd MAX_PATH			;
	pushd offset szNewVoiceFileName	;
	pushd 200			;
	pushd hDlg			;
	call _GetDlgItemTextA@16	;
	; Open/Check it
	push offset szNewVoiceFileName	;
	call ReadWAVFile		;
	jnc WAVFile_is_OK		;
	mov esi, offset szBadWAVFile	;
	jmp config_bad
WAVFile_is_OK:

	; **** Check Modem ****
	; We must first check if there are actually modems for the user to
	; select from.  We can't castigate the user for not selecting a modem
	; if there isn't one available.
	cmp nDevs,0			; Check if there was at least one modem
	jle no_modems_present1		; Jump if not

	; Get index of currently selected modem
	xor eax,eax			;
	pushd eax			; lParam
	pushd eax			; wParam
	pushd CB_GETCURSEL		; Msg
	mov al,120			;
	pushd eax			; nIDDlgItem
	pushd hDlg			; hDlg
	call _SendDlgItemMessageA@20	;
	cmp eax,0			;
	jge modem_sel_OK		; Jump if something selected
	mov esi, offset szNoModem	; Else nothing selected
	jmp config_bad2
modem_sel_OK:
	; Get Item Data (=Modem ID)
	xor ecx,ecx			;
	pushd ecx			; lParam
	pushd eax			; wParam
	pushd CB_GETITEMDATA		; Msg
	mov cl,120			;
	pushd ecx			; nIDDlgItem
	pushd hDlg			; hDlg
	call _SendDlgItemMessageA@20	;
	mov dwNewDeviceID,eax		; Save
	; Also determine permanent ID
	mov esi,eax			;
	call GetPermanentLineID		;
	jc cant_get_PermanentLineID	; Jump if error
	mov dwNewPermanentLineID,eax	; Save

cant_get_PermanentLineID:		; # what do we do here ? ##

no_modems_present1:

	; **** Data is OK, so save it ****
retry_ini_write:

	; *** Just in case we were running and someone has turned on 
	;     autostart, update the start time *** 
	call record_new_starttime	;

	; *** Save Autostart ***
	push 130			;
	push hDlg			;
	call _IsDlgButtonChecked@8	;
	and eax,1			; Mask extraneous bits
	jz now_not_autostart		;

	; **** Autostart Selected ****
	; If we are setting autostart for the first time, record what 
	; the previous settings were.  
	cmp bAutoStart,0		;
	jnz already_autostart		;

	; *** Get previous settings ***

	; *** Get Previous Autoscan setting ****
	mov esi, offset temp_buffer	;
	pushd offset szMSDOSSYS		; lpFileName
	pushd 128			; nSize
	pushd esi			; lpReturnedString
	pushd offset szOne		; lpDefault
	pushd offset szAutoScan		; lpKeyName
	pushd offset szOptions		; lpAppName
	call _GetPrivateProfileStringA@24; [Options] Autoscan

	; *** Write it to out INI file ***
	mov ecx, offset szAutoScan	;
	mov eax, offset szOptions	;
	call Write_INI			;

already_autostart:

	; ** Make MSDOS.SYS writable **
	push offset szMSDOSSYS		; lpFileName
	call _GetFileAttributesA@4	;
	and eax,0FFFFFFFEH		; Turn off the bit FILE_ATTRIBUTE_READONLY (bit 1)
	push eax			; dwFileAttributes
	push offset szMSDOSSYS		; lpFileName
	call _SetFileAttributesA@8	;

	; ** Turn on Autoscan **
	pushd offset szMSDOSSYS		;
	pushd offset szZero		;
	pushd offset szAutoScan		;
	pushd offset szOptions		;
	call _WritePrivateProfileStringA@16; [Options] Autoscan = 0
	; Don't care if it fails

	; ** If we are already counting, set up registry to auto start **
	cmp bCountingDown,0		;
	jz not_yet_counting		;
	call AddAutoRun			;
not_yet_counting:			;

	mov eax,1			;
	jmp now_autostart		;

now_not_autostart:			;

	; **** Autostart De-Selected ****

	; If we are resetting autostart for the first time, restore the 
	; settings we stored in the INI file
	cmp bAutoStart,0		;
	jz already_not_autostart	;

	; ** Make MSDOS.SYS writable **
	push offset szMSDOSSYS		; lpFileName
	call _GetFileAttributesA@4	;
	and eax,0FFFFFFFEH		; Turn off the bit FILE_ATTRIBUTE_READONLY (bit 1)
	push eax			; dwFileAttributes
	push offset szMSDOSSYS		; lpFileName
	call _SetFileAttributesA@8	;

	; ** Read previous Autoscan State from our INI file **
	mov esi, offset temp_buffer	;
	pushd offset szIniFileName	; lpFileName
	pushd 128			; nSize
	pushd esi			; lpReturnedString
	pushd offset szOne		; lpDefault
	pushd offset szAutoScan		; lpKeyName
	pushd offset szOptions		; lpAppName
	call _GetPrivateProfileStringA@24; [Options] Autoscan

	; ** Restore Autoscan State **
	pushd offset szMSDOSSYS		;
	pushd esi			;
	pushd offset szAutoScan		;
	pushd offset szOptions		;
	call _WritePrivateProfileStringA@16; [Options] Autoscan = 0 or 1
	; Don't care if it fails

already_not_autostart:			;
	call RemoveAutoRun		; Remove any auto-run option in registry

	xor eax,eax			;

now_autostart:				;
	mov bAutoStart,al		; Set New Autostart Status
					; And write it to INI file
	add al,"0"			; AX = string "1",0
	mov dword ptr temp_buffer, eax	; Write to buffer
	mov ecx, offset szAutoStart	;
	mov eax, offset szSettings	;
	call Write_INI			; [Settings] Autostart


	; *** Save Autostop ***
	push 131			;
	push hDlg			;
	call _IsDlgButtonChecked@8	;
	and eax,1			; Mask extraneous bits
	cmp al,bAutoStop		;
	je autostop_same		;
	mov bAutoStop,al		;
	add al,"0"			; AX = string "0",0
	mov dword ptr temp_buffer, eax	; Write to buffer
	mov ecx, offset szAutoStop	;
	mov eax, offset szSettings	;
	call Write_INI			; [Settings] Autostop
	or eax,eax			;
	jz config_ini_write_fail	;
autostop_same:

	;** Save Debug status **
	push 132			;
	push hDlg			;
	call _IsDlgButtonChecked@8	;
	and eax,1			; Mask extraneous bits
	cmp al,bDebugToFile		;
	je debug_same			;
	mov bDebugToFile,al		;
	add al,"0"			; AX = string "0",0
	mov dword ptr temp_buffer, eax	; Write to buffer
	mov ecx, offset szDebug		;
	mov eax, offset szSettings	;
	call Write_INI			; Write to INI file
	or eax,eax			; Did we write OK ?
	jz config_ini_write_fail	;
debug_same:

	; *** Voice file to play ***
	; ## check for change ?
	mov ecx,255				;
	mov esi, offset szNewVoiceFileName	;
	mov edi, offset szVoiceFileName		;
CopyVoiceFileName:			;
	lodsb				;
	stosb				;
	or al,al			;
	jz VoiceFileNameCopied		;
	loop CopyVoiceFileName		;
	xor al,al			;
	stosb				; Append null if there isn't one
VoiceFileNameCopied:			;
	pushd offset szIniFileName	;
	pushd offset szVoiceFileName	;
	pushd offset szPlay		;
	pushd offset szSettings		;
	call _WritePrivateProfileStringA@16	; [Settings] Play
	or eax,eax			;
	jz config_ini_write_fail	;

	; *** File to run (if any) ***
	pushd MAX_PATH			;
	pushd offset szRunFileName	;
	pushd 210			;
	pushd hDlg			;
	call _GetDlgItemTextA@16	;
	; ## check for change ?		;
	pushd offset szIniFileName	;
	pushd offset szRunFileName	;
	pushd offset szRun		;
	pushd offset szSettings		;
	call _WritePrivateProfileStringA@16	; [Settings] Run
	or eax,eax			;
	jz config_ini_write_fail	;

	; ** Save Number to Dial **
	; ### check length, format, change ###
	pushd 32			;
	pushd offset szNumber2Dial	;
	pushd 105			;
	pushd hDlg			;
	call _GetDlgItemTextA@16	;
	; #### szNewNumber2Dial #####
	pushd offset szIniFileName	;
	pushd offset szNumber2Dial	;
	pushd offset szNumber		;
	pushd offset szSettings	;
	call _WritePrivateProfileStringA@16	; [Settings] Number
	or eax,eax			;
	jz config_ini_write_fail	;

	; ** Attempts **
	mov eax,NewMax_attempts		; Look at attempts
	cmp eax,Max_attempts		; Has it changed ?
	je attempts_same		; If not, jump
	mov Max_attempts,eax		; If so, save
	mov edi,offset temp_buffer	; Then write to INI file
	call itoa			; Convert to ASCIIZ
	mov ecx,offset szAttempts	;
	mov eax,offset szSettings	;
	call Write_INI			; [Settings] Attempts
	or eax,eax			;
	jz config_ini_write_fail	;
attempts_same:				;

	; ** Repeats **
	mov eax,NewRepeats		; Look at repeats
	cmp eax,dwRepeats		; Has it changed ?
	je repeats_same			; If not, jump
	mov dwRepeats,eax		; If so, save
	mov edi,offset temp_buffer	; Then write to INI file
	call itoa			; Convert to ASCIIZ
	mov ecx, offset szRepeats	;
	mov eax, offset szSettings	;
	call Write_INI			; [Settings] Repeats
	or eax,eax			;
	jz config_ini_write_fail	;
repeats_same:				;

	; ** Initial Time delay **
	mov eax,NewICountdownTime	;
	mov dword ptr [ICountdownTime], eax;
	call TimetoS			;
	mov ecx, offset szInitialDelay	;
	mov eax, offset szSettings	;
	call Write_INI			; [Settings] Initial Delay
	or eax,eax			;
	jz config_ini_write_fail	;
init_time_bad:				;

	; ** Retry time delay **
	mov eax,NewRCountdownTime	;
	mov dword ptr [RCountdownTime], eax;
	call TimetoS			;
	mov ecx, offset szRetryDelay	;
	mov eax, offset szSettings	;
	call Write_INI			; [Settings] Retry Delay
	or eax,eax			;
	jz config_ini_write_fail	;
retry_time_bad:				;

	; ** Modem **
	cmp nDevs,0			; Check if there was at least one modem
	jg modems_present		; Jump if so

	; ** No modems were present **
	;  This is an error condition which prevents us from marking the
	; config as "OK".  Unlike the other errors above the user can't do
        ; anything about it, so we need to permit exit from the dialog while
	; marking the config as invalid
	mov esi,offset szNoModems	;
	call ErrorMessage		;

	mov al,0			; Sets "ConfigValid" flag
	jmp no_modems_present		;

modems_present:				;
	mov eax,dwNewPermanentLineID	; Look at Permanent Device ID
	cmp eax,dwPermanentLineID	; Has it changed ?
	je DevNo_same			; If not, jump
	mov dwPermanentLineID,eax	; If so, save
	mov edi,offset temp_buffer	; Then write to INI file
	call itoa			; Convert to ASCIIZ
	mov ecx, offset szModem		;
	mov eax, offset szSettings	;
	call Write_INI			; [Settings] Modem
	or eax,eax			;
	jz config_ini_write_fail	;
DevNo_same:				;

	mov eax,dwNewDeviceID		;
	mov dwNewDeviceID,eax		;

	; *** Successfully entered config ***

	; ** Initial Config Done **
	cmp bInitialConfig,0		; Were we already configured ?
	jnz config_was_done		; Jump if so
	mov bInitialConfig,1		; We are now configured
	mov dword ptr [temp_buffer],"1"	; Write this to INI File
	mov ecx,offset szConfigured	;
	mov eax,offset szSettings	;
	call Write_INI			; [Settings] Config
	or eax,eax			;
	jz config_ini_write_fail	;
config_was_done:			;

	mov al,1			;
no_modems_present:
	mov bConfigValid,al		;

	jmp end_config_dialog		;

config_ini_write_fail:			;
	; **** Failed to write INI file - retry ? ****

	pushd MB_ICONEXCLAMATION + MB_RETRYCANCEL
	pushd offset szAppTitle		;
	pushd offset szNoIniFile	;
	pushd hDlg			;
	call _MessageBoxA@16		;
	cmp eax,IDRETRY			; Does user want to retry ?
	je retry_ini_write		; If so, jump back

	jmp end_config_dialog		;

number_ret_bad:				;
	mov esi,offset szAttBad		;
	jmp config_bad			;
number_rep_bad:				;
	mov esi,offset szRepBad		;
	jmp config_bad			;
new_port_bad:				;
	mov esi,offset szModemBad	;
config_bad:				;
	mov edi, offset szAppTitle	;
config_bad2:				;

	pushd MB_ICONEXCLAMATION
	pushd edi			;
	pushd esi			;
	pushd hDlg			;
	call _MessageBoxA@16		;

	jmp exit_config_true		;

config_cancel_button:			; ***** CANCEL buttton ******
end_config_dialog:			;

	mov esi,offset temp_buffer	;
	mov edi,offset xcoord_config	;
	pushd esi			;
	pushd hDlg			;
	call _GetWindowRect@8		; Get bounding rectangle
	movsd				; Transfer X coord
	movsd				; Transfer Y coord

	xor eax,eax			;
	pushd eax			;
	pushd hDlg			;
	call _EndDialog@8		;
	jmp end_configproc		;


config_help_button:			; ***** HELP button *****
	pushd 3				; Help Topic 3 = Config
	pushd HELP_CONTEXT		;
	pushd offset szHelpFileName	;
	pushd hDlg			;
	call _WinHelpA@16		;
;;	jmp exit_config_true		;

exit_config_true:			;
	mov eax,1			;
end_configproc:				;
	pop edi				;
	pop esi				;
	pop ebx				;
	ret 16				;
ConfigProc ENDP


;****************************************************************************
; Name: RecordProc
; Purpose: Handles commands for "Record Message" pop-up.
;	[esp+4] = hDlg
;	[esp+8] = iMessage
;	[esp+12] = wParam
;	[esp+16] = lParam
; Note:
;****************************************************************************
RecordProc PROC
	push ebx			;
	push esi			;
	push edi			;
	mov eax,[esp+8+12]		; iMessage
	cmp eax,WM_INITDIALOG		;
	je record_init			; Initialise
	cmp eax,WM_TIMER		;
	je record_wm_timer		; Timer tick while recording or playing
	cmp eax,MM_WIM_DATA		;
	je record_end_data		; Finished recording data
	cmp eax,WOM_DONE		;
	je record_end_preview		; Finished previewing data
	cmp eax,help_message		;
	je record_dlg_help		; [Help] button on "open file" dialogues
	cmp eax,WM_COMMAND		;
	jne exit_record_false		;

	;  ***** Command *****
	mov eax,[esp+12+12]		; wParam
	cmp eax,IDCANCEL		; [Cancel] Button
	je record_cancel_button		;
	cmp eax,IDOK			; [Write to File] / OK button
	je record_ok_button		;
	cmp eax,130			; [Record] button
	je record_button		;
	cmp eax,132			; [Preview] button
	je record_Preview_button	;
	cmp eax,133			; [Stop] button
	je record_Stop_button		;
	cmp eax,140			; [Help] button
	je record_Help_button		;
	; Otherwise Unknown command

exit_record_false:			;
	xor eax,eax			;
	jmp end_recordproc		;

record_init: ; ***** Initialise Dialog *****

	mov ebx,dword ptr [esp+4+12]	; hDlg
	mov hDlg,ebx			;
	mov hwndOwner,ebx		; Set owner of Open/SaveAs File standard dialogues

	pushd ycoord_record		; Resposition Dialogue Box
	pushd xcoord_record		;
	call PositionDialog		;

	; ** Create Progress Bar **
	xor eax,eax			;
	pushd eax			; lpParam
	pushd hInstance			; hInstance
   	pushd 20            		; hmenu/child window identifier
    	pushd ebx            		; parent hwnd

	; ** Find how big the dialogue box units are **
	; We need to do this because the progress bar is in units of Pixels
	pushd 32			; Bottom
	pushd 200			; Right
	pushd 20			; Top
	pushd 2				; Left

	pushd esp			; lpRect
	pushd ebx			; hDlg
	call _MapDialogRect@8		;

	; The corrected RECT structure is on the stack
	; Convert it to the right format for CreateWindowEx()

	mov eax,dword ptr [esp]		; Left
	inc eax				;
	sub dword ptr [esp+8],eax	; Width  = Left-Right+1
	mov eax,dword ptr [esp+4]	; Top
	inc eax				;
	sub dword ptr [esp+12],eax	; Height  = Bottom-Top+1

;	pushd 				; hHeight
;	pushd 				; hWidth
;	pushd 				; y
;	pushd 				; x

	xor eax,eax			;
	pushd WS_VISIBLE + WS_CHILD	;
	pushd eax			;
	pushd offset szProgressClassA	;
	pushd eax			; Extended Style
	call _CreateWindowExA@48	;
	mov hWndProgressBar,eax		; Save window handle

	; ** Get handles to all buttons **
	pushd 130			; nIDDlgIte
	pushd ebx			; hDlg
	call _GetDlgItem@8		;
	mov hButtonRecord,eax		;
	pushd 132			; nIDDlgIte
	pushd ebx			; hDlg
	call _GetDlgItem@8		;
	mov hButtonPreview,eax		;
	pushd 133			; nIDDlgIte
	pushd ebx			; hDlg
	call _GetDlgItem@8		;
	mov hButtonStop,eax		;
	pushd IDOK			; nIDDlgIte
	pushd ebx			; hDlg
	call _GetDlgItem@8		;
	mov hButtonWrite,eax		;
	pushd IDCANCEL			; nIDDlgIte
	pushd ebx			; hDlg
	call _GetDlgItem@8		;
	mov hButtonFinished,eax		;

	; **** Open default wave input device ****

	xor eax,eax			;

	mov dwBytesRecorded,eax		; Nothing recorded yet

	pushd CALLBACK_WINDOW    	; fdwOpen
	pushd eax			; dwCallbackInstance
	pushd ebx 			; dwCallback
	pushd offset WaveFormat		; pwfx
	pushd WAVE_MAPPER		; uDeviceID
	pushd offset hWavInDevRec	; phwi
	call _waveInOpen@24		;
	or eax,eax			;
	jz record_waveInOpen_OK		;

	;** Error Message: Can't open wave input device **
	mov esi,offset szNoWaveIn	;
	call ErrorMessage		;

	jmp record_waveInOpen_fail	; Jump if error
record_waveInOpen_OK:			;

	; ** Allocate Heap Memory for Wave Input Buffer **
	xor eax,eax			;
	pushd SizeOfWaveInBufferRec	; dwBytes
	pushd eax			; dwFlags
	pushd hProcessHeap		; hHeap
	call _HeapAlloc@12		;
	or eax,eax			;
	jnz record_HeapAlloc_OK		;

	;** Error Message: Can't allocate memory **
	mov esi,offset szNoMemory	;
	call ErrorMessage		;

	jmp record_malloc_fail		;
record_HeapAlloc_OK:			;
	mov WaveInHeaderRec,eax		; Save pointer to heap memory

	; ** Prepare Header **
	pushd 32			; cbwh
	pushd offset WaveInHeaderRec	; pwh
	pushd hWavInDevRec		; hwi
	call _waveInPrepareHeader@12	;
	or eax,eax			;
	jz record_waveInPrepareHeader_OK	;

	;** Error Message: Can't open wave input device **
	mov esi,offset szNoWaveIn	;
	call ErrorMessage		;
	jmp record_waveInPrepareHeader_fail	; Jump if error
record_waveInPrepareHeader_OK:		;

	; ** Open WAV device for output **
	xor eax,eax			;
	pushd CALLBACK_WINDOW		; fdwOpen
	pushd eax			; dwCallbackInstance
	pushd ebx			; dwCallback
	pushd offset WaveFormat		; pwfx
	pushd WAVE_MAPPER		; uDeviceID
	pushd offset hWavOutDevPlay	; phwo
	call _waveOutOpen@24		;
        or eax,eax			;
	jz record_WAVOpen_OK		;

	;** Error Message: Can't open wave output device **
	mov esi,offset szNoWaveOut	;
	call ErrorMessage		;

	jmp record_WAVOpen_fail		;
record_WAVOpen_OK:			;

	mov tRecording,0		; Idle at this stage

	jmp exit_record_true		;

record_dlg_help:			; **** [Help] button on open file dialogs ****
record_Help_button:			; **** [Help] button ****
	pushd 7				; Help Topic (7=Record)
	pushd HELP_CONTEXT		;
	pushd offset szHelpFileName	;
	pushd hWnd			;
	call _WinHelpA@16		;
	jmp exit_record_true		;

record_ok_button:			; ***** Write To File Button ******

	; ** Was anything actually recorded ? **
	mov eax,dwBytesRecorded		;
	or eax,eax			;
	jnz record_something_recorded	;
	mov esi,offset szNothingRecorded;
	call ErrorMessage		; ** Print error message **
	jmp exit_record_true		;
record_something_recorded:		;

	; **** Get User to enter a File Name ****
	sub esp,MAX_PATH			; Allocate space for file name
	mov esi,esp			;
	mov dword ptr [esi],0		; File name initially null

	; ** Display standard "Save As" dialogue **
	mov OFNFlags, OFN_PATHMUSTEXIST+OFN_OVERWRITEPROMPT+OFN_SHOWHELP+OFN_HIDEREADONLY
	mov lpstrTitle,offset szBrowseVfiles
	mov lpstrFilter,offset vfile_filter
	mov lpstrDefExt,offset szWAV
	mov lpstrFile, esi
	mov lpstrInitialDir,offset szWaveDirName
	push offset ofn_struct		;
	call _GetSaveFileNameA@4	;
	or eax,eax			; File name entered ?
	jnz record_FileNameEntered	; If so, jump
	; User has cancelled
	add esp,MAX_PATH		;  Clean up stack
	jmp exit_record_true		;

record_FileNameEntered:			;

	; ** Update the directory we were using **
	pushd offset szWaveDirName	; lpBuffer
	pushd MAX_PATH			; nBufferLength
	call _GetCurrentDirectoryA@8	;

	;** Open File **
	xor eax,eax			;
	pushd MMIO_CREATE + MMIO_READWRITE + MMIO_ALLOCBUF; dwOpenFlags
	pushd eax			; lpmmioinfo
	pushd esi			; szFilename
	call _mmioOpenA@12		;
	add esp,MAX_PATH		; Finished with file name
	or eax,eax			;
	jnz record_mmioOpenA_OK		;

	mov esi,offset szCantCreate	;
	call ErrorMessage		; ** Print error message **
	jmp exit_record_true		;

record_mmioOpenA_OK:			;
	mov edi,eax			; hmmio

	sub esp,40			; Allocate space for 2xMMCKINFO Structures
;	; MMCKINFO Structure
;	dd ?				; ckid
;	dd ?				; cksize
;	dd ?				; fccType
;	dd ?				; dwDataOffset
;	dd ?				; dwFlags

	; ** Create RIFF WAVE chunk **
	xor eax,eax			;
	mov dword ptr [esp+4], eax	; cksize=0
	mov eax,esp			;
	mov dword ptr [esp+8], "EVAW"	; fccType="WAVE"
	pushd MMIO_CREATERIFF 		; wFlags
	pushd eax			; lpck
	pushd edi			; hmmio
	call _mmioCreateChunk@12	;
	or eax,eax			;
	jnz record_CreateChunk1_fail	;

	; ** Create "fmt " chunk **
	lea ebx,dword ptr [esp+20]	;
	mov dword ptr [ebx], " tmf"	; ckid="fmt "
	; EAX=0
	pushd eax			; wFlags
	pushd ebx			; lpck
	pushd edi			; hmmio
	call _mmioCreateChunk@12	;
	or eax,eax			;
	jnz record_CreateChunk2_fail	;

	; ** Write format data to "fmt " chunk **
	pushd cSizeOfWaveFormatEx	; cch
	pushd offset WaveFormat		; pch
	pushd edi			; hmmio
	call _mmioWrite@12		;
	cmp eax,cSizeOfWaveFormatEx	;
	jne record_write1_fail		;

	; ** Ascend out of fmt chunk **
	xor eax,eax			;
	pushd eax			; wFlags
	pushd ebx			; lpck
	pushd edi			; hmmio
	call _mmioAscend@12		;
	or eax,eax			;
	jnz record_Ascend1_fail		;

	; ** Create "data" chunk **
	mov dword ptr [esp+20], "atad"	; ckid="data"
	; EAX=0
	pushd eax	 		; wFlags
	pushd ebx			; lpck
	pushd edi			; hmmio
	call _mmioCreateChunk@12	;
	or eax,eax			;
	jnz record_CreateChunk3_fail	;

	; ** Write data to "data" chunk **
	pushd dwBytesRecorded		; cch
	pushd WaveInHeaderRec		; pch
	pushd edi			; hmmio
	call _mmioWrite@12		;
	cmp eax,dwBytesRecorded		;
	jne record_write1_fail

	; ** Mark end of "data" chunk **
	xor eax,eax			;
	pushd eax			; wFlags
	pushd ebx			; lpck
	pushd edi			; hmmio
	call _mmioAscend@12		;
	or eax,eax			;
	jnz record_Ascend2_fail		;

	; ** Mark end of "WAVE" chunk **
	mov ebx,esp			;
	; EAX=0
	pushd eax			; wFlags
	pushd ebx			; lpck
	pushd edi			; hmmio
	call _mmioAscend@12		;
	or eax,eax			;
;;	jnz record_Ascend3_fail		;
;;
;;record_Ascend3_fail:			;
record_Ascend2_fail:			;
record_CreateChunk3_fail:		;
record_Ascend1_fail:			;
record_write1_fail:			;
record_CreateChunk2_fail:		;
record_CreateChunk1_fail:		;
	; Note: NZ flag is set here on error

	jz record_no_error		; Jump if there is no problem
	mov esi,offset szErrorCreating	; "Error creating WAV File"
	call ErrorMessage		; ** Print error message **
record_no_error:			;

	add esp,40			; Deallocate MMCKINFO Structure

	; ** Close File **
	xor eax,eax			;
	pushd eax			; wFlags
	pushd edi			; hmmio
	call _mmioClose@8		;

	jmp exit_record_true		;

record_button:				; ***** Record Button ******

	pushd 32			; cbwh
	pushd offset WaveInHeaderRec	; pwh
	pushd hWavInDevRec		; hwi
	call _waveInAddBuffer@12	;
	or eax,eax			;
	jnz exit_record_true		;

	pushd hWavInDevRec		; hwi
	call _waveInStart@4		;
	or eax,eax			;
	jnz exit_record_true		;

	mov tRecording,1		; We are now recording

	;** Update Status **
	pushd offset szRecording	;
record_preview_exit:			; *** Jumped to from "Preview below" ***
	pushd 129			;
	pushd hDlg			;
	call _SetDlgItemTextA@12	; Status: Recording

	; ** Reset  Progress Bar **
	xor eax,eax			;
	pushd eax			; lParam
	pushd eax			; wParam 0% done
	pushd PBM_SETPOS		; Msg
	pushd hWndProgressBar		; hWnd
	call _SendMessageA@16		;

	; ** Disable "Preview", "Record", "Write to file" and "Finished" Buttons **
	xor ebx,ebx			;
	pushd ebx			; bEnable
	pushd hButtonPreview		; hWnd
	call _EnableWindow@8		;
	pushd ebx			; bEnable
	pushd hButtonRecord		; hWnd
	call _EnableWindow@8		;
	pushd ebx			; bEnable
	pushd hButtonWrite		; hWnd
	call _EnableWindow@8		;
	pushd ebx			; bEnable
	pushd hButtonFinished		; hWnd
	call _EnableWindow@8		;

	; ** Enable "Stop" button **
	inc ebx				;
	pushd ebx			; bEnable =1
	pushd hButtonStop		; hWnd
	call _EnableWindow@8		;

	; ** Enable timer to update progress **
	xor eax,eax			;
	pushd eax			; lpTimerFunc
	pushd 500			; uElapse = 1/2 Sec
	mov al,3			;
	pushd eax			; nIDEvent
	pushd hDlg			; hWnd
	call _SetTimer@16		;
	; If this fails, we don't care, progress just won't get updated

	jmp exit_record_true		;

record_Preview_button:			; ***** Preview Button ******

	; ** Set up header **
	mov eax,dwBytesRecorded		; How much was recorded
	mov WavePreviewHdr_dwBufferLength,eax	; This is how much to play

	mov eax,WaveInHeaderRec		; Pointer to recorded data
	mov WavePreviewHdr,eax		; Is also pointer to play data

	; ** Prepare Wave Header **
	pushd 32			; cbwh = SizeOf(WaveOutHdr)
	pushd offset WavePreviewHdr	; pwh
	pushd hWavOutDevPlay		; hwo
	call _waveOutPrepareHeader@12	;
        or eax,eax			;
	jnz record_WAVPrepareHeader_fail;

	; ** Start Speaking Message **
	pushd 32			; cbwh
	pushd offset WavePreviewHdr	; pwh
	pushd hWavOutDevPlay		; hwo
	call _waveOutWrite@12		;
        or eax,eax			;
	jnz record_WAVWrite_fail	;


	mov tRecording,-1		; We are now previewing

	;** Update Status **
	pushd offset szPlaying		;

	jmp record_preview_exit		; Jump to common code in "Record" above

record_WAVWrite_fail:			;

	; ** Unprepare Wave Header **
	pushd 32			; cbwh = SizeOf(WaveOutHdr)
	pushd offset WavePreviewHdr	; pwh
	pushd hWavOutDevPlay		; hwo
	call _waveOutUnprepareHeader@12	;

record_WAVPrepareHeader_fail:		;

	jmp exit_record_true		;

record_wm_timer:			; ***** Timer Tick while recording/playing ****

	call RecordUpdateProgressBar	;

	jmp exit_record_true		;

record_Stop_button:			; ***** Stop Button ******

	call RecordUpdateProgressBar	; Update where we stopped

	; ** Stop recording/playing
	mov al,tRecording		; What were we doing ?
	or al,al			;
	jz record_idle			; Jump if not doing anything
	cmp al,1			;
	je record_recording		; Jump if recording
	; ** Else we were previewing/playing **

	; ** Stop playing **
	pushd hWavOutDevPlay		; hwo
	call _waveOutReset@4		;
	jmp record_Stop_End		;
record_recording:
	; ** Stop recording **
	pushd hWavInDevRec		; hwi
	call _waveInReset@4		;

record_Stop_End:			;
	mov tRecording,0		; Now idle (but waiting for WOM_DONE or MM_WIM_DATA)

record_idle:				;
	jmp exit_record_true		;

record_end_preview:			;  ***** Message: WOM_DONE *****
	; This message is sent when previewing is finished

	call RecordUpdateProgressBar	;

	; ** Reset position **
	pushd hWavOutDevPlay		; hwo
	call _waveOutReset@4		;

	; ** Unprepare Wave Output Header **
	pushd 32			; cbwh = SizeOf(WaveOutHdr)
	pushd offset WavePreviewHdr	; pwh
	pushd hWavOutDevPlay		; hwo
	call _waveOutUnprepareHeader@12	;

	jmp record_end_data2		;

record_end_data:			; ***** Message: MM_WIM_DATA *****
	; This message is sent when recording is finished

	call RecordUpdateProgressBar	;

	; ** Reset position **
	pushd hWavInDevRec		; hwi
	call _waveInReset@4		;

record_end_data2:
	; ** Finished with timer **
	pushd 3				; uIDEvent
	pushd hDlg			; hWnd
	call _KillTimer@8		;

	;** Update Status **
	pushd offset szDone		;
	pushd 129			;
	pushd hDlg			;
	call _SetDlgItemTextA@12	; Status: Done

	mov tRecording,0		; Now idle

	; ** Disable "Stop"Button **
	xor ebx,ebx			;
	pushd ebx			; bEnable
	pushd hButtonStop		; hWnd
	call _EnableWindow@8		;

	; ** Enable "Finished", "Record", Write to File" and "Preview" buttons **
	inc ebx				;
	pushd ebx			; bEnable =1
	pushd hButtonFinished		; hWnd
	call _EnableWindow@8		;
	pushd ebx			; bEnable =1
	pushd hButtonRecord		; hWnd
	call _EnableWindow@8		;
	pushd ebx			; bEnable
	pushd hButtonWrite		; hWnd
	call _EnableWindow@8		;
	pushd ebx			; bEnable
	pushd hButtonPreview		; hWnd
	call _EnableWindow@8		;

	jmp exit_record_true		;


record_cancel_button:			; ***** Finished / Cancel buttton ******

end_record_dialog:			;

	; ** Close WAV output device **
	pushd hWavOutDevPlay		; hwo
	call _waveOutClose@4		;

record_WAVOpen_fail:			;

	; ** Unprepare Wave Input Header **
	pushd 32			; cbwh
	pushd offset WaveInHeaderRec	; pwh
	pushd hWavInDevRec		; hwi
	call _waveInUnprepareHeader@12	;

record_waveInPrepareHeader_fail:	;

	; ** Deallocate heap **
	xor eax,eax			;
	pushd WaveInHeaderRec		; lpMem
	pushd eax			; dwFlags
	pushd hProcessHeap		; hHeap
	call _HeapFree@12		; Free up heap for GUIDs

record_malloc_fail:			;

	; ** Close Wave Input device **
	pushd hWavInDevRec		; hwi
	call _waveInClose@4		;


record_waveInOpen_fail:			;
	mov esi,offset temp_buffer	;
	mov edi,offset xcoord_record	;
	pushd esi			;
	pushd hDlg			;
	call _GetWindowRect@8		; Get bounding rectangle
	movsd				; Transfer X coord
	movsd				; Transfer Y coord

	; ** Remove progress bar **
	; (May not be strictly necessary)
	pushd hWndProgressBar		; hWnd
	call _DestroyWindow@4		;

	xor eax,eax			;
	pushd eax			;
	pushd hDlg			;
	call _EndDialog@8		;
	jmp end_recordproc		;

exit_record_true:			;
	mov eax,1			;
end_recordproc:				;
	pop edi				;
	pop esi				;
	pop ebx				;
	ret 16				;
RecordProc ENDP

;****************************************************************************
; Name: RecordUpdateProgressBar
; Purpose: Updates progress bar for Record Procedure
; No parameters or return values
;****************************************************************************
RecordUpdateProgressBar PROC

	cmp tRecording,0		; Are we idle ?
	jz RecordProgBarEnd		; If so, finished (can't do anything)

	; ** Allocate MMTIME structure on stack **
	mov eax,12			;
	sub esp,eax			;
	mov ecx,esp			; pmmt
	mov dword ptr [ecx],4		; pmmt.wType = TIME_BYTES
	pushd eax			; cbmmt (size of MMTIME structure)
	pushd ecx			; pmmt

	; ** Call correct "GetPosition" function according to what we are doing **
	cmp tRecording,1		; Are we recording ?
	je record_record_tick		; Jump if so

	; ** We are playing back  **
	pushd hWavOutDevPlay		; hwo
	call _waveOutGetPosition@12	;
	mov ecx,WavePreviewHdr_dwBufferLength	;
	jmp record_play_tick		;

record_record_tick:
	; ** We are recording  **
	pushd hWavInDevRec		; hwi
	call _waveInGetPosition@12	;
	mov ecx,SizeOfWaveInBufferRec	;

record_play_tick:			;
	pop edx				; pmmt.wType
	pop ebx				; pmmt.cb
	add esp,4			; Deallocate rest of MMTIME structure
	or eax,eax			; Error code
	jnz RecordProgBarEnd		;
	cmp edx,4			; Check pmmt.wType = TIME_BYTES
	jne RecordProgBarEnd		; Jump if not

	; ** Determine % done **
	; ECX = total bytes		;
	; EBX = bytes played		;
	mov al,100			;
	mul ebx				;
	div ecx				; EAX = % done

	; ** Update Progress Bar **
	xor ecx,ecx			;
	pushd ecx			; lParam
	pushd eax			; wParam
	pushd PBM_SETPOS		; Msg
	pushd hWndProgressBar		; hWnd
	call _SendMessageA@16		;

RecordProgBarEnd:			;
	ret
RecordUpdateProgressBar ENDP

;****************************************************************************
; Name: SynthProc
; Purpose: Handles commands for "Synthesise Message" pop-up.
;	[esp+4] = hDlg
;	[esp+8] = iMessage
;	[esp+12] = wParam
;	[esp+16] = lParam
; Note:
;****************************************************************************
SynthProc PROC

	push ebx			;
	push esi			;
	push edi			;
	mov eax,[esp+8+12]		; iMessage
	cmp eax,WM_INITDIALOG		;
	je Synth_init			;
	cmp eax,help_message		;
	je Synth_dlg_help		; [Help] button on "open file" dialogues
	cmp eax,WM_COMMAND		;
	jne exit_Synth_false		;

	;  ***** Command *****
	mov eax,[esp+12+12]		; wParam
	cmp eax,IDCANCEL		; [Cancel] Button
	je Synth_cancel_button		;
	cmp eax,IDOK			; [Write to File] / OK button
	je Synth_ok_button		;
	cmp eax,131			; [Preview] button
	je Synth_Preview_button		;
	cmp eax,132			; [Stop Playing] button
	je Synth_Stop_button		;
	cmp eax,140			; [Help] button
	je Synth_Help_button		;
	; Otherwise Unknown command

exit_Synth_false:			;
	xor eax,eax			;
	jmp end_Synthproc		;

Synth_init: ; ***** Initialise Dialog *****


	mov ebx,dword ptr [esp+4+12]	; hDlg
	mov hDlg,ebx			;
	mov hwndOwner,ebx		; Set owner of Open/SaveAs File standard dialogues

	pushd ycoord_Synth		; Resposition Dialogue Box
	pushd xcoord_Synth		;
	call PositionDialog		;

	; ** Get handles to all buttons **
	pushd 131			; nIDDlgIte
	pushd ebx			; hDlg
	call _GetDlgItem@8		;
	mov hButtonPreview,eax		;
	pushd 132			; nIDDlgIte
	pushd ebx			; hDlg
	call _GetDlgItem@8		;
	mov hButtonStop,eax		;
	pushd IDOK			; nIDDlgIte
	pushd ebx			; hDlg
	call _GetDlgItem@8		;
	mov hButtonWrite,eax		;
	pushd IDCANCEL			; nIDDlgIte
	pushd ebx			; hDlg
	call _GetDlgItem@8		;
	mov hButtonFinished,eax		;

	; ** Clear all interface pointers **
	mov ecx,3			; Number of interfaces
	mov edi, offset pITTSFind	; Point to first
	xor eax,eax			;
	rep stosd			;

	; ** Initialize COM **
	; The COM Interface is used by the Speech API
  	pushd COINIT_APARTMENTTHREADED	; dwCoInit
	xor eax,eax			;
	pushd eax			; void * pvReserved (must be NULL)
	call _CoInitializeEx@8		;
	or eax,eax			;
	jz ComInitOK			;
	mov esi,offset szNoCOM		;
	call ErrorMessage		;
	jmp SynthInitFail1		;
ComInitOK:

	; ** Access TTS Voice Enumerator **
	pushd eax			; pITTSEnum is on stack (not a parameter)
	pushd esp			; &pITTSEnum
	pushd offset IID_ITTSEnumA	;
	pushd CLSCTX_ALL		;
	xor eax,eax			;
	pushd eax			;
	pushd offset CLSID_TTSEnumerator;
	call _CoCreateInstance@20	;
	pop esi				; Get pITTSEnum in ESI
	or eax,eax			; Check return code
	jz TTSEnumOK			; Jump if OK

	mov esi,offset szNoTTS		; "Can't open text-to-speech engine"
SynthInitFailMessage:			;
	call ErrorMessage		;

	jmp SynthInitFail2		; Jump if error
TTSEnumOK:

	; ** Put all available voices into dialog box **
	sub esp,1432			; Allocate space for TTSMODEINFOA Structure
	mov dwnVoices,0			; Count of voices so far
	mov ebx,esp			; Pointer to TTSInfo
NextVoice:				;
	; ** Get Voice **
	xor eax,eax			;
	pushd eax			; 0
	pushd ebx	 		; TTSInfo
	inc eax				;
	pushd eax			; 1
	pushd esi			;
	mov edi,dword ptr [esi]		;
	call dword ptr [edi+12]		; pITTSEnum->Next()
	or eax,eax			;
	jnz NoMoreVoices		;

	; ** Add Voice name to list box **
	lea eax,dword ptr [ebx+556]	; TTSInfo.szModeName
	pushd eax	 		; lParam
	xor eax,eax			;
	pushd eax			; wParam
	pushd CB_ADDSTRING		; Msg
	mov al,120			;
	pushd eax			; nIDDlgItem
	pushd hDlg			; hDlg
	call _SendDlgItemMessageA@20	;

	inc dwnVoices			; One more voice

	; ** Store GUID of voice temporarily on stack **
	mov eax,esi			; Save ESI
	lea esi,dword ptr [ebx+540]	; TTSInfo.gModeID
	sub esp,16			; Make room on stack
	mov edi,esp			;
	movsd				; Move GUID for this voice onto stack
	movsd				;
	movsd				;
	movsd				;
	mov esi,eax			; Restore ESI

	jmp NextVoice			;
NoMoreVoices:				;
	; Have found all of the voices available

	; ** Check there are some voices available **
	cmp dwnVoices,0			;
	jg SomeVoices			; Jump if at least one voice available
	mov esi, offset szNoVoices	; "Text-to-speech engine has no voices available"
	jmp SynthInitFailMessage	;
SomeVoices:				;

	; There are now 16*dwnVoices bytes of GUID data on the sack, we need
	; to save these on the heap for later

	; ** Allocate Heap Memory **
	mov ebx,dwnVoices		; Number of voices
	sal ebx,4			; * 16 = number of bytes
	pushd ebx			; dwBytes
	xor eax,eax			;
	pushd eax			; dwFlags
	pushd hProcessHeap		; hHeap
	call _HeapAlloc@12		;
	or eax,eax			;
	jnz Synth_HeapAlloc_OK		;
	;* Failed to allocate memory *
	; (This should never happen)
	mov dwnVoices,eax		; No voices, because we can't store their GUIDs
	add ebx,1432			;
	add esp,ebx			; De-allocate memory for TTSInfo and GUIDs
	mov esi, offset szNoMemory	; Error: No memory
 	jmp SynthInitFailMessage	; Abort
Synth_HeapAlloc_OK:			;
	mov lpVoiceGUIDs,eax		;

	; ** Store GUIDs on heap (in correct order) **
	mov edi,eax			;
	mov ecx,dwnVoices		; Number of voices
	push esi			; Save ESI
	lea esi,[esp+ebx+20]		; Stack prior to first GUID Pushed
Synth_Next_GUID:			;
	sub esi,32			; Next GUID Pushed
	movsd				;
	movsd				;
	movsd				;
	movsd				;
	loop Synth_Next_GUID		;
	pop esi				; Restore ESI
	add ebx,1432			;
	add esp,ebx			; De-allocate memory for TTSInfo and GUIDs

	; ** Finished with voice enumerator **
	push esi				;
	mov edi,dword ptr [esi]		;
      	call dword ptr [edi+8]		; pITTSEnum->Release()

	; ** Select Voice **
	xor eax,eax			;
	pushd eax			;lParam
	pushd dwVoiceSelected		;wParam = Index to select
	pushd CB_SETCURSEL		;Msg
	mov al,120			;
	pushd eax			;nIDDlgItem
	pushd hDlg			;hDlg
	call _SendDlgItemMessageA@20

	; *** Not playing anything yet ***
	mov bPlaying,0			;

	jmp exit_synth_true		;

Synth_dlg_help:				; **** [Help] button on open file dialog ****
Synth_Help_button:			; **** [Help] button ****
	pushd 6				; Help Topic (6=Synthesise)
	pushd HELP_CONTEXT		;
	pushd offset szHelpFileName	;
	pushd hWnd			;
	call _WinHelpA@16		;
	jmp exit_Synth_true		;


Synth_Preview_button:			; ***** PREVIEW Button ****

	; Jump into the "write to file" function
	mov al,0			;
	jmp Synth_play			;


Synth_Stop_button:			; ***** STOP PLAYING Button ****

	mov esi,pITTSCentral		;
	mov edi,dword ptr [esi]		;
	pushd esi			;
	call dword ptr [edi+44]		; ITTSCentral->AudioReset()

	sub esp,16			; Dummy data on stack
	call ITTSBufNotifySink_TextDataDone	; Call the "Done" function

	call SynthClean			; Clean up

	jmp exit_Synth_true		;


Synth_ok_button:			; ***** WRITE TO FILE (OK) Button ******

	; **** Get User to enter File Name ****
	sub esp,MAX_PATH		; Allocate space for file name

	mov esi,esp			;
	mov dword ptr [esi],0		; File name initially null

	; ** Display standard Save As  dialogue **
	mov OFNFlags, OFN_PATHMUSTEXIST+OFN_OVERWRITEPROMPT+OFN_SHOWHELP+OFN_HIDEREADONLY
	mov lpstrTitle,offset szBrowseVfiles
	mov lpstrFilter,offset vfile_filter
	mov lpstrDefExt,offset szWAV
	mov lpstrFile, esi
	mov lpstrInitialDir,offset szWaveDirName
	push offset ofn_struct		;
	call _GetSaveFileNameA@4	;
	or eax,eax			; File name entered ?
	jnz Synth_FileNameEntered	; If so, jump

	; ## Possibly check here file name is valid ##

Synth_MultiByteToWide_Fail:
	; Some problem: User hasn't entered a file name, or this name
	; won't convert to Unicode.  Either way, we can't proceed.
	add esp,MAX_PATH		;  Clean up stack
	jmp exit_Synth_true		;

Synth_FileNameEntered:			;

	; ** Update the directory we were using **
	pushd offset szWaveDirName	; lpBuffer
	pushd MAX_PATH			; nBufferLength
	call _GetCurrentDirectoryA@8	;

	; ** Convert file name to Unicode **
	xor eax,eax			;
	pushd MAX_PATH			; cchWideChar
	pushd offset wszVoiceFile	; lpWideCharStr ## Local Memory ##
	pushd -1			; cchMultiByte
	pushd esi			; lpMultiByteStr
	pushd eax			; dwFlags
	pushd eax			; CodePage = CP_ACP
	call _MultiByteToWideChar@24	;
	or eax,eax			;
	jz Synth_MultiByteToWide_Fail	; Jump if error (shouldn't happen)

	add esp,MAX_PATH		; Remove file name off stack

	mov al,1			; We are writing to file
Synth_play:				; **** Jumped to from "Preview" ****
	mov bWriteToFile,al		; This flag selects where we send our text

	; **** Get Text to Speak ****
	pushd SizeOfTextToSpeak		; nMaxCount
	pushd offset sTextToSpeak	; lpString ##
	pushd 130			; nIDDlgItem
	pushd hDlg			; hDlg
	call _GetDlgItemTextA@16	;
	or eax,eax			; Was anything entered ?
	jnz Synth_text_entered		; Jump if so

	; ** Error: User hasn't entered anything **
	mov esi,offset szNoText		;
	call ErrorMessage		;

	jmp exit_Synth_true		;
Synth_text_entered:			;
	inc eax				; Length Plus null terminator
	mov dwlTextToSpeak, eax		; Save

	; ** Access TTS Voice Enumerator **
	xor eax,eax			;
	pushd offset pITTSFind		;
	pushd offset IID_ITTSFindA	;
	pushd CLSCTX_ALL		;
	pushd eax			;
	pushd offset CLSID_TTSEnumerator;
	call _CoCreateInstance@20	;
	or eax,eax			;
	jnz TTSEnumFail			;

	mov esi,pITTSFind		;
	mov edi,dword ptr [esi]		;

	; ** Get Voice Selected **
	; EAX=0 on entry
	pushd eax			; lParam
	pushd eax			; wParam
	pushd CB_GETCURSEL		; Msg
	mov al,120			;
	pushd eax			; nIDDlgItem
	pushd hDlg			; hDlg
	call _SendDlgItemMessageA@20	;
	; EAX Contains offset (0 to n-1) of voice selected
	cmp eax,0			; Was something selected ?
	jge Synth_VoiceSelected		; Jump if so
	xor eax,eax			; If not, select first one (should never happen)
Synth_VoiceSelected:			;
	sal eax,4			;
	add eax,lpVoiceGUIDs		; Get offset of GUID
	mov ebx,eax			; in EBX

	; What we have to do now is open an audio destination and
	; attach it to this TTS engine to create an engine object which
	; can play our message.  We open either a multimedia destination
	; (i.e. our speakers) or a file destination and feed this to our
	; engine via a telemetry audio object.  We need to do this so as
	; we can set the data rate to the required value .
	; (PCM, 1 channel, 8000 samples/sec, 16 bits/sample)

	; ** Write it to file or multimedia device ?
	cmp bWriteToFile,0		;
	jne Synth_Dest_File		;

	; **** Multimedia Destination ****
	xor eax,eax			;
	pushd offset gpIAudio		;
	pushd offset IID_IAudioMMDevice	;
	pushd CLSCTX_ALL		;
	pushd eax			;
	pushd offset CLSID_MMAudioDest;
	call _CoCreateInstance@20	;
	or eax,eax			;
	jnz AudioDestFail		;

	; ** Set this destination to be the Wave Mapper **
	pushd WAVE_MAPPER		;
	mov eax,gpIAudio		;
	pushd eax			;
	mov eax,dword ptr [eax]		;
	call dword ptr [eax+20]		; g_pIAMM->DeviceNumSet()
	or eax,eax			;
	jnz AudioDestFail		;

	jmp Synth_Dest_Audio		;

Synth_Dest_File:			;

	; **** Audio Destination File ****
	xor eax,eax			;
	pushd offset gpIAudio		;
	pushd offset IID_IAudioFile	;
	pushd CLSCTX_ALL		;
	pushd eax			;
	pushd offset CLSID_AudioDestFile;
	call _CoCreateInstance@20	;
	or eax,eax			;
	jnz AudioDestFail		;

Synth_Dest_Audio:			;

	; **** Audio Destination Telephony ****
	; EAX=0 on entry
	pushd offset gpIAudioTel	;
	pushd offset IID_IAudioTel	;
	pushd CLSCTX_ALL		;
	pushd eax			;
	pushd offset CLSID_AudioDestTel	;
	call _CoCreateInstance@20	;
	or eax,eax			;
	jnz AudioTelFail		;

	; **** Set the wave format ****
	mov al,cSizeOfWaveFormatEx	; EAX=0 on entry
	pushd eax			;
	pushd offset waveformat		; SDATA WAVEFORMATEX
	mov eax,gpIAudioTel		;
	pushd eax			;
	mov eax,dword ptr [eax]		;
	call dword ptr [eax+16]		; gpIAudioTel->WaveFormatSet
	or eax,eax			;
	jnz AudioTelFail		;

	; **** Make the audio object (file or multimedia) go via the Telephony object ****
	; We need to do this to get the data rate right
	push gpIAudio			;
	mov eax,gpIAudioTel		;
	pushd eax			;
	mov eax,dword ptr [eax]		;
	call dword ptr [eax+12]		; gpIAudioTel->AudioObject(gpIAudio)
	or eax,eax			;
	jnz AudioTelFail		;

	; ** Create Engine Object **
	; This links the selected output device to the TTS engine
	pushd gpIAudio;Tel 		; PIUNKNOWN pIUnknownForAudio = gpIAudioTel
	pushd offset pITTSCentral	; PITTSCENTRAL *ppiTTSCentral =  &pITTSCentral
	pushd dword ptr [ebx+12]	; GUID gModeID =  ttsResult.gModeID
	pushd dword ptr [ebx+8]		;
	pushd dword ptr [ebx+4]		;
	pushd dword ptr [ebx]		;
	pushd esi			;
	call dword ptr [edi+16]		; pITTSFind->Select
	or eax,eax			;
	jnz AudioDestFail		;

	; ** Write it to file or multimedia device ?
	cmp bWriteToFile,0		;
	je Synth_No_File		;

	; ** Set file name **
	mov esi,gpIAudio			;
	mov edi,dword ptr [esi]		;
	pushd 1				; dwID
	pushd offset wszVoiceFile	; pszFile  ##
	pushd esi			;
	call dword ptr [edi+16]		; gpIAudio->Set()

	or eax,eax			;
	jnz CantOpenVoiceFile		;
Synth_No_File:

	; ** Finished with voice enumerator **
	mov esi,pITTSFind		;
	mov edi,dword ptr [esi]		;
	pushd esi			;
      	call dword ptr [edi+8]		; pITTSFind->Release()
	xor eax,eax			;
	mov pITTSFind,eax		;

	; *** Write on screen what we are doing ***
	cmp bWriteToFile,0		;
	jne Synth_Writing		;
	mov eax,offset szPlaying	;
	jmp Synth_Playing		;
Synth_Writing:				;
	mov eax,offset szWriting	;
Synth_Playing:				;

	;** Write Status **
	pushd eax			;
	pushd 129			;
	pushd hDlg			;
	call _SetDlgItemTextA@12	; Status: Playing... or Writing...

	; ** Disable "Preview", "Write to File" and "Finished" Buttons **
	xor ebx,ebx			;
	pushd ebx			; bEnable
	pushd hButtonPreview		; hWnd
	call _EnableWindow@8		;
	pushd ebx			; bEnable
	pushd hButtonWrite		; hWnd
	call _EnableWindow@8		;
	pushd ebx			; bEnable
	pushd hButtonFinished		; hWnd
	call _EnableWindow@8		;

	; ** Enable "Stop" Button **
	inc ebx				;
	pushd ebx			; bEnable = 1
	pushd hButtonStop		; hWnd
	call _EnableWindow@8		;

	; *** We are (almost) Playing ***
	; Due to potential race conditions on a very fast computer, we
	; set this before we start playing the message
	mov bPlaying,1			;

	mov esi,pITTSCentral		;
	mov edi,dword ptr [esi]		;

	; **** Speak it ****
	; Actually start speaking the message; either to file, or the
	; Multimedia device (speaker).
	mov eax, offset IID_ITTSBufNotifySink
	pushd dword ptr [eax+12]	; IIDNotifyInterface
	pushd dword ptr [eax+8]		;
	pushd dword ptr [eax+4]		;
	pushd dword ptr [eax]		;
	xor eax,eax			;
	pushd offset pITTSBufNotifySink	; pNotifyInterface
	pushd dwlTextToSpeak		; dText.dwSize
	pushd offset sTextToSpeak	; dText.pData
	pushd 1				; dwFlags (Note: 1 = TTSDATAFLAG_TAGGED)
	pushd eax			; eCharacterSet = CHARSET_TEXT
	pushd esi			;
	call dword ptr [edi+28]		; ITTSCentral->TextData()

	jmp exit_Synth_true		;

AudioTelFail:				;

AudioDestFail:				;
FindVoiceFail:				;
	; ** Finished with voice enumerator **
CantOpenVoiceFile:			;
;	pushd esi			;
;      	call dword ptr [edi+8]		; pITTSFind->Release()

	; ** Close off anything that is open **
	call SynthClean

TTSEnumFail:

	mov esi,offset szNoTTS		;
	call ErrorMessage		;

Synth_cancel_button:			; ***** FINISHED buttton ******

	cmp bPlaying,0			; Are we already playing ?
	jne exit_Synth_true		; Exit if so

end_Synth_dialog:			;

	; **** Exit ****

	; ** Close off anything that is open **
	call SynthClean
SynthInitFail2:
	; ** Uninitialize COM interface **
	call _CoUninitialize@0		;
SynthInitFail1:

	; ** Deallocate GUID memory **
	cmp dwnVoices,0			; Were there any voices (should always be in theory)
	je Synth_no_voices		; Jump if not
	xor eax,eax			;
	pushd lpVoiceGUIDs		; lpMem
	pushd eax			; dwFlags
	pushd hProcessHeap		; hHeap
	call _HeapFree@12		; Free up heap for GUIDs
Synth_no_voices:			;

	; ** Get coordinates of dialogue box **
	mov esi,offset temp_buffer	;
	mov edi,offset xcoord_Synth	;
	pushd esi			;
	pushd hDlg			;
	call _GetWindowRect@8		; Get bounding rectangle
	movsd				; Transfer X coord
	movsd				; Transfer Y coord

	; ** Exit **
exit_Synth:

	; ** Save Voice Selected **
	xor eax,eax			;
	pushd eax			; lParam
	pushd eax			; wParam
	pushd CB_GETCURSEL		; Msg
	mov al,120			;
	pushd eax			; nIDDlgItem
	pushd hDlg			; hDlg
	call _SendDlgItemMessageA@20	;
	mov dwVoiceSelected,eax		; Save it for next time

	; ** Now Exit **
	xor eax,eax			;
	pushd eax			;
	pushd hDlg			;
	call _EndDialog@8		;
	jmp end_Synthproc		;

exit_Synth_true:			;
	mov eax,1			;
end_Synthproc:				;
	pop edi				;
	pop esi				;
	pop ebx				;
	ret 16				;
SynthProc ENDP


;****************************************************************************
; Name: SynthClean
; Purpose: Calls the "release" functions for all interfaces that may have been
;   opened by the synth procedure above.
;****************************************************************************

SynthClean PROC
	push esi

	mov ecx,4			; Number of interfaces
	mov esi, offset pITTSFind	; Point to first
SynthCleanNext:				;
	push ecx			; Save count
	lodsd				; Get interface pointer
	or eax,eax			; Was it initialized ?
	jz SynthDontRelease		; Jump if not
	pushd eax			;
	mov eax,dword ptr [eax]		;
      	call dword ptr [eax+8]		; x->Release()
	xor eax,eax			;
	mov [esi-4],eax			; Mark it as not initialized
SynthDontRelease:			;
	pop ecx				; Restore count
	loop SynthCleanNext		; Next interface

	pop esi				;


	mov bPlaying,0			;

	ret				;
SynthClean ENDP

ITTSBufNotifySink_QueryInterface PROC	; QueryInterface (REFIID riid, LPVOID *ppv)
	; [esp+8] = riid
	; [esp+12] = *ppv
	push esi			;
	push edi			;

	mov ecx,4			;
	mov esi, [esp+8+8]		;
	mov edi, offset IID_IUnknown	;
	repe cmpsd			;
	je IID_IUnknown_OK		; Jump if riid=IID_IUnknown
	mov ecx,4			;
	mov esi, [esp+8+8]		;
	mov edi, offset IID_ITTSBufNotifySink ;
	repe cmpsd			;
	je IID_IUnknown_OK		; Jump if riid=IID_ITTSBufNotifySink
	; Otherwise Error
	xor eax,eax			;
	mov ecx,dword ptr [esp+12+8]	;
	mov dword ptr [ecx],eax	;
	mov eax,080004002h		; Return E_NOINTERFACE
	jmp EndQueryInterface		;
IID_IUnknown_OK:
	mov eax,dword ptr [esp+16+8]	; ## Not sure if this is right ##
	mov ecx,dword ptr [esp+12+8]	;
	mov dword ptr [ecx],eax		;
	xor eax,eax			; Return S_OK
EndQueryInterface:			;
	pop edi				;
	pop esi				;
	ret 12				;
ITTSBufNotifySink_QueryInterface ENDP
ITTSBufNotifySink_AddRef PROC		; AddRef(void) also Release(void)

	mov eax,1			;
	ret 4				;
ITTSBufNotifySink_AddRef ENDP
ITTSBufNotifySink_TextDataDone PROC	; TextDataDone    (QWORD, DWORD)

	pushd offset szDone		;
	pushd 129			;
	pushd hDlg			;
	call _SetDlgItemTextA@12	; Status: Done

	; ** Disable "Stop" Button **
	xor ebx,ebx			;
	pushd ebx			; bEnable
	pushd hButtonStop		; hWnd
	call _EnableWindow@8		;

	; ** Enable "Preview", "Write to File" and "Finished" Buttons **
	inc ebx				;
	pushd ebx			; bEnable = 1
	pushd hButtonPreview		; hWnd
	call _EnableWindow@8		;
	pushd ebx			; bEnable
	pushd hButtonWrite		; hWnd
	call _EnableWindow@8		;
	pushd ebx			; bEnable
	pushd hButtonFinished		; hWnd
	call _EnableWindow@8		;


	xor eax,eax			;
	mov bPlaying,al			;
	ret 16				;
ITTSBufNotifySink_TextDataDone ENDP
ITTSBufNotifySink_TextDataStarted PROC	; TextDataStarted (QWORD)

	xor eax,eax			;
	ret 12				;
ITTSBufNotifySink_TextDataStarted ENDP
ITTSBufNotifySink_BookMark PROC		; BookMark(QWORD, DWORD) also WordPosition (THIS_ QWORD, DWORD)

	xor eax,eax			;
	ret 16				;
ITTSBufNotifySink_BookMark ENDP


;****************************************************************************
; Name: PlayBackProc
; Purpose: Handles commands for "Play Back" pop-up.
;	[esp+4] = hDlg
;	[esp+8] = iMessage
;	[esp+12] = wParam
;	[esp+16] = lParam
; Note: We could use _PlaySound@12 here, but it is better to use the
; same method that we will use to deliver the actual message.
;****************************************************************************
PlayBackProc PROC

	push ebx			;
	push esi			;
	push edi			;
	mov eax,[esp+8+12]		; iMessage
	cmp eax,WM_INITDIALOG		;
	je Play_init			; Initial entry
	cmp eax,WM_TIMER		;
	je Play_wm_timer		; Timer tick while playing
	cmp eax,WOM_DONE		;
	je play_end_preview		; Finished playing message
	cmp eax,WM_COMMAND		;
	jne exit_Play_false		;

	;  ***** Command *****
	mov eax,[esp+12+12]		; wParam
	cmp eax,IDCANCEL		; [Cancel] Button
	je Play_cancel_button		;
	cmp eax,130			; [Play] button
	je play_button			;
	cmp eax,131			; [Stop] button
	je play_Stop_button		;
;;	cmp eax,121			; () Speaker Radio button
;;	je play_speaker_button		;
;;	cmp eax,122			; () Modem Radio button
;;	je play_modem_button		;
	cmp eax,140			; [Help] button
	je play_Help_button		;
	; Otherwise Unknown command

exit_Play_false:			;
	xor eax,eax			;
	jmp end_Playproc		;

Play_init: 				; ***** Initialise Dialog *****



	mov ebx,dword ptr [esp+4+12]	; hDlg
	mov hDlgPlay,ebx			;

	pushd ycoord_Play		; Resposition Dialogue Box
	pushd xcoord_Play		;
	call PositionDialog		;

	; ** Create Progress Bar **
	xor eax,eax			;
	pushd eax			; lpParam
	pushd hInstance			; hInstance
   	pushd 20            		; hmenu/child window identifier
    	pushd ebx            		; parent hwnd

	; ** Find how big the dialogue box units are **
	; We need to do this because the progress bar is in units of Pixels
	pushd 36			; Bottom
	pushd 200			; Right
	pushd 24			; Top
	pushd 2 ;; 90			; Left

	pushd esp			; lpRect
	pushd ebx			; hDlg
	call _MapDialogRect@8		;

	; The corrected RECT structure is on the stack
	; Convert it to the right format for CreateWindowEx()

	mov eax,dword ptr [esp]		; Left
	inc eax				;
	sub dword ptr [esp+8],eax	; Width  = Left-Right+1
	mov eax,dword ptr [esp+4]	; Top
	inc eax				;
	sub dword ptr [esp+12],eax	; Height  = Bottom-Top+1

;	pushd 			; hHeight
;	pushd 			; hWidth
;	pushd 			; y
;	pushd 			; x

	xor eax,eax			;
	pushd WS_VISIBLE + WS_CHILD	;
	pushd eax			;
	pushd offset szProgressClassA	;
	pushd eax			; Extended Style
	call _CreateWindowExA@48	;
	mov hWndProgressBar,eax		; Save window handle

	; ** Get handles to all buttons **
	pushd 130			; nIDDlgIte
	pushd ebx			; hDlg
	call _GetDlgItem@8		;
	mov hButtonPlay,eax		;
	pushd 131			; nIDDlgIte
	pushd ebx			; hDlg
	call _GetDlgItem@8		;
	mov hButtonStopPlaying,eax	;
	pushd IDCANCEL			; nIDDlgIte
	pushd ebx			; hDlg
	call _GetDlgItem@8		;
	mov hButtonFinishedPlaying,eax	;

	;** Display WAV File Name **
	pushd offset szNewVoiceFileName	;
	pushd 200			;
	pushd ebx			;
	call _SetDlgItemTextA@12	;

;;	xor eax,eax			;
;;	mov bModemSpeaker,al		; ### from INI file ? ##
;;
;;	;** Fix up "Speaker"/"Modem" Radio Buttons **
;;	add eax,121			;
;;	pushd eax			; nIDCheckButton
;;	pushd 122			; nIDLastButton
;;	pushd 121			; nIDFirstButton
;;	pushd ebx			; hDlg
;;	call _CheckRadioButton@16	;
;;
;;	; **** Open WAV device appropriate for destination **

	; ** Open WAV device for Multimedia Speaker **
	xor eax,eax			;
	pushd CALLBACK_WINDOW		; fdwOpen
	pushd eax			; dwCallbackInstance
	pushd ebx			; dwCallback
	pushd offset WaveFormat		; pwfx
	pushd WAVE_MAPPER		; uDeviceID
	pushd offset hWavOutDevPlay	; phwo
	call _waveOutOpen@24		;
        or eax,eax			;
	jz PlayWAVOpen_OK		;

	;** Error Message: Can't open wave output device **
	mov esi,offset szNoWaveOut	;
	call ErrorMessage		;
	jmp PlayWAVOpen_fail		;

PlayWAVOpen_OK:				;


	; ** Prepare Wave Header **
	pushd 32			; cbwh = SizeOf(WaveOutHdr)
	pushd offset WaveOutHdr		; pwh
	pushd hWavOutDevPlay		; hwo
	call _waveOutPrepareHeader@12	;
        or eax,eax			;
	jnz PlayWAVPrepareHeader_fail	;


	mov bPlaying,0			;

	jmp exit_play_true		;

play_button:				; ***** [Play] Button ******

;;	pushd offset temp_buffer 	; LPDWORD pdwVolume   
;;	pushd hWavOutDevPlay		; HWAVEOUT hwo
;;	call _waveOutGetVolume@8	;
;;	mov eax, dword ptr [temp_buffer]
;;	call debugeax
;;
;;	pushd 0FFFFFFFFH 		; DWORD dwVolume 
;;	pushd hWavOutDevPlay		; HWAVEOUT hwo
;;	call _waveOutSetVolume@8	;

	; ** Start Speaking Message **
	pushd 32			; cbwh
	pushd offset WaveOutHdr		; pwh
	pushd hWavOutDevPlay		; hwo
	call _waveOutWrite@12		;
        or eax,eax			;
	jnz PlayWAVWrite_fail		;

	; **** We are speaking message now ****
	mov bPlaying,1			;

	; ** Reset  Progress Bar **
	xor eax,eax			;
	pushd eax			; lParam
	pushd eax			; wParam 0% done
	pushd PBM_SETPOS		; Msg
	pushd hWndProgressBar		; hWnd
	call _SendMessageA@16		;

	;** Update Status **
	pushd offset szPlaying		;
	pushd 129			;
	pushd hDlgPlay			;
	call _SetDlgItemTextA@12	; Status: Playing Message...

	; ** Disable "Play" and "Finished" Buttons **
	xor ebx,ebx			;
	pushd ebx			; bEnable
	pushd hButtonPlay		; hWnd
	call _EnableWindow@8		;
	pushd ebx			; bEnable
	pushd hButtonFinishedPlaying	; hWnd
	call _EnableWindow@8		;

	; ** Enable "Stop" button **
	inc ebx				;
	pushd ebx			; bEnable =1
	pushd hButtonStopPlaying	; hWnd
	call _EnableWindow@8		;

	; ** Enable timer to update progress **
	xor eax,eax			;
	pushd eax			; lpTimerFunc
	pushd 200			; uElapse = 1/5 Sec
	mov al,2			;
	pushd eax			; nIDEvent
	pushd hDlgPlay			; hWnd
	call _SetTimer@16		;
	; If this fails, we don't care, progress just won't get updated

PlayWAVWrite_fail:			;
	jmp exit_Play_true		;

Play_wm_timer:				; ***** Timer Tick while playing ****

	call PlayUpdateProgressBar	;

	jmp exit_play_true		;

play_end_preview:			; ***** Play Sound Finished ****

	call PlayUpdateProgressBar	;

	mov bPlaying,0			;

	; ** Finished with timer **
	pushd 2				; uIDEvent
	pushd hDlgPlay			; hWnd
	call _KillTimer@8		;

	;** Update Status **
	pushd offset szDone		;
	pushd 129			;
	pushd hDlgPlay			;
	call _SetDlgItemTextA@12	; Status: Done

	; ** Disable "Stop" Button **
	xor ebx,ebx			;
	pushd ebx			; bEnable
	pushd hButtonStopPlaying 	; hWnd
	call _EnableWindow@8		;

	; ** Enable "Play" and "Finished" buttons **
	inc ebx				;
	pushd ebx			; bEnable =1
	pushd hButtonPlay		; hWnd
	call _EnableWindow@8		;
	pushd ebx			; bEnable
	pushd hButtonFinishedPlaying	; hWnd
	call _EnableWindow@8		;

	; Fall though to [Stop] button to reset everything

play_Stop_button:			; **** [Stop] button  ****


	call PlayUpdateProgressBar	;

	; ** Stop playing **
	pushd hWavOutDevPlay		; hwo
	call _waveOutReset@4		;


	mov bPlaying,0			;

	jmp exit_play_true		;

;;play_speaker_button:			; **** () Speaker Radio button  ****
;;	xor eax,eax			;
;;	jmp play_speaker
;;play_modem_button:			; **** () Modem Radio button  ****
;;	mov eax,1			;
;;play_speaker:				;
;;
;;	mov bModemSpeaker,al		; Set flag
;;
;;	;** Fix up "Speaker"/"Modem" Radio Buttons **
;;	mov ebx,121			;
;;	add eax,ebx			;
;;	pushd eax			; nIDCheckButton
;;	pushd 122			; nIDLastButton
;;	pushd ebx			; nIDFirstButton
;;	pushd hDlgPlay			; hDlg
;;	call _CheckRadioButton@16	;
;;
;;
;;	; #####
;;
;;
;;	jmp exit_play_true		;
;;


play_Help_button:			; **** [Help] button ****
	pushd 8				; Help Topic
	pushd HELP_CONTEXT		;
	pushd offset szHelpFileName	;
	pushd hWnd			;
	call _WinHelpA@16		;
	jmp exit_Play_true		;



Play_cancel_button:			; ***** FINISHED buttton ******
end_Play_dialog:			;

	; ** Unprepare Wave Header **
	pushd 32			; cbwh = SizeOf(WaveOutHdr)
	pushd offset WaveOutHdr		; pwh
	pushd hWavOutDevPlay		; hwo
	call _waveOutUnprepareHeader@12	;

PlayWAVPrepareHeader_fail:		;



	; ** Close WAV device **
	pushd hWavOutDevPlay		; hwo
	call _waveOutClose@4		;


;;	cmp bModemSpeaker,0		;  ###########
;;	jz play_exit_mmspeaker		;  ###########
;;
;;play_exit_mmspeaker:			;

PlayWAVOpen_fail:			;

	call FreeWAVMemory		; Free any memory allocations

	mov esi,offset temp_buffer	;
	mov edi,offset xcoord_Play	;
	pushd esi			;
	pushd hDlgPlay			;
	call _GetWindowRect@8		; Get bounding rectangle
	movsd				; Transfer X coord
	movsd				; Transfer Y coord

	; ** Remove progress bar **
	; (May not be strictly necessary)
	pushd hWndProgressBar		; hWnd
	call _DestroyWindow@4		;

	xor eax,eax			;
	pushd eax			;
	pushd hDlgPlay			;
	call _EndDialog@8		;
	jmp end_Playproc		;

exit_Play_true:				;
	mov eax,1			;
end_Playproc:				;
	pop edi				;
	pop esi				;
	pop ebx				;
	ret 16				;
PlayBackProc ENDP

;****************************************************************************
; Name: PlayUpdateProgressBar
; Purpose: Updates progress bar for Play Procedure
; No parameters or return values
;****************************************************************************
PlayUpdateProgressBar PROC


	cmp bPlaying,0			; Are we idle ?
	jz PlayProgBarEnd		; If so, finished (can't do anything)

	; ** Allocate MMTIME structure on stack **
	mov eax,12			;
	sub esp,eax			;
	mov ecx,esp			; pmmt
	mov dword ptr [ecx],4		; pmmt.wType = TIME_BYTES
	pushd eax			; cbmmt (size of MMTIME structure)
	pushd ecx			; pmmt
	pushd hWavOutDevPlay		; hwo
	call _waveOutGetPosition@12	;

	pop edx				; pmmt.wType
	pop ebx				; pmmt.cb
	pop ecx				; Deallocate rest of MMTIME structure
	or eax,eax			; Error code
	jnz PlayProgBarEnd		;
	cmp edx,4			; Check pmmt.wType = TIME_BYTES
	jne PlayProgBarEnd		; Jump if not

	; ** Determine % done **
	mov ecx,WaveHdr_dwBufferLength	;
	; ECX = total bytes		;
	; EBX = bytes played		;
	mov al,100			;
	mul ebx				;
	div ecx				; EAX = % done

	; ** Update Progress Bar **
	xor ecx,ecx			;
	pushd ecx			; lParam
	pushd eax			; wParam
	pushd PBM_SETPOS		; Msg
	pushd hWndProgressBar		; hWnd
	call _SendMessageA@16		;

PlayProgBarEnd:				;
	ret
PlayUpdateProgressBar ENDP

;****************************************************************************
; Name: PositionDialog
; Purpose : Positions a current dialog box at the specified screen
; coordinates (x,y).  Normally, this will be called on entry to the dialogue
; box routine, ie: on receiving a WM_INITDIALOG message.
; Parameters: (x,y) on stack, EBX = hDlg
;****************************************************************************
PositionDialog PROC


	; (y and x on stack)

	; ** Find size of dialog box (in pixels) **
	; Note that while this may not change, it is not known in advance,
	; as it depends on the system font size.
	mov esi,offset temp_buffer		;
	pushd esi				; lpRect
	pushd ebx				; hWnd
	call _GetWindowRect@8			; Get bounding rectangle
	mov eax,dword ptr [esi]			; Left
	sub dword ptr [esi+8],eax		; Width = Right - Left
	mov eax,dword ptr [esi+4]		; Top
	sub dword ptr [esi+12],eax		; Height = Bottom - Top

	; ** Get size of screen **
	; Note : We can't just do this on startup, and store the result,
	; because someone might change the video mode or task bar size
	; while this program is running.

	xor eax,eax				;
	pushd eax				; fWinIni = 0
	pushd offset temp_buffer+16		; pvParam
	inc eax					;
	pushd eax				; uiParam = 1
	pushd SPI_GETWORKAREA			; uiAction
	call _SystemParametersInfoA@16		;

	; ** Ensure dialog box will fit on screen **

	; [esi+8] = dialog box width
	; [esi+12] = dialog box height
	; [esi+16] = screen left
	; [esi+20] = screen top
	; [esi+24] = screen right
	; [esi+28] = screen bottom
	; x=[esp+4]
	; y=[esp+8]

	; ** Overlap on right **
	mov eax,[esp+4]				; Get initial X

	add eax,[esi+8]				; eax = dialog box right
	cmp eax,[esi+24]			; Does it overlap on right ?
	jle no_r_overlap			; If not, jump
	mov eax,[esi+24]			; If so, fix it
no_r_overlap:					;

	sub eax,[esi+8]				; eax = new "X"
	; ** Overlap on left **
	cmp eax,[esi+16]			; Does it overlap at left
	jge no_l_overlap			; if not, jump
	mov eax,[esi+16]			; If so, fix it
no_l_overlap:
	mov ecx,eax				; Save X

	; ** Overlap at bottom **
	mov eax,[esp+8]				; Get initial Y
	add eax,[esi+12]			; eax = dialog box bottom
	cmp eax,[esi+28]			; Does it overlap at bottom ?
	jle no_b_overlap			; If not, jump
	mov eax,[esi+28]			; If so, fix it
no_b_overlap:					;
	sub eax,[esi+12]			; eax = new "Y"
	; ** Overlap at top **
	cmp eax,[esi+20]			; Does it overlap at top
	jge no_t_overlap			; if not, jump
	mov eax,[esi+20]			; If so, fix it
no_t_overlap:
	xor edx,edx				;
	pushd edx				; Repaint ? =0
	pushd dword ptr [esi+12]		; Height
	pushd dword ptr [esi+8]			; Width
	pushd eax				; Y
	pushd ecx				; X
	pushd ebx				; hWnd
	call _MoveWindow@24			; Reposition window
	ret 8
PositionDialog ENDP


;****************************************************************************
; Name: ErrorMessage
; Purpose: Message Box with commonly used parameters
; Parameters: ESI contains pointer to error message
;****************************************************************************
ErrorMessage PROC
	push esi			;
	call DebugMsg			; Debug it too

	pushd MB_ICONEXCLAMATION	;
	pushd offset szAppTitle		;
	pushd esi			;
	pushd hDlg			; hWnd
	call _MessageBoxA@16		;
	ret				;
ErrorMessage ENDP

;****************************************************************************
; Name: StartCounting
; Purpose: Call this when we want to start the counting process
;****************************************************************************
StartCounting PROC NEAR

	pushd offset szStarted		;
	call DebugMsg			; Debug what we are doing

	mov bCountingDown,1		; Start timing

	pushd MF_GRAYED			;
	pushd 202			;
	pushd hMenu			;
	call _EnableMenuItem@12		;  Grey "Configure..."

	; Fix up menu items
	mov ebx,12			;
	pushd MF_GRAYED			;
	pushd ebx			;
	pushd hMenu			;
	call _EnableMenuItem@12		;  Grey "Start"
	xor eax,eax			;
	pushd eax			; 0=MF_ENABLED
	pushd 13			;
	pushd hMenu			;
	call _EnableMenuItem@12		;  Enable "Stop"

	; Fix up pushbuttons
	pushd ebx			; nIDCheckButton
	pushd 13			; nIDLastButton
	pushd ebx			; nIDFirstButton
	pushd hWnd			; hDlg
	call _CheckRadioButton@16	; Depress "Start" pushbutton
	ret
StartCounting ENDP

;****************************************************************************
; Name: StopCounting
; Purpose: Call this when we want to stop the counting process
;****************************************************************************
StopCounting PROC NEAR

	call RemoveAutoRun		;  Remove any auto-run option in registry

	mov bCountingDown,0		; Stop timing

	; Fix up menu items
	xor eax,eax			;
	pushd eax			;
	pushd 202			;
	pushd hMenu			;
	call _EnableMenuItem@12		;  Enable "Configure..."
	xor eax,eax			;
	mov ebx,12			;
	pushd eax			;  0=MF_ENABLED
	pushd ebx			;
	pushd hMenu			;
	call _EnableMenuItem@12		;  Enable "Start"
	pushd MF_GRAYED		;
	pushd 13			;
	pushd hMenu			;
	call _EnableMenuItem@12		;  Grey "Stop"

	; Fix up pushbuttons
	mov eax,13
	pushd eax			; nIDCheckButton
	pushd eax			; nIDLastButton
	pushd ebx			; nIDFirstButton
	pushd hWnd			; hDlg
	call _CheckRadioButton@16	; Depress "Stop" pushbutton
	ret
StopCounting ENDP

;****************************************************************************
; Name: lineCallbackFunc
; Purpose: Callback function for TAPI Line Interface
;
; VOID FAR PASCAL lineCallbackFunc(
;  DWORD hDevice,
;  DWORD dwMsg,
;  DWORD dwCallbackInstance,
;  DWORD dwParam1,
;  DWORD dwParam2,
;  DWORD dwParam3  }
;
;****************************************************************************
lineCallbackFunc: ; PROC
			;

	pushad				;

	mov eax,dword ptr [esp+8+32]	; dwMsg

	cmp eax,LINE_REPLY		;
	je got_line_reply		;
	cmp eax,LINE_CALLSTATE		;
	jne lineCallbackFuncEnd		;

	mov eax,dword ptr [esp+16+32]	; dwParam1

	cmp eax,LINECALLSTATE_DIALTONE	;
	jne not_dialtone		;
	mov eax,offset szDialTone	;
	jmp DebugLineCallback		;
not_dialtone:				;

	cmp eax,LINECALLSTATE_DIALLING	;
	jne not_dialling		;
	mov eax,offset szDialling	;
	jmp DebugLineCallback		;
not_dialling:				;

	cmp eax,LINECALLSTATE_RINGBACK	;
	jne not_ringback		;
	mov eax,offset szRing		; NOTE: Won't occur with UniModemV
	jmp DebugLineCallback		;
not_ringback:				;

	cmp eax,LINECALLSTATE_BUSY	;
	je got_linecallstate_busy	;

	cmp eax,LINECALLSTATE_CONNECTED	;
	je got_linecallstate_connected	;

	cmp eax,LINECALLSTATE_DISCONNECTED;
	je got_linecallstate_disconnected;

	jmp lineCallbackFuncEnd		;


got_line_reply:				; ***** LINE_REPLY *****
	; We have a Reply, check who it is for
	mov eax,dword ptr [esp+16+32]	; dwParam1 = idRequest
	cmp eax, idTAPIAssynch 		; Is it for us ?
	jne lineCallbackFuncEnd		; Jump if not
	; ** Reply is for us **
	mov eax,dword ptr [esp+20+32]	; dwParam2 = Status
	cmp bDialStatus,20		; Were we awaiting a reply for our dial request ?
	jne not_got_line_reply		; Jump if not
	; **** We have got a LINE_REPLY to our dial request. ****
	call Dial_line_reply_dial	;
	jmp lineCallbackFuncEnd		;
not_got_line_reply:			;
	cmp bDialStatus,90		; Were we awaiting a reply for our hang-up request ?
	jne lineCallbackFuncEnd		; Jump if not
	; **** We have got a LINE_REPLY to our hangup request. ****
	call Dial_line_reply_hangup	;
	jmp lineCallbackFuncEnd		;

got_linecallstate_connected:		; ***** LINECALLSTATE_CONNECTED *****
	cmp bDialStatus,20		; Were we expecting to connect ?
	je got_connected		;
	cmp bDialStatus,30		; Were we expecting to connect ?
	jne lineCallbackFuncEnd		; Jump if not
got_connected:				;
	push offset szConnected		;
	call debugmsg			;
	call Dial_linecallstate_connected
	jmp lineCallbackFuncEnd		;

got_linecallstate_busy:			; ***** LINECALLSTATE_BUSY *****
	push offset szBusy		;
	jmp got_linecallstate_busy2	;

got_linecallstate_disconnected:		; ***** LINECALLSTATE_DISCONNECTED *****
	push offset szDisconnected	;
got_linecallstate_busy2:		;
	call debugmsg			;
	; We may get disconnected at any stage.  A pathological case occurs
	; when we get a disconnect in the middle of doing something to the call
	; (i.e. preparing to speak the message)
	mov al,bDialStatus		; What were we doing ?
	cmp al,30			; Were we dialling and waiting for connect ?
	jne discon_not30		; Jump if not
	call Dial_disconnected_during_dial
	jmp lineCallbackFuncEnd		;
discon_not30:				;
	cmp al,70			; Were we speaking the message ?
	jne lineCallbackFuncEnd		; Jump if not
	call Dial_disconnected_during_speak
	jmp lineCallbackFuncEnd		;

DebugLineCallback:			;
	push eax			;
	call debugmsg			;
;;	jmp lineCallbackFuncEnd		;

lineCallbackFuncEnd:			;

	popad				;
 	ret 24				;
;;lineCallbackFunc ENDP


;****************************************************************************
; Name: Dial
; Purpose: Dials number and delivers message.
;  This must be done in a number of stages, as we have to wait a number of
;  times for things to complete.  Whenever we need to wait for something, we
;  set bDialStatus to indicate where we are and return.
;
; 0 = not dialling
; 10* = Preparing to Dial
; 20 = Waiting for LINE_REPLY (to dial request)
; 30 = Waiting for LINECALLSTATE_CONNECTED
; 40* = Preparing to speak message
; 70 = Waiting for WOM_DONE
; 80* = Preparing to Hang Up
; 90 = Waiting for second LINE_REPLY (to hangup request)
;100* = Shutting down
;
; *=program is doing something (as opposed to waiting).  If something changes 
;   during a state of this type, we cannot act on it at once.  
;****************************************************************************
Dial:
	mov bAbortDial,0		; Don't abort yet
	mov bDialStatus,10		; We are now dialling

	; ** Count down the attempts remaining **
	mov ebx,Attempts2Go		; One fewer attempt remaining
	dec ebx				;
	mov Attempts2Go,ebx		;

	; ** Reset Redial Timer **
	mov eax,RCountdownTime		;
	mov Sec2Go, eax			;

	; ** Screen text: "dialling n of m" **
	mov edi, offset szMainText	;
	mov eax,"laiD"			;
	stosd				;
	mov eax,"gnil"			;
	stosd				;
	mov al," "			;
	stosb				; "Dialling "
	mov eax,Max_attempts		;
	sub eax,ebx			;
	call itoa			; n
	mov eax," fo "			; " of "
	stosd				;
	xor al,al			;
	mov eax,Max_attempts		;
	call itoa			; m

	call update_window		; Update screen text

	push offset offset szMainText	;
	call DebugMsg			;

;;	; ** Number of times to loop message **
;;	mov eax,dwRepeats		; Get number of times to repeat
;;	cmp eax,100			; Is it too big ?
;;	jle Repeats_not_high		; If not, jump
;;	mov eax,100			; Too big, so set to maximum
;;	jmp Repeats_not_low		;
;;Repeats_not_high:			;
;;	cmp eax,0			; Is it too small ?
;;	jg Repeats_not_low		; If not, jump
;;	mov eax,1			; Too small, set to 1
;;Repeats_not_low:			;
;;	mov WaveHdr_dwLoops,eax		; Set number of times to loop
;;
;;	; ** Configure for Looping **
;;	mov WaveHdr_dwFlags, WHDR_BEGINLOOP + WHDR_ENDLOOP
;;
;; **** NOTE: Looping doesn't work with the modem wave/out device, so do it manually ****
;;
	; ** Get API Version supported by modem **
	sub esp,20			; Make space for dwAPIVersion and ExtensionID on stack
	lea eax,dword ptr [esp+16]	; lpdwAPIVersion
	pushd esp			; lpExtensionID
	pushd eax			; lpdwAPIVersion
	pushd 020001h			; dwAPIHighVersion
	pushd 010004h			; dwAPILowVersion
	pushd dwDeviceID		; dwDeviceID
	pushd hLineApp			; hLineApp
	call _lineNegotiateAPIVersion@24;
	add esp,16			; Dispose of ExtensionID
	pop ecx				; dwAPIVersion
	or eax,eax			; Look at return code
	jnz LineOpenFail		; Jump if error

	; ** Open Line (Modem) **
	xor eax,eax			;
	pushd eax			; lpCallParams
	pushd eax			; dwMediaModes
	pushd LINECALLPRIVILEGE_NONE	; dwPrivileges
	pushd eax			; dwCallbackInstance
	pushd eax			; dwExtVersion
	pushd ecx			; dwAPIVersion
	pushd offset hLine		; lphLine
	pushd dwDeviceID		; dwDeviceID
	pushd hLineApp			; hLineApp
	call _lineOpen@36		;
	or eax,eax			;
	jnz LineOpenFail		;

	; ** LINECALLPARAMS **
	; LINECALLPARAMS is 176 bytes long
	mov eax,44*4			; SizeOf(LINECALLPARAMS)
	sub esp,eax			;
	mov edi,esp			;
	stosd				; dwTotalSize
	mov eax,LINEBEARERMODE_VOICE	;
	stosd				; dwBearerMode
	xor eax,eax			;
	stosd				; dwMinRate
	stosd				; dwMaxRate
	mov eax,LINEMEDIAMODE_AUTOMATEDVOICE ;
	stosd				; dwMediaMode
	mov eax,LINECALLPARAMFLAGS_IDLE	;
	stosd				; dwCallParamFlags
	mov eax,LINEADDRESSMODE_ADDRESSID	;
	stosd				; dwAddressMode
	xor eax,eax			;
	mov ecx,37			;
	rep stosd			; Clear rest of structure

	; ** Dial Number **
	pushd esp			; lpCallParams
	pushd eax			; dwCountryCode
	pushd offset szNumber2Dial	; lpszDestAddress
	pushd offset hCall		; lphCall
	pushd hLine			; hLine
	call _lineMakeCall@20		;

	add esp,44*4			; Deallocate LINECALLPARAMS

	push offset szNumber2Dial	;
	call debugmsg			;

	cmp eax,0			;
	jl MakeCallFail			;
	mov idTAPIAssynch,eax		; Save line request identifier

	; We have now started dialling the call.  We need to wait for
	; a LINE_REPLY message before we can check the status of our dial
	; request.

	mov bDialStatus,20		; Waiting for LINE_REPLY
	cmp bAbortDial,0		; Was a manual abort received ?
	jnz MakeCallAbort		; Jump if so

	ret				;
	; ****** Wait for LINE_REPLY ******
	;------------------------------------------------------------
	; ****** Got LINE_REPLY ******

Dial_line_reply_dial:

	or eax,eax			; Check error code
	jnz MakeCallFail		; Jump if there was an error

	; We have successfully dialled, we now have to wait for the call
	; to connect (i.e. LINECALLSTATE_CONNECTED)

	mov bDialStatus,30		; Waiting for LINECALLSTATE_CONNECTED

	ret
	; ****** Wait for LINECALLSTATE_CONNECTED ******
	;------------------------------------------------------------

	; ****** Got LINECALLSTATE_CONNECTED ******
Dial_linecallstate_connected:

	mov bDialStatus,40		; 

	; Allocate space for DeviceID (VARSTRING structure) on stack
	mov eax,28			; Guess at initial size
resize_lineGetID:
	sub esp,eax			; Allocate this space on stack
	mov edi,eax			; Save the size
	mov dword ptr [esp],eax		; Set dwTotalSize

	; ** Get the Wave OUT ID of the call **
	mov eax,esp
	pushd offset szWaveOut		; lpszDeviceClass
	pushd eax			; lpDeviceID
	pushd LINECALLSELECT_CALL 	; dwSelect
	pushd hCall			; hCall
	xor eax,eax			;
	pushd eax			; dwAddressID
	pushd hLine			; hLine
	call _lineGetID@24		;

	or eax,eax			;
	jz lineGetID_ok			;
	add esp,edi			; Deallocate space on stack
	jmp lineGetID_fail		;
lineGetID_ok:				;

	mov eax, dword ptr [esp+4]	; Get dwNeededSize
	cmp eax,edi			; Did we allocate enough memory already ?
	jle lineGetID_memOK		; Jump if so
	add esp,edi			; Deallocate existing space
	cmp eax,MaxAlloc		; Check this is sensible
	ja lineGetID_fail		; Jump if not
	cmp edi,28			; Was this the first iteration ?
	jne lineGetID_fail		; Jump if not
	add eax,3			; Round up required size to nearest DWORD
	and eax,0fffffffch		; (this preserves stack alignment)
	jmp resize_lineGetID		; Try again
lineGetID_memOK:			;

	mov esi, dword ptr[esp+20]	; dwStringOffset
	add esi,esp			; Point to Device ID
	lodsd				; Read device ID into EAX
	add esp,edi			; Deallocate space on stack
	;EAX= uDeviceID

	; ** Open WAV Out device for line **
	; Note: Could do a WAVE_FORMAT_QUERY here, but why bother ?
	xor ecx,ecx			;
	pushd CALLBACK_WINDOW		; fdwOpen
	pushd ecx			; dwCallbackInstance
	pushd hWnd			; dwCallback
	pushd offset WaveFormat		; pwfx
	pushd eax			; uDeviceID
	pushd offset hWavOutDev		; phwo
	call _waveOutOpen@24		;
        or eax,eax			;
	jnz waveOutOpen_fail		;

	; ** Prepare Wave Out Header **
	pushd 32			; cbwh = SizeOf(WaveOutHdr)
	pushd offset WaveOutHdr		; pwh
	pushd hWavOutDev		; hwo
	call _waveOutPrepareHeader@12	;
        or eax,eax			;
	jnz waveOutPrepareHeader_fail	;

	cmp bAbortDial,0		; Was a manual abort received ?
	jnz waveOutWrite_fail		; Jump if so

	; We can now speak the message (i.e. do a waveOutWrite()).
	; It is true that the phone is almost certainly still ringing 
	; at this stage, however the UniModem driver has no way of 
	; differentiating ring tone from voice and we would have to 
	; try to determine this in software (extremely difficult). 
	; Therefore it is better to start speaking at once. 



;;	pushd offset temp_buffer 	; LPDWORD pdwVolume   
;;	pushd hWavOutDev		; HWAVEOUT hwo
;;	call _waveOutGetVolume@8	;
;;	mov eax, dword ptr [temp_buffer]
;;	call debugeax
;;
;;	pushd 0FFFFFFFFH 		; DWORD dwVolume 
;;	pushd hWavOutDev		; HWAVEOUT hwo
;;	call _waveOutSetVolume@8	;

	; *** Repeat Message the specified number of times ***
	mov eax,dwRepeats		; Total times to repeat
Dial_RepeatMessage:			;
	mov dwRepeatsRemaining,eax	; Repetitions Remaining

	; ** Start Speaking Message **
	pushd 32			; cbwh
	pushd offset WaveOutHdr		; pwh
	pushd hWavOutDev		; hwo
	call _waveOutWrite@12		;
        or eax,eax			;
	jnz waveOutWrite_fail		;

	push offset szSpeaking		; "Speaking Message"
	call DebugMsg			;

	mov bDialStatus,70		; Waiting for WOM_DONE
	cmp bAbortDial,0		; Was a manual abort received ?
	jnz Dial_ManualStop_during_speak; Jump if so
	ret				;
	; ****** Wait for WOM_DONE ******
	;------------------------------------------------------------

	; ****** Got LINECALLSTATE_DISCONNECTED While speaking message ******
Dial_ManualStop_during_speak:		;
Dial_disconnected_during_speak:		;
	mov bDialStatus,80		;

	; ** Stop speaking **
	pushd hWavOutDev		; hwo
	call _waveOutReset@4		;
	jmp Dial_stop_speaking		;
	; It is possible for the listener to hang up while we are speaking the
	; message.  If we have configured for a large number of repeats, this
	; is in fact the most likely scenario.

	; ****** Got WOM_DONE (i.e. Finished speaking message) ******
dial_wom_DONE:

	; *** Check if we need to repeat ***
	mov eax,dwRepeatsRemaining	; How many times to repeat were remaining ?
	dec eax				; One fewer now
	jg Dial_RepeatMessage		; Jump if a positive number remaining

Dial_stop_speaking:			;
	mov bDialStatus,80		;
	push offset szFinishedSpeaking	; "Finished Speaking Message"
	call DebugMsg			;
dial_hangup_during_wait:		;
waveOutWrite_fail:			;

	; ** Unprepare Wave Header **
	pushd 32			; cbwh = SizeOf(WaveOutHdr)
	pushd offset WaveOutHdr		; pwh
	pushd hWavOutDev		; hwo
	call _waveOutUnprepareHeader@12	;

waveOutPrepareHeader_fail:		;

	; ** Close WAV Out device **
	pushd hWavOutDev		; hwo
	call _waveOutClose@4		;

waveOutOpen_fail:			;
lineGetID_fail:				;

	; ** Hang Up **
	pushd eax			; dwSize
	pushd eax			; lpsUserUserInfo
	pushd hCall			; hCall
	call _lineDrop@12
	cmp eax,0			;
	jl HangUpFail			;
	mov idTAPIAssynch,eax		; Save line request identifier

	;** Wait for line to hang up **

	mov bDialStatus,90		; Waiting for second LINE_REPLY to hangup request
	ret				;
	; ****** Wait for LINE_REPLY ******
	;------------------------------------------------------------
	; ****** Got LINECALLSTATE_DISCONNECTED During Dial ******
Dial_disconnected_during_dial:		;


	; ****** Got LINE_REPLY ******
Dial_line_reply_hangup:			;
HangUpFail:				;
MakeCallFail:				;
MakeCallAbort:				;

	mov bDialStatus,100		;

	; ** Close Line **
	pushd hLine			; hLine
	call _lineClose@4		;

LineOpenFail:				;

	; ***** **** Finished Delivering Message **** *****

	; ** Update INI File **
	; Record time and attempts in INI file.  These were set on entry 
	; to Dial:, but we don't write them until after the attempt has 
	; finished - this ensures we repeat the attempt if the computer 
	; goes down while dialling.  
	call record_new_starttime	;

	xor eax,eax			;
	mov bDialStatus,al		; Finished dialling
	cmp Attempts2Go,eax		; Have we used up our attempts ?
	jg dial_attempts_remaining	;

	; **** Finished Dialling all attempts ****
	call StopCounting		; Finished Counting

	; **** Done ****
	mov edi, offset szMainText	;
	mov eax,"enoD"			;
	stosd				;
	xor eax,eax			;
	stosb				;
	call update_window		; Update screen text

	push offset offset szMainText	;
	call DebugMsg			;

	; ** Reset Dial Timer **
	mov eax,ICountdownTime		;
	mov Sec2Go, eax			;

	; ****** Execute the specified program (if any) ******
	mov esi, offset szRunFileName	; Get file name to run (if any)
	mov al,byte ptr [esi]		; Is there something there ?
	or al,al			;
	jz no_file_to_run		; Jump if not

	push esi			;
	call DebugMsg			; Debug we are running the file

	xor eax,eax			;
	sub esp,80			; Allocate space for PROCESS_INFORMATION and STARTUPINFO on stack
	mov edi,esp			; Zero all of StartupInfo structure...
	mov ecx,17			;
	rep stosd			;
	pushd 68			; ... except for StartupInfo.cb = 68 (size of structure)
	mov ebx,esp			; Point to StartupInfo
	lea edx,dword ptr [esp+68]	; and to ProcessInformation
	pushd edx			; lpProcessInformation (16 bytes on stack)
	pushd ebx			; lpStartupInfo (68 bytes on stack)
	pushd eax			; lpCurrentDirectory
	pushd eax			; lpEnvironment
	pushd eax			; dwCreationFlags
	pushd eax			; bInheritHandles
	pushd eax			; lpThreadAttributes
	pushd eax			; lpProcessAttributes
	pushd esi			; lpCommandLine
	pushd eax			; lpApplicationName
	call _CreateProcessA@40		;
	add esp,84			; Deallocate space from stack
no_file_to_run:				;

	; ** Auto Stop **
	cmp bAutoStop,0			; Should we now exit ?
	jz no_auto_exit			; Jump if not
	pushd eax			;
	pushd eax			;
	pushd WM_CLOSE			;
	pushd hWnd			;
	call _SendMessageA@16		;
no_auto_exit:				;
dial_attempts_remaining:
	mov bAbortDial,0		;
	mov bDialStatus,0		; Not Dialling any more
	ret				;
	; ****** Dialling Complete ******
	;------------------------------------------------------------


;****************************************************************************
; Name: StoTime
; Purpose: Converts a string of the format "ddd hh:mm:ss" at DS:ESI
; into an equivalent time in seconds.
; Acceptable formats:  "ddd hh:mm:ss", "hh:mm:ss", "ddd hh:mm", "hh:mm"
; Returns:
; If OK, Carry reset and seconds in EAX (0 to 8726399)
; If Error, Carry set and error code in EAX as follows:
; -1 bad format, -2 days bad, -3 hours bad, -4 minutes bad, -5 seconds bad
;****************************************************************************
StoTime PROC
	call atoi			; Extract Days
	cmp al," "-"0"			; Is separator " " ?
	je sep1ok			; Jump if so (OK)
	cmp al,":"-"0"			; Is separator ":" ?
	jne time_string_bad		; If not, jump (error)
	xor eax,eax			; No days specified
	push eax				;
	jmp do_hours			; Treat this number as hours
sep1ok:					;
	cmp ebx,100			; 0-100 days ?
	jbe days_ok			;
	mov eax,-2			; Error code for bad days
	stc				;
	ret				;
days_ok:				;
	mov eax,24			;
	mul ebx				; Days -> Hours
	push eax			; Save hours
	call atoi			; Extract Hours
	cmp al,":"-"0"			; Is separator ":" ?
	jne time_string_bad2		; If not, jump (error)
do_hours:				;
	cmp ebx,23			; 0-23 hours ?
	jbe hrs_ok			; Jump if OK
	pop eax				; Clean stack
	mov eax,-3			; Error code for bad hours
	stc				;
	ret				;
hrs_ok:					;
	pop eax				; Get hours equivalent of days
	add eax,ebx			; Add hours
	mov ebx,60			;
	mul ebx				; Hours -> Min
	push eax			; Save hours
	xor ebx,ebx			;
	call atoi			; Extract Minutes
	cmp ebx,59			; 0-59 Min ?
	jbe mins_ok			; Jump if OK
	pop eax				; Clean stack
	mov eax,-4			; Error code for bad minutes
	stc				;
	ret				;
mins_ok:				;
	pop eax				; Get minutes equivalent of days+hrs
	add eax,ebx			; Add minutes
	mov ebx,60			;
	mul ebx				; Min -> Sec
	push eax			; Save seconds

	mov al,[esi-1]			; Get separator
	cmp al, ":"			; Is separator ":" ?
	je sep2OK			; Jump if so
	cmp al,0			; Is this a null ?
	jne time_string_bad3		; If not, jump (error)
	; Null, so no seconds field
	pop eax				; Get seconds
	clc				;
	ret				;
sep2OK:					;
	xor ebx,ebx			;
	call atoi			; Extract Seconds
;;	cmp al,-"0"			; Is this a null ?
;;	jne time_string_bad3		; If not, jump (error)
	cmp ebx,59			; 0-59 Sec ?
	jbe secs_ok			; Jump if OK
	pop eax				; Clean stack
	mov eax,-5			; Error code for bad seconds
	stc				;
	ret				;
secs_ok:				;
	pop eax				; Get seconds
	add eax,ebx			; Add seconds
	; Seconds should be OK, but cross-check number to be sure
	cmp eax,8726399			; Is it in the range 0-8726399 ?
	ja time_string_xcheck_bad	; Jump if not
	clc				;
	ret				;
time_string_xcheck_bad:
	mov eax,-6			;
	stc				;
	ret				;

time_string_bad3:			;
	pop eax				; Clean stack
time_string_bad2:			;
	pop eax				; Clean stack
time_string_bad:			;
	mov eax,-1			;
	stc				;
	ret				;

StoTime ENDP


;****************************************************************************
; Name: TimetoS
; Purpose: Converts a time in seconds in EAX to a string of the format
; "ddd hh:mm:ss" at DS:temp_buffer
; Returns: String in temp_buffer
;****************************************************************************
TimetoS PROC
	push ebx			;
	xor edx,edx			;
	mov ebx,60			;
	div ebx				;
	push edx			; Save seconds
	xor edx,edx			;
	div ebx				;
	push edx			; Save minutes
	xor edx,edx			;
	mov ebx,24			;
	div ebx				;
	push edx			; Save hours
	; Days in EAX
	mov edi,offset temp_buffer	; Where to write string
	call itoa			; Convert days to ASCIIZ
	mov al," "			;
	stosb				;
	pop eax				; Restore hours
	call ito2a			; Convert hours to ASCIIZ
	mov al,":"			;
	stosb				;
	pop eax				; Restore Minutes
	call ito2a			; Convert minutes to ASCIIZ
	mov al,":"			;
	stosb				;
	pop eax				; Restore Seconds
	call ito2a			; Convert seconds to ASCIIZ
	xor al,al			;
	stosb				; Null ternimator
	pop ebx				;
	ret				;
TimetoS ENDP


;;;****************************************************************************
;;; Name: DebugEAX
;;; Purpose: Temporary test procedure to print EAX
;;;****************************************************************************
;;
;;DebugEAX PROC
;;	pushad
;;	mov edi, offset temp_buffer
;;	push edi
;;	call itoa
;;	call DebugMsg
;;	popad
;;	ret
;;DebugEAX ENDP

;****************************************************************************
; Name: DebugMsg
; Purpose: Given an ASCIIZ debug string (pointer on stack), prints this with
;   a time-stamp to the debug file.
; Preserves: All registers
; Returns: nothing
;****************************************************************************
DebugMsg PROC
	cmp bDebugToFile,0		;
	jz noDebugMsg			;

	pusha				; Save all except segment registers
	mov esi, [esp+36]		; Get pointer to debug string in ESI
	mov edi,offset debug_buffer	;

 	; ** Get date and time **
	pushd offset wYear		; SYSTEMTIME structure
	call _GetLocalTime@4		; Use local time here for the benefit of the user

	; ** current date **
	mov ax,wYear			; Year
	cwd				;
	call itoa			; write it
	mov al,"/"			;
	stosb				;
	mov ax,wMonth			; Month
	call ito2a			;
	mov al,"/"			;
	stosb				;
	mov ax,wDay			; Day
	call ito2a			; write it
	mov al," "			;
	stosb				;
	mov ax,wHour			; Hour
	call ito2a			; write hours
	mov al,":"			;
	stosb				;
	mov ax,wMinute			; Minutes
	call ito2a			; write minutes
	mov al,":"			;
	stosb				;
	mov ax,wSecond			; Get seconds
	call ito2a			; write seconds
	mov al,"."			;
	stosb				;
	mov ax,wMilliseconds		; Get milliseconds
	xor edx,edx			;
	mov ebx,10			;
	div bx				; Get 100ths second
	call ito2a			; write seconds
	mov al," "			;
	stosb				;

	; ** append message **
debug_cat_next:				;
	lodsb				; Read from source
	stosb				; Write to destination
	or al,al			; Was it the null terminator ?
	jnz debug_cat_next		; If not, jump

	; **** Open Debug Logging File ****
	xor eax,eax			;
	pushd eax			; hTemplateFile
	pushd FILE_ATTRIBUTE_NORMAL	; dwFlagsAndAttributes
	pushd OPEN_ALWAYS		; dwCreationDisposition
	pushd eax			; lpSecurityAttributes
	inc eax				;
	pushd eax			; dwShareMode =FILE_SHARE_READ=1
	pushd GENERIC_WRITE		; dwDesiredAccess
	pushd offset szLogFileName	; lpFileName
	call _CreateFileA@28		;
	cmp eax,-1			;
	je DebugMsg_CreateFile_fail	;
	mov ebx,eax			; Handle to file

	; *** Go to end of file ***
	xor eax,eax			;
	pushd FILE_END			; dwMoveMethod
	pushd eax			; lpDistanceToMoveHigh
	pushd eax			; lDistanceToMove
	pushd ebx			; hFile
	call _SetFilePointer@16		;
	cmp eax,-1			;
	je DebugMsg_SetFilePointer_fail	;

	; *** How long is string to write ? ***
	mov esi, offset debug_buffer	;
	xor ecx,ecx			;
DebugMsg_strlen:			;
	lodsb				;
	or al,al			;
	loopnz DebugMsg_strlen		;
	mov word ptr [esi-1],0a0dh	;
	neg ecx				;
	inc ecx				;

	; *** Write to file ***
	xor eax,eax			;
	pushd eax			; lpOverlapped
	pushd offset dwBytesWritten	; lpNumberOfBytesWritten
	pushd ecx			; nNumberOfBytesToWrite
	pushd offset debug_buffer	; lpBuffer
	pushd ebx			; hFile
	call _WriteFile@20		;
	or eax,eax			;
	jz DebugMsg_WriteFile_fail

DebugMsg_WriteFile_fail:		;
DebugMsg_SetFilePointer_fail:		;
	; **** Close File ****
	push ebx			; hObject
	call _CloseHandle@4		;

DebugMsg_CreateFile_fail:

	popa				;
noDebugMsg:				;
	ret 4				;
DebugMsg ENDP


;****************************************************************************
; Name: itoa
; Purpose: Converts a word in EAX to an ASCIIZ string.
; Preserves: All except EAX, ECX, EDI
; Returns: Writes result to EDI
;   Length (excluding null-terminator) to ECX, ES:EDI points to null terminator
;****************************************************************************
itoa PROC
	pushd edx			;

	xor ecx,ecx			;
	pushd ecx			; Store null terminator
	mov cl,10			; Base is 10
next_i_digit:				;
	xor edx,edx			;
	div ecx				;
	add dl,"0"			; Convert digit to ASCII
	push edx				; Save digit
	or eax,eax			; Have we conveted entire number ?
	jnz next_i_digit		; If not, process next digit

	; Number is now on stack, so drag it off
	xor ecx,ecx			; Count = 0
pop_next_digit:				;
	inc ecx				; Increment Count
	pop eax				; Get digit
	stosb				; Store it in string
	or eax,eax			; Is this the null terminator
	jnz pop_next_digit		; If not, move next digit
	dec ecx				; Don't count null terminator
	dec edi				; Point to null-terminator

	pop edx				;
	ret				;
itoa ENDP

;****************************************************************************
; Name: ito2a
; Purpose: Similar to itoa, only converts AL to a two-digit, zero-padded
;  number and writes to [EDI] with no terminator
; Assumes: 0<=AL<100
; Preserves: all except EAX and EDI
;****************************************************************************

ito2a PROC
	aam				; AH = AL/10, AL = AL MOD 10
	xchg al,ah			; Low digit in AH
	add ax,'0'*100H+'0'		; Convert digits to ASCII
	stosw				; Append both digits to output string
	ret
ito2a ENDP


;****************************************************************************
; Name: atoi
; Purpose: Converts an unsigned decimal ASCII string at DS:[ESI] to a number
; Preserves: all but EAX, EBX & ESI
; Returns: Number is returned in EBX, ESI points to first char after the
;	terminator.  Terminator character-"0" is in AL
; Note: Skips white space before number.  Number is terminated by any
;	non-digit character.  If the first (non-whitespace) character is
;	not a digit, EBX=0.
;****************************************************************************
atoi PROC
	push ecx			;
	push edx			;
	xor eax,eax
	; Skip whitespace
	mov ecx,1000			; Insurance against an infinite loop
atoi_white:				;
	lodsb				; Get first character
	call iswhite			; Is it white space ?
	loopz atoi_white		; If so, jump

	; Convert number (first char is in AL)
	mov ecx,10			; Base 10
	xor ebx,ebx			; Zero number to start with
next_digit:				;
	sub al,"0"			; If not, convert to number
	cmp al,9			; In range ?
	ja end_number			; If not, jump
	xchg ebx,eax			; Number in EAX, digit in EBX
	mul ecx				; Multiply number by 10
	add ebx,eax			; Add digit and number
	lodsb				; Get next digit
	jmp next_digit			;
end_number:				;

	pop edx				;
	pop ecx				;
	ret				;
atoi ENDP

;****************************************************************************
; Name: iswhite
; Purpose: Tests if byte in AL is a white space character
; Returns: ZR if so, NZ if not
;****************************************************************************
iswhite PROC
	cmp al," "			; Space
	je end_white			;
	cmp al,9			; TAB
	je end_white			;
	cmp al,13			; CR
	je end_white			;
	cmp al,10			; LF
end_white:				;
	ret				;
iswhite ENDP

;****************************************************************************
; Name: str_cat
; Purpose: Concatenates one ASCIIZ string to another.
; On entry ESI points to start of string to concatenate from, EDI points to one
; past the null terminator on the end of the string to concatenate to.
;****************************************************************************
str_cat PROC
	dec edi				; Point to null terminator
str_cat_next:				;
	lodsb				; Read from source
	stosb				; Write to destination
	or al,al			; Was it the null terminator ?
	jnz str_cat_next		; If not, jump
	ret				;
str_cat ENDP

;****************************************************************************
; Name: instrs
; Purpose: Checks to see if one ASCIIZ string contains another ASCIIZ string
;  Comparison is case-sensitive.
; Parameters: ESI = string to search within, EDI = string to find
; Returns: NC=Found, CY=Not Found
;****************************************************************************
instrs PROC
	jmp instrs_start			;
instrs_nomatch:				;
	sub esi,ebx			; Go back to where we last matched
instrs_start:
	xor ebx,ebx			; Reset index into find string
instrs_next:
	lodsb				; Read byte from source string
	or al,al			; Reached end of string ?
	jz instrs_end			; Jump if so (not found)
	cmp al,byte ptr [edi+ebx]	; Does it match find string ?
	jne instrs_nomatch		; Jump if not
	; Byte has matched		;
	inc ebx				; Next byte next time
	cmp byte ptr [edi+ebx],0	; Reached end of find string ?
	jne instrs_next			; If not, keep looking
	; Found entire string
	clc				; Found
	ret				;

instrs_end:
	stc				; Not found
	ret				;
instrs ENDP

;****************************************************************************
; Name: instri
; Purpose: Checks to see if one ASCIIZ string contains another ASCIIZ string
;  Comparison is case-insensitive.
; Parameters: ESI = string to search within, EDI = string to find
; Returns: NC=Found, CY=Not Found
;****************************************************************************
instri PROC
	jmp instri_start			;
instri_nomatch:				;
	sub esi,ebx			; Go back to where we last matched
instri_start:
	xor ebx,ebx			; Reset index into find string
instri_next:
	lodsb				; Read byte from source string
	or al,al			; Reached end of string ?
	jz instri_end			; Jump if so (not found)
	cmp al,"a"			;
	jl instri_ucase1		;
	cmp al,"z"			;
	ja instri_ucase1		;
	sub al,020H			; Character is lower case so convert to upper
instri_ucase1:
	mov ah,byte ptr [edi+ebx]	; Get character from match string
	cmp ah,"a"			;
	jl instri_ucase2		;
	cmp ah,"z"			;
	ja instri_ucase2		;
	sub ah,020H			; Character is lower case so convert to upper
instri_ucase2:
	cmp al,ah			; Do characters match ?
	jne instri_nomatch		; Jump if not
	; Byte has matched		;
	inc ebx				; Next byte next time
	cmp byte ptr [edi+ebx],0	; Reached end of find string ?
	jne instri_next			; If not, keep looking
	; Found entire string
	clc				; Found
	ret				;

instri_end:
	stc				; Not found
	ret				;
instri ENDP


	END  _start
